@@ -34,16 +34,17 @@ extensions = ['matplotlib.sphinxext.mathmpl', 'sphinxext.math_symbol_table',
               'sphinxext.github',
               'numpydoc']
 
-exclude_patterns = ['api/api_changes/README.rst']
+exclude_patterns = ['api/api_changes/*', 'users/whats_new/*']
 
 # Use IPython's console highlighting by default
 try:
     from IPython.sphinxext import ipython_console_highlighting
 except ImportError:
-    extensions.append('matplotlib.sphinxext.ipython_console_highlighting')
+    raise ImportError(
+        "IPython must be installed to build the matplotlib docs")
 else:
-    print("Using IPython's ipython_console_highlighting directive")
     extensions.append('IPython.sphinxext.ipython_console_highlighting')
+    extensions.append('IPython.sphinxext.ipython_directive')
 
 try:
     import numpydoc
@@ -126,6 +127,7 @@ mpl_example_sections = (
     ('color', 'Color'),
     ('text_labels_and_annotations', 'Text, labels, and annotations'),
     ('ticks_and_spines', 'Ticks and spines'),
+    ('scales', 'Axis scales'),
     ('subplots_axes_and_figures', 'Subplots, axes, and figures'),
     ('style_sheets', 'Style sheets'),
     ('specialty_plots', 'Specialty plots'),
@@ -184,9 +186,12 @@ html_index = 'index.html'
 #html_sidebars = {}
 
 # Custom sidebar templates, maps page names to templates.
-html_sidebars = {'index': 'indexsidebar.html',
-                 }
-
+html_sidebars = {
+    'index': ['badgesidebar.html','donate_sidebar.html',
+              'indexsidebar.html', 'searchbox.html'],
+    '**': ['badgesidebar.html', 'localtoc.html',
+           'relations.html', 'sourcelink.html', 'searchbox.html']
+}
 
 # Additional templates that should be rendered to pages, maps page names to
 # template names.
@@ -222,9 +227,9 @@ latex_font_size = '11pt'
 # (source start file, target name, title, author, document class [howto/manual]).
 
 latex_documents = [
-  ('contents', 'Matplotlib.tex', 'Matplotlib',
-   'John Hunter, Darren Dale, Eric Firing, Michael Droettboom and the '
-   'matplotlib development team', 'manual'),
+    ('contents', 'Matplotlib.tex', 'Matplotlib',
+     'John Hunter, Darren Dale, Eric Firing, Michael Droettboom and the '
+     'matplotlib development team', 'manual'),
 ]
 
 
@@ -233,7 +238,7 @@ latex_documents = [
 latex_logo = None
 
 # Additional stuff for the LaTeX preamble.
-latex_preamble =r"""
+latex_preamble = r"""
    % In the parameters section, place a newline after the Parameters
    % header.  (This is stolen directly from Numpy's conf.py, since it
    % affects Numpy-style docstrings).
@@ -271,13 +276,60 @@ rst_epilog = """
 """ % matplotlib.__version__numpy__
 
 texinfo_documents = [
-  ("contents", 'matplotlib', 'Matplotlib Documentation',
-   'John Hunter@*Darren Dale@*Eric Firing@*Michael Droettboom@*'
-   'The matplotlib development team',
-   'Matplotlib', "Python plotting package", 'Programming',
-   1),
+    ("contents", 'matplotlib', 'Matplotlib Documentation',
+     'John Hunter@*Darren Dale@*Eric Firing@*Michael Droettboom@*'
+     'The matplotlib development team',
+     'Matplotlib', "Python plotting package", 'Programming',
+     1),
 ]
 
+try:
+    from unittest.mock import MagicMock
+except:
+    from mock import MagicMock
+
+
+class MyWX(MagicMock):
+    class Panel(object):
+        pass
+
+    class ToolBar(object):
+        pass
+
+    class Frame(object):
+        pass
+
+    VERSION_STRING = '2.8.12'
+
+
+class MyPyQt4(MagicMock):
+    class QtGui(object):
+        class QToolBar(object):
+            pass
+
+        class QDialog(object):
+            pass
+
+        class QWidget(object):
+            pass
+
+        class QMainWindow(object):
+            pass
+
+
+class MySip(MagicMock):
+    def getapi(*args):
+        return 1
+
+
+mockwxversion = MagicMock()
+mockwx = MyWX()
+mocksip = MySip()
+mockpyqt4 = MyPyQt4()
+sys.modules['wxversion'] = mockwxversion
+sys.modules['wx'] = mockwx
+sys.modules['sip'] = mocksip
+sys.modules['PyQt4'] = mockpyqt4
 
 ################# numpydoc config ####################
 numpydoc_show_class_members = False
