@@ -1,7 +1,14 @@
 import matplotlib.pyplot as plt
 
-from demo_image import get_demo_image
-from mpl_toolkits.axes_grid.axes_rgb import RGBAxes
+from mpl_toolkits.axes_grid1.axes_rgb import RGBAxes
+
+def get_demo_image():
+    import numpy as np
+    from matplotlib.cbook import get_sample_data
+    f = get_sample_data("axes_grid/bivariate_normal.npy", asfileobj=False)
+    z = np.load(f)
+    # z is a numpy array of 15x15
+    return z, (-3,4,-4,3)
 
 def get_rgb():
     Z, extent = get_demo_image()
