@@ -3,14 +3,21 @@ Show what matplotlib colormaps look like in grayscale.
 Uses lightness L* as a proxy for grayscale value.
 '''
 
-from skimage import color
+from colormaps import cmaps
+
+#from skimage import color
+# we are using a local copy of colorconv from scikit-image to reduce dependencies.
+# You should probably use the one from scikit-image in most cases.
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib import cm
 import matplotlib as mpl
+from colorspacious import cspace_converter
 
 mpl.rcParams.update({'font.size': 14})
-mpl.rcParams['font.sans-serif'] = 'Arev Sans, Bitstream Vera Sans, Lucida Grande, Verdana, Geneva, Lucid, Helvetica, Avant Garde, sans-serif'
+mpl.rcParams['font.sans-serif'] = ('Arev Sans, Bitstream Vera Sans, '
+                                   'Lucida Grande, Verdana, Geneva, Lucid, '
+                                   'Helvetica, Avant Garde, sans-serif')
 mpl.rcParams['mathtext.fontset'] = 'custom'
 mpl.rcParams['mathtext.cal'] = 'cursive'
 mpl.rcParams['mathtext.rm'] = 'sans'
@@ -20,34 +27,19 @@ mpl.rcParams['mathtext.bf'] = 'sans:bold'
 mpl.rcParams['mathtext.sf'] = 'sans'
 mpl.rcParams['mathtext.fallback_to_cm'] = 'True'
 
-# Have colormaps separated into categories: http://matplotlib.org/examples/color/colormaps_reference.html
-
-cmaps = [('Sequential',     ['binary', 'Blues', 'BuGn', 'BuPu', 'gist_yarg',
-                             'GnBu', 'Greens', 'Greys', 'Oranges', 'OrRd',
-                             'PuBu', 'PuBuGn', 'PuRd', 'Purples', 'RdPu',
-                             'Reds', 'YlGn', 'YlGnBu', 'YlOrBr', 'YlOrRd']),
-         ('Sequential2', ['afmhot', 'autumn', 'bone', 'cool', 'copper',
-                             'gist_gray', 'gist_heat', 'gray', 'hot', 'pink',
-                             'spring', 'summer', 'winter']),
-         ('Diverging',      ['BrBG', 'bwr', 'coolwarm', 'PiYG', 'PRGn', 'PuOr',
-                             'RdBu', 'RdGy', 'RdYlBu', 'RdYlGn', 'seismic']),
-         ('Qualitative',    ['Accent', 'Dark2', 'hsv', 'Paired', 'Pastel1',
-                             'Pastel2', 'Set1', 'Set2', 'Set3', 'spectral']),
-         ('Miscellaneous',  ['gist_earth', 'gist_ncar', 'gist_rainbow',
-                             'gist_stern', 'jet', 'brg', 'CMRmap', 'cubehelix',
-                             'gnuplot', 'gnuplot2', 'ocean', 'rainbow',
-                             'terrain', 'flag', 'prism'])]
 
 # indices to step through colormap
 x = np.linspace(0.0, 1.0, 100)
 
-nrows = max(len(cmap_list) for cmap_category, cmap_list in cmaps)
+# nrows = max(len(cmap_list) for cmap_category, cmap_list in cmaps)
 gradient = np.linspace(0, 1, 256)
 gradient = np.vstack((gradient, gradient))
 
 def plot_color_gradients(cmap_category, cmap_list):
+    nrows = len(cmap_list)
     fig, axes = plt.subplots(nrows=nrows, ncols=2)
-    fig.subplots_adjust(top=0.95, bottom=0.01, left=0.2, right=0.99, wspace=0.05)
+    fig.subplots_adjust(top=0.95, bottom=0.01, left=0.2, right=0.99,
+                        wspace=0.05)
     fig.suptitle(cmap_category + ' colormaps', fontsize=14, y=1.0, x=0.6)
 
     for ax, name in zip(axes, cmap_list):
@@ -55,8 +47,8 @@ def plot_color_gradients(cmap_category, cmap_list):
         # Get rgb values for colormap
         rgb = cm.get_cmap(plt.get_cmap(name))(x)[np.newaxis,:,:3]
 
-        # Get colormap in CIE LAB. We want the L here.
-        lab = color.rgb2lab(rgb)
+        # Get colormap in CAM02-UCS colorspace. We want the lightness.
+        lab = cspace_converter("sRGB1", "CAM02-UCS")(rgb)
         L = lab[0,:,0]
         L = np.float32(np.vstack((L, L, L)))
 
@@ -71,6 +63,7 @@ def plot_color_gradients(cmap_category, cmap_list):
     for ax in axes:
         ax[0].set_axis_off()
         ax[1].set_axis_off()
+    plt.show()
 
 
 for cmap_category, cmap_list in cmaps:
