@@ -4,6 +4,7 @@ import matplotlib.pyplot as plt
 import scipy.stats as ss
 from matplotlib.animation import FuncAnimation
 
+
 class UpdateDist(object):
     def __init__(self, ax, prob=0.5):
         self.success = 0
@@ -39,9 +40,8 @@ class UpdateDist(object):
         self.line.set_data(self.x, y)
         return self.line,
 
-fig = plt.figure()
-ax = fig.add_subplot(1, 1, 1)
+fig, ax = plt.subplots()
 ud = UpdateDist(ax, prob=0.7)
 anim = FuncAnimation(fig, ud, frames=np.arange(100), init_func=ud.init,
-        interval=100, blit=True)
+                     interval=100, blit=True)
 plt.show()
