@@ -6,12 +6,15 @@ fig, ax = plt.subplots()
 line, = ax.plot(np.random.rand(10))
 ax.set_ylim(0, 1)
 
+
 def update(data):
     line.set_ydata(data)
     return line,
 
+
 def data_gen():
-    while True: yield np.random.rand(10)
+    while True:
+        yield np.random.rand(10)
 
 ani = animation.FuncAnimation(fig, update, data_gen, interval=100)
 plt.show()
