@@ -26,11 +26,11 @@ nverts = 50
 npts = 100
 
 # Make some spirals
-r = np.array(range(nverts))
-theta = np.array(range(nverts)) * (2*np.pi)/(nverts-1)
+r = np.arange(nverts)
+theta = np.linspace(0, 2*np.pi, nverts)
 xx = r * np.sin(theta)
 yy = r * np.cos(theta)
-spiral = list(zip(xx,yy))
+spiral = list(zip(xx, yy))
 
 # Make some offsets
 rs = np.random.RandomState([12345678])
@@ -39,27 +39,28 @@ yo = rs.randn(npts)
 xyo = list(zip(xo, yo))
 
 # Make a list of colors cycling through the rgbcmyk series.
-colors = [colorConverter.to_rgba(c) for c in ('r','g','b','c','y','m','k')]
+colors = [colorConverter.to_rgba(c)
+          for c in ('r', 'g', 'b', 'c', 'y', 'm', 'k')]
 
-fig, axes = plt.subplots(2,2)
-((ax1, ax2), (ax3, ax4)) = axes # unpack the axes
+fig, axes = plt.subplots(2, 2)
+((ax1, ax2), (ax3, ax4)) = axes  # unpack the axes
 
 
 col = collections.LineCollection([spiral], offsets=xyo,
-                                transOffset=ax1.transData)
+                                 transOffset=ax1.transData)
 trans = fig.dpi_scale_trans + transforms.Affine2D().scale(1.0/72.0)
 col.set_transform(trans)  # the points to pixels transform
-    # Note: the first argument to the collection initializer
-    # must be a list of sequences of x,y tuples; we have only
-    # one sequence, but we still have to put it in a list.
+# Note: the first argument to the collection initializer
+# must be a list of sequences of x,y tuples; we have only
+# one sequence, but we still have to put it in a list.
 ax1.add_collection(col, autolim=True)
-    # autolim=True enables autoscaling.  For collections with
-    # offsets like this, it is neither efficient nor accurate,
-    # but it is good enough to generate a plot that you can use
-    # as a starting point.  If you know beforehand the range of
-    # x and y that you want to show, it is better to set them
-    # explicitly, leave out the autolim kwarg (or set it to False),
-    # and omit the 'ax1.autoscale_view()' call below.
+# autolim=True enables autoscaling.  For collections with
+# offsets like this, it is neither efficient nor accurate,
+# but it is good enough to generate a plot that you can use
+# as a starting point.  If you know beforehand the range of
+# x and y that you want to show, it is better to set them
+# explicitly, leave out the autolim kwarg (or set it to False),
+# and omit the 'ax1.autoscale_view()' call below.
 
 # Make a transform for the line segments such that their size is
 # given in points:
@@ -71,7 +72,7 @@ ax1.set_title('LineCollection using offsets')
 
 # The same data as above, but fill the curves.
 col = collections.PolyCollection([spiral], offsets=xyo,
-                                transOffset=ax2.transData)
+                                 transOffset=ax2.transData)
 trans = transforms.Affine2D().scale(fig.dpi/72.0)
 col.set_transform(trans)  # the points to pixels transform
 ax2.add_collection(col, autolim=True)
@@ -84,9 +85,9 @@ ax2.set_title('PolyCollection using offsets')
 # 7-sided regular polygons
 
 col = collections.RegularPolyCollection(7,
-                                        sizes = np.fabs(xx)*10.0, offsets=xyo,
+                                        sizes=np.fabs(xx) * 10.0, offsets=xyo,
                                         transOffset=ax3.transData)
-trans = transforms.Affine2D().scale(fig.dpi/72.0)
+trans = transforms.Affine2D().scale(fig.dpi / 72.0)
 col.set_transform(trans)  # the points to pixels transform
 ax3.add_collection(col, autolim=True)
 col.set_color(colors)
@@ -104,7 +105,7 @@ offs = (0.1, 0.0)
 
 yy = np.linspace(0, 2*np.pi, nverts)
 ym = np.amax(yy)
-xx = (0.2 + (ym-yy)/ym)**2 * np.cos(yy-0.4) * 0.5
+xx = (0.2 + (ym - yy)/ym)**2 * np.cos(yy - 0.4)*0.5
 segs = []
 for i in range(ncurves):
     xxx = xx + 0.02*rs.randn(nverts)
@@ -123,5 +124,3 @@ ax4.set_ylim(ax4.get_ylim()[::-1])
 
 
 plt.show()
-
-
