@@ -17,8 +17,8 @@ import matplotlib.pyplot as plt
 import matplotlib.dates as mdates
 import matplotlib.cbook as cbook
 
-years    = mdates.YearLocator()   # every year
-months   = mdates.MonthLocator()  # every month
+years = mdates.YearLocator()   # every year
+months = mdates.MonthLocator()  # every month
 yearsFmt = mdates.DateFormatter('%Y')
 
 # load a numpy record array from yahoo csv data with fields date,
@@ -26,7 +26,14 @@ yearsFmt = mdates.DateFormatter('%Y')
 # The record array stores python datetime.date as an object array in
 # the date column
 datafile = cbook.get_sample_data('goog.npy')
-r = np.load(datafile).view(np.recarray)
+try:
+    # Python3 cannot load python2 .npy files with datetime(object) arrays
+    # unless the encoding is set to bytes. Hovever this option was
+    # not added until numpy 1.10 so this example will only work with
+    # python 2 or with numpy 1.10 and later.
+    r = np.load(datafile, encoding='bytes').view(np.recarray)
+except TypeError:
+    r = np.load(datafile).view(np.recarray)
 
 fig, ax = plt.subplots()
 ax.plot(r.date, r.adj_close)
@@ -38,11 +45,13 @@ ax.xaxis.set_major_formatter(yearsFmt)
 ax.xaxis.set_minor_locator(months)
 
 datemin = datetime.date(r.date.min().year, 1, 1)
-datemax = datetime.date(r.date.max().year+1, 1, 1)
+datemax = datetime.date(r.date.max().year + 1, 1, 1)
 ax.set_xlim(datemin, datemax)
 
+
 # format the coords message box
-def price(x): return '$%1.2f'%x
+def price(x):
+    return '$%1.2f' % x
 ax.format_xdata = mdates.DateFormatter('%Y-%m-%d')
 ax.format_ydata = price
 ax.grid(True)
