@@ -11,25 +11,25 @@ X = numpy.random.rand(100, 1000)
 xs = numpy.mean(X, axis=1)
 ys = numpy.std(X, axis=1)
 
-fig = plt.figure()
-ax = fig.add_subplot(111)
+fig, ax = plt.subplots()
 ax.set_title('click on point to plot time series')
 line, = ax.plot(xs, ys, 'o', picker=5)  # 5 points tolerance
 
 
 def onpick(event):
 
-    if event.artist!=line: return True
+    if event.artist != line:
+        return True
 
     N = len(event.ind)
-    if not N: return True
-
+    if not N:
+        return True
 
     figi = plt.figure()
     for subplotnum, dataind in enumerate(event.ind):
-        ax = figi.add_subplot(N,1,subplotnum+1)
+        ax = figi.add_subplot(N, 1, subplotnum + 1)
         ax.plot(X[dataind])
-        ax.text(0.05, 0.9, 'mu=%1.3f\nsigma=%1.3f'%(xs[dataind], ys[dataind]),
+        ax.text(0.05, 0.9, 'mu=%1.3f\nsigma=%1.3f' % (xs[dataind], ys[dataind]),
                 transform=ax.transAxes, va='top')
         ax.set_ylim(-0.5, 1.5)
     figi.show()
@@ -38,8 +38,3 @@ def onpick(event):
 fig.canvas.mpl_connect('pick_event', onpick)
 
 plt.show()
-
-
-
-
-
