@@ -3,12 +3,12 @@ import matplotlib.pyplot as plt
 import numpy as np
 
 fig = plt.figure()
-ax = Axes3D(fig)
+ax = fig.add_subplot(111, projection='3d')
 x, y = np.random.rand(2, 100) * 4
 hist, xedges, yedges = np.histogram2d(x, y, bins=4)
 
 elements = (len(xedges) - 1) * (len(yedges) - 1)
-xpos, ypos = np.meshgrid(xedges[:-1]+0.25, yedges[:-1]+0.25)
+xpos, ypos = np.meshgrid(xedges[:-1] + 0.25, yedges[:-1] + 0.25)
 
 xpos = xpos.flatten()
 ypos = ypos.flatten()
@@ -20,4 +20,3 @@ dz = hist.flatten()
 ax.bar3d(xpos, ypos, zpos, dx, dy, dz, color='b', zsort='average')
 
 plt.show()
-
