@@ -2,11 +2,12 @@ import numpy as np
 from mpl_toolkits.mplot3d import Axes3D
 import matplotlib.pyplot as plt
 
+
 def randrange(n, vmin, vmax):
-    return (vmax-vmin)*np.random.rand(n) + vmin
+    return (vmax - vmin)*np.random.rand(n) + vmin
 
 fig = plt.figure()
-ax = Axes3D(fig)
+ax = fig.add_subplot(111, projection='3d')
 n = 100
 for c, m, zl, zh in [('r', 'o', -50, -25), ('b', '^', -30, -5)]:
     xs = randrange(n, 23, 32)
@@ -19,4 +20,3 @@ ax.set_ylabel('Y Label')
 ax.set_zlabel('Z Label')
 
 plt.show()
-
