@@ -1,45 +1,49 @@
-#!/usr/bin/env python
 # -*- noplot -*-
 
 """
-
 This example shows how to use matplotlib to provide a data cursor.  It
 uses matplotlib to draw the cursor and may be a slow since this
 requires redrawing the figure with every mouse move.
 
 Faster cursoring is possible using native GUI drawing, as in
-wxcursor_demo.py
+wxcursor_demo.py.
+
+Also, mpldatacursor can be used to achieve a similar effect. See webpage:
+https://github.com/joferkington/mpldatacursor
 """
 from __future__ import print_function
-from pylab import *
+import matplotlib.pyplot as plt
+import numpy as np
 
 
-class Cursor:
+class Cursor(object):
     def __init__(self, ax):
         self.ax = ax
         self.lx = ax.axhline(color='k')  # the horiz line
         self.ly = ax.axvline(color='k')  # the vert line
 
         # text location in axes coords
-        self.txt = ax.text( 0.7, 0.9, '', transform=ax.transAxes)
+        self.txt = ax.text(0.7, 0.9, '', transform=ax.transAxes)
 
     def mouse_move(self, event):
-        if not event.inaxes: return
+        if not event.inaxes:
+            return
 
         x, y = event.xdata, event.ydata
         # update the line positions
-        self.lx.set_ydata(y )
-        self.ly.set_xdata(x )
+        self.lx.set_ydata(y)
+        self.ly.set_xdata(x)
 
-        self.txt.set_text( 'x=%1.2f, y=%1.2f'%(x,y) )
-        draw()
+        self.txt.set_text('x=%1.2f, y=%1.2f' % (x, y))
+        plt.draw()
 
 
-class SnaptoCursor:
+class SnaptoCursor(object):
     """
     Like Cursor but the crosshair snaps to the nearest x,y point
     For simplicity, I'm assuming x is sorted
     """
+
     def __init__(self, ax, x, y):
         self.ax = ax
         self.lx = ax.axhline(color='k')  # the horiz line
@@ -47,34 +51,34 @@ class SnaptoCursor:
         self.x = x
         self.y = y
         # text location in axes coords
-        self.txt = ax.text( 0.7, 0.9, '', transform=ax.transAxes)
+        self.txt = ax.text(0.7, 0.9, '', transform=ax.transAxes)
 
     def mouse_move(self, event):
 
-        if not event.inaxes: return
+        if not event.inaxes:
+            return
 
         x, y = event.xdata, event.ydata
 
-        indx = searchsorted(self.x, [x])[0]
+        indx = np.searchsorted(self.x, [x])[0]
         x = self.x[indx]
         y = self.y[indx]
         # update the line positions
-        self.lx.set_ydata(y )
-        self.ly.set_xdata(x )
+        self.lx.set_ydata(y)
+        self.ly.set_xdata(x)
 
-        self.txt.set_text( 'x=%1.2f, y=%1.2f'%(x,y) )
-        print ('x=%1.2f, y=%1.2f'%(x,y))
-        draw()
+        self.txt.set_text('x=%1.2f, y=%1.2f' % (x, y))
+        print('x=%1.2f, y=%1.2f' % (x, y))
+        plt.draw()
 
-t = arange(0.0, 1.0, 0.01)
-s = sin(2*2*pi*t)
-fig, ax = subplots()
+t = np.arange(0.0, 1.0, 0.01)
+s = np.sin(2*2*np.pi*t)
+fig, ax = plt.subplots()
 
-cursor = Cursor(ax)
-#cursor = SnaptoCursor(ax, t, s)
-connect('motion_notify_event', cursor.mouse_move)
+#cursor = Cursor(ax)
+cursor = SnaptoCursor(ax, t, s)
+plt.connect('motion_notify_event', cursor.mouse_move)
 
 ax.plot(t, s, 'o')
-axis([0,1,-1,1])
-show()
-
+plt.axis([0, 1, -1, 1])
+plt.show()
