@@ -1,7 +1,7 @@
 
 """
 When plotting daily data, a frequent request is to plot the data
-ignoring skips, eg no extra spaces for weekends.  This is particularly
+ignoring skips, e.g., no extra spaces for weekends.  This is particularly
 common in financial time series, when you may have data for M-F and
 not Sat, Sun and you don't want gaps in the x axis.  The approach is
 to simply use the integer index for the xdata and a custom tick
@@ -16,9 +16,10 @@ import matplotlib.cbook as cbook
 from matplotlib.ticker import Formatter
 
 datafile = cbook.get_sample_data('msft.csv', asfileobj=False)
-print ('loading %s' % datafile)
+print('loading %s' % datafile)
 r = csv2rec(datafile)[-40:]
 
+
 class MyFormatter(Formatter):
     def __init__(self, dates, fmt='%Y-%m-%d'):
         self.dates = dates
@@ -27,7 +28,8 @@ class MyFormatter(Formatter):
     def __call__(self, x, pos=0):
         'Return the label for time x at position pos'
         ind = int(round(x))
-        if ind>=len(self.dates) or ind<0: return ''
+        if ind >= len(self.dates) or ind < 0:
+            return ''
 
         return self.dates[ind].strftime(self.fmt)
 
