@@ -2,17 +2,17 @@ import matplotlib.patches as mpatch
 import matplotlib.pyplot as plt
 
 styles = mpatch.BoxStyle.get_styles()
+spacing = 1.2
 
-figheight = (len(styles)+.5)
+figheight = (spacing * len(styles) + .5)
 fig1 = plt.figure(1, (4/1.5, figheight/1.5))
 fontsize = 0.3 * 72
 
-for i, (stylename, styleclass) in enumerate(styles.items()):
-    fig1.text(0.5, (float(len(styles)) - 0.5 - i)/figheight, stylename,
+for i, stylename in enumerate(sorted(styles.keys())):
+    fig1.text(0.5, (spacing * (float(len(styles)) - i) - 0.5)/figheight, stylename,
               ha="center",
               size=fontsize,
               transform=fig1.transFigure,
               bbox=dict(boxstyle=stylename, fc="w", ec="k"))
 plt.draw()
 plt.show()
-
