@@ -9,11 +9,11 @@ import numpy as np
 
 files = []
 
-fig, ax = plt.subplots(figsize=(5,5))
+fig, ax = plt.subplots(figsize=(5, 5))
 for i in range(50):  # 50 frames
     plt.cla()
-    plt.imshow(np.random.rand(5,5), interpolation='nearest')
-    fname = '_tmp%03d.png'%i
+    plt.imshow(np.random.rand(5, 5), interpolation='nearest')
+    fname = '_tmp%03d.png' % i
     print('Saving frame', fname)
     plt.savefig(fname)
     files.append(fname)
@@ -23,4 +23,5 @@ os.system("mencoder 'mf://_tmp*.png' -mf type=png:fps=10 -ovc lavc -lavcopts vco
 #os.system("convert _tmp*.png animation.mng")
 
 # cleanup
-for fname in files: os.remove(fname)
+for fname in files:
+    os.remove(fname)
