@@ -2,8 +2,9 @@
 # are now supported in pdf usetex.
 
 import matplotlib
+import matplotlib.pyplot as plt
 matplotlib.rc('text', usetex=True)
-import pylab
+
 
 def setfont(font):
     return r'\font\a %s at 14pt\a ' % font
@@ -13,10 +14,10 @@ for y, font, text in zip(range(5),
                          ['Nimbus Roman No9 L ' + x for x in
                           ['', 'Italics (real italics for comparison)',
                            '(slanted)', '(condensed)', '(extended)']]):
-    pylab.text(0, y, setfont(font) + text)
+    plt.text(0, y, setfont(font) + text)
 
-pylab.ylim(-1, 5)
-pylab.xlim(-0.2, 0.6)
-pylab.setp(pylab.gca(), frame_on=False, xticks=(), yticks=())
-pylab.title('Usetex font effects')
-pylab.savefig('usetex_fonteffects.pdf')
+plt.ylim(-1, 5)
+plt.xlim(-0.2, 0.6)
+plt.setp(plt.gca(), frame_on=False, xticks=(), yticks=())
+plt.title('Usetex font effects')
+plt.savefig('usetex_fonteffects.pdf')
