@@ -8,7 +8,8 @@ import matplotlib.pyplot as plt
 N = 50
 x = np.random.rand(N)
 y = np.random.rand(N)
-area = np.pi * (15 * np.random.rand(N))**2 # 0 to 15 point radiuses
+colors = np.random.rand(N)
+area = np.pi * (15 * np.random.rand(N))**2  # 0 to 15 point radiuses
 
-plt.scatter(x, y, s=area, alpha=0.5)
+plt.scatter(x, y, s=area, c=colors, alpha=0.5)
 plt.show()
