@@ -10,21 +10,21 @@ from matplotlib.figure import Figure
 from numpy import arange, sin, pi
 
 # uncomment to select /GTK/GTKAgg/GTKCairo
-from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
-#from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
+#from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
+from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
 #from matplotlib.backends.backend_gtkcairo import FigureCanvasGTKCairo as FigureCanvas
 
 
 win = gtk.Window()
 win.connect("destroy", lambda x: gtk.main_quit())
-win.set_default_size(400,300)
+win.set_default_size(400, 300)
 win.set_title("Embedding in GTK")
 
-f = Figure(figsize=(5,4), dpi=100)
+f = Figure(figsize=(5, 4), dpi=100)
 a = f.add_subplot(111)
-t = arange(0.0,3.0,0.01)
+t = arange(0.0, 3.0, 0.01)
 s = sin(2*pi*t)
-a.plot(t,s)
+a.plot(t, s)
 
 canvas = FigureCanvas(f)  # a gtk.DrawingArea
 win.add(canvas)
