@@ -1,3 +1,4 @@
+from __future__ import division, print_function
 import numpy as np
 import matplotlib
 import matplotlib.colors as colors
@@ -8,8 +9,9 @@ import matplotlib.artist as artist
 import matplotlib.image as image
 
 
-class ItemProperties:
-    def __init__(self, fontsize=14, labelcolor='black', bgcolor='yellow', alpha=1.0):
+class ItemProperties(object):
+    def __init__(self, fontsize=14, labelcolor='black', bgcolor='yellow',
+                 alpha=1.0):
         self.fontsize = fontsize
         self.labelcolor = labelcolor
         self.bgcolor = bgcolor
@@ -18,11 +20,14 @@ class ItemProperties:
         self.labelcolor_rgb = colors.colorConverter.to_rgb(labelcolor)
         self.bgcolor_rgb = colors.colorConverter.to_rgb(bgcolor)
 
+
 class MenuItem(artist.Artist):
     parser = mathtext.MathTextParser("Bitmap")
     padx = 5
-    pady =5
-    def __init__(self, fig, labelstr, props=None, hoverprops=None, on_select=None):
+    pady = 5
+
+    def __init__(self, fig, labelstr, props=None, hoverprops=None,
+                 on_select=None):
         artist.Artist.__init__(self)
 
         self.set_figure(fig)
@@ -37,27 +42,26 @@ class MenuItem(artist.Artist):
         self.props = props
         self.hoverprops = hoverprops
 
-
         self.on_select = on_select
 
         x, self.depth = self.parser.to_mask(
             labelstr, fontsize=props.fontsize, dpi=fig.dpi)
 
-        if props.fontsize!=hoverprops.fontsize:
-            raise NotImplementedError('support for different font sizes not implemented')
-
+        if props.fontsize != hoverprops.fontsize:
+            raise NotImplementedError(
+                'support for different font sizes not implemented')
 
         self.labelwidth = x.shape[1]
         self.labelheight = x.shape[0]
 
         self.labelArray = np.zeros((x.shape[0], x.shape[1], 4))
-        self.labelArray[:,:,-1] = x/255.
+        self.labelArray[:, :, -1] = x/255.
 
         self.label = image.FigureImage(fig, origin='upper')
         self.label.set_array(self.labelArray)
 
         # we'll update these later
-        self.rect = patches.Rectangle((0,0), 1,1)
+        self.rect = patches.Rectangle((0, 0), 1, 1)
 
         self.set_hover_props(False)
 
@@ -72,14 +76,14 @@ class MenuItem(artist.Artist):
             self.on_select(self)
 
     def set_extent(self, x, y, w, h):
-        print x, y, w, h
+        print(x, y, w, h)
         self.rect.set_x(x)
         self.rect.set_y(y)
         self.rect.set_width(w)
         self.rect.set_height(h)
 
-        self.label.ox = x+self.padx
-        self.label.oy = y-self.depth+self.pady/2.
+        self.label.ox = x + self.padx
+        self.label.oy = y - self.depth + self.pady/2.
 
         self.rect._update_patch_transform()
         self.hover = False
@@ -95,27 +99,26 @@ class MenuItem(artist.Artist):
             props = self.props
 
         r, g, b = props.labelcolor_rgb
-        self.labelArray[:,:,0] = r
-        self.labelArray[:,:,1] = g
-        self.labelArray[:,:,2] = b
+        self.labelArray[:, :, 0] = r
+        self.labelArray[:, :, 1] = g
+        self.labelArray[:, :, 2] = b
         self.label.set_array(self.labelArray)
         self.rect.set(facecolor=props.bgcolor, alpha=props.alpha)
 
     def set_hover(self, event):
         'check the hover status of event and return true if status is changed'
-        b,junk = self.rect.contains(event)
+        b, junk = self.rect.contains(event)
 
         changed = (b != self.hover)
 
         if changed:
             self.set_hover_props(b)
 
-
         self.hover = b
         return changed
 
-class Menu:
 
+class Menu(object):
     def __init__(self, fig, menuitems):
         self.figure = fig
         fig.suppressComposite = True
@@ -126,27 +129,23 @@ class Menu:
         maxw = max([item.labelwidth for item in menuitems])
         maxh = max([item.labelheight for item in menuitems])
 
-
-        totalh = self.numitems*maxh + (self.numitems+1)*2*MenuItem.pady
-
+        totalh = self.numitems*maxh + (self.numitems + 1)*2*MenuItem.pady
 
         x0 = 100
         y0 = 400
 
         width = maxw + 2*MenuItem.padx
-        height = maxh+MenuItem.pady
+        height = maxh + MenuItem.pady
 
         for item in menuitems:
             left = x0
-            bottom = y0-maxh-MenuItem.pady
-
+            bottom = y0 - maxh - MenuItem.pady
 
             item.set_extent(left, bottom, width, height)
 
             fig.artists.append(item)
             y0 -= maxh + MenuItem.pady
 
-
         fig.canvas.mpl_connect('motion_notify_event', self.on_move)
 
     def on_move(self, event):
@@ -168,13 +167,10 @@ hoverprops = ItemProperties(labelcolor='white', bgcolor='blue',
 menuitems = []
 for label in ('open', 'close', 'save', 'save as', 'quit'):
     def on_select(item):
-        print 'you selected', item.labelstr
-    item = MenuItem(fig, label, props=props, hoverprops=hoverprops, on_select=on_select)
+        print('you selected %s' % item.labelstr)
+    item = MenuItem(fig, label, props=props, hoverprops=hoverprops,
+                    on_select=on_select)
     menuitems.append(item)
 
 menu = Menu(fig, menuitems)
 plt.show()
-
-
-
-
