@@ -1,10 +1,11 @@
+import numpy as np
+import matplotlib.pyplot as plt
 from matplotlib.widgets import MultiCursor
-from pylab import figure, show, pi, arange, sin
 
-t = arange(0.0, 2.0, 0.01)
-s1 = sin(2*pi*t)
-s2 = sin(4*pi*t)
-fig = figure()
+t = np.arange(0.0, 2.0, 0.01)
+s1 = np.sin(2*np.pi*t)
+s2 = np.sin(4*np.pi*t)
+fig = plt.figure()
 ax1 = fig.add_subplot(211)
 ax1.plot(t, s1)
 
@@ -13,4 +14,4 @@ ax2 = fig.add_subplot(212, sharex=ax1)
 ax2.plot(t, s2)
 
 multi = MultiCursor(fig.canvas, (ax1, ax2), color='r', lw=1)
-show()
+plt.show()
