@@ -20,16 +20,35 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import six
-from six.moves import xrange, zip
+from matplotlib.externals import six
+from matplotlib.externals.six.moves import xrange, zip
 
+import os
+import platform
 import sys
 import itertools
+try:
+    # python3
+    from base64 import encodebytes
+except ImportError:
+    # python2
+    from base64 import encodestring as encodebytes
 import contextlib
+import tempfile
 from matplotlib.cbook import iterable, is_string_like
 from matplotlib.compat import subprocess
 from matplotlib import verbose
-from matplotlib import rcParams
+from matplotlib import rcParams, rcParamsDefault
+
+# Process creation flag for subprocess to prevent it raising a terminal
+# window. See for example:
+# https://stackoverflow.com/questions/24130623/using-python-subprocess-popen-cant-prevent-exe-stopped-working-prompt
+if platform.system() == 'Windows':
+    CREATE_NO_WINDOW = 0x08000000
+    subprocess_creation_flags = CREATE_NO_WINDOW
+else:
+    # Apparently None won't work here
+    subprocess_creation_flags = 0
 
 # Other potential writing methods:
 # * http://pymedia.org/
@@ -189,7 +208,8 @@ class MovieWriter(object):
                        ' '.join(command))
         self._proc = subprocess.Popen(command, shell=False,
                                       stdout=output, stderr=output,
-                                      stdin=subprocess.PIPE)
+                                      stdin=subprocess.PIPE,
+                                      creationflags=subprocess_creation_flags)
 
     def finish(self):
         'Finish any processing for writing the movie.'
@@ -226,6 +246,7 @@ class MovieWriter(object):
     def cleanup(self):
         'Clean-up and collect the process used to write the movie file.'
         out, err = self._proc.communicate()
+        self._frame_sink().close()
         verbose.report('MovieWriter -- '
                        'Command stdout:\n%s' % out, level='debug')
         verbose.report('MovieWriter -- '
@@ -246,11 +267,14 @@ class MovieWriter(object):
         Check to see if a MovieWriter subclass is actually available by
         running the commandline tool.
         '''
+        if not cls.bin_path():
+            return False
         try:
             p = subprocess.Popen(cls.bin_path(),
                              shell=False,
                              stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE)
+                             stderr=subprocess.PIPE,
+                             creationflags=subprocess_creation_flags)
             p.communicate()
             return True
         except OSError:
@@ -333,7 +357,7 @@ class FileMovieWriter(MovieWriter):
         All keyword arguments in savefig_kwargs are passed on to the 'savefig'
         command that saves the figure.
         '''
-        #Overloaded to explicitly close temp file.
+        # Overloaded to explicitly close temp file.
         verbose.report('MovieWriter.grab_frame: Grabbing frame.',
                        level='debug')
         try:
@@ -367,9 +391,8 @@ class FileMovieWriter(MovieWriter):
     def cleanup(self):
         MovieWriter.cleanup(self)
 
-        #Delete temporary files
+        # Delete temporary files
         if self.clear_temp:
-            import os
             verbose.report(
                 'MovieWriter: clearing temporary fnames=%s' %
                 str(self._temp_names),
@@ -380,7 +403,7 @@ class FileMovieWriter(MovieWriter):
 
 # Base class of ffmpeg information. Has the config keys and the common set
 # of arguments that controls the *output* side of things.
-class FFMpegBase:
+class FFMpegBase(object):
     exec_key = 'animation.ffmpeg_path'
     args_key = 'animation.ffmpeg_args'
 
@@ -389,6 +412,12 @@ class FFMpegBase:
         # The %dk adds 'k' as a suffix so that ffmpeg treats our bitrate as in
         # kbps
         args = ['-vcodec', self.codec]
+        # For h264, the default format is yuv444p, which is not compatible
+        # with quicktime (and others). Specifying yuv420p fixes playback on
+        # iOS,as well as HTML5 video in firefox and safari (on both Win and
+        # OSX). Also fixes internet explorer. This is as of 2015/10/29.
+        if self.codec == 'h264' and '-pix_fmt' not in self.extra_args:
+            args.extend(['-pix_fmt', 'yuv420p'])
         if self.bitrate > 0:
             args.extend(['-b', '%dk' % self.bitrate])
         if self.extra_args:
@@ -415,7 +444,7 @@ class FFMpegWriter(MovieWriter, FFMpegBase):
         return args
 
 
-#Combine FFMpeg options with temp file-based writing
+# Combine FFMpeg options with temp file-based writing
 @writers.register('ffmpeg_file')
 class FFMpegFileWriter(FileMovieWriter, FFMpegBase):
     supported_formats = ['png', 'jpeg', 'ppm', 'tiff', 'sgi', 'bmp',
@@ -450,7 +479,7 @@ class AVConvFileWriter(AVConvBase, FFMpegFileWriter):
 
 # Base class of mencoder information. Contains configuration key information
 # as well as arguments for controlling *output*
-class MencoderBase:
+class MencoderBase(object):
     exec_key = 'animation.mencoder_path'
     args_key = 'animation.mencoder_args'
 
@@ -468,10 +497,12 @@ class MencoderBase:
     @property
     def output_args(self):
         self._remap_metadata()
-        args = ['-o', self.outfile, '-ovc', 'lavc', '-lavcopts',
-                'vcodec=%s' % self.codec]
+        lavcopts = {'vcodec': self.codec}
         if self.bitrate > 0:
-            args.append('vbitrate=%d' % self.bitrate)
+            lavcopts.update(vbitrate=self.bitrate)
+        args = ['-o', self.outfile, '-ovc', 'lavc', '-lavcopts',
+                ':'.join(itertools.starmap('{0}={1}'.format,
+                                           lavcopts.items()))]
         if self.extra_args:
             args.extend(self.extra_args)
         if self.metadata:
@@ -509,7 +540,7 @@ class MencoderFileWriter(FileMovieWriter, MencoderBase):
 
 
 # Base class for animated GIFs with convert utility
-class ImageMagickBase:
+class ImageMagickBase(object):
     exec_key = 'animation.convert_path'
     args_key = 'animation.convert_args'
 
@@ -521,6 +552,27 @@ class ImageMagickBase:
     def output_args(self):
         return [self.outfile]
 
+    @classmethod
+    def _init_from_registry(cls):
+        if sys.platform != 'win32' or rcParams[cls.exec_key] != 'convert':
+            return
+        from matplotlib.externals.six.moves import winreg
+        for flag in (0, winreg.KEY_WOW64_32KEY, winreg.KEY_WOW64_64KEY):
+            try:
+                hkey = winreg.OpenKeyEx(winreg.HKEY_LOCAL_MACHINE,
+                                        'Software\\Imagemagick\\Current',
+                                        0, winreg.KEY_QUERY_VALUE | flag)
+                binpath = winreg.QueryValueEx(hkey, 'BinPath')[0]
+                winreg.CloseKey(hkey)
+                binpath += '\\convert.exe'
+                break
+            except Exception:
+                binpath = ''
+        rcParams[cls.exec_key] = rcParamsDefault[cls.exec_key] = binpath
+
+
+ImageMagickBase._init_from_registry()
+
 
 @writers.register('imagemagick')
 class ImageMagickWriter(MovieWriter, ImageMagickBase):
@@ -572,9 +624,6 @@ class Animation(object):
         self.frame_seq = self.new_frame_seq()
         self.event_source = event_source
 
-        # Clear the initial frame
-        self._init_draw()
-
         # Instead of starting the event source now, we connect to the figure's
         # draw_event, so that we only start once the figure has been drawn.
         self._first_draw_id = fig.canvas.mpl_connect('draw_event', self._start)
@@ -591,14 +640,18 @@ class Animation(object):
         Starts interactive animation. Adds the draw frame command to the GUI
         handler, calls show to start the event loop.
         '''
-        # On start, we add our callback for stepping the animation and
-        # actually start the event_source. We also disconnect _start
-        # from the draw_events
-        self.event_source.add_callback(self._step)
-        self.event_source.start()
+        # First disconnect our draw event handler
         self._fig.canvas.mpl_disconnect(self._first_draw_id)
         self._first_draw_id = None  # So we can check on save
 
+        # Now do any initial draw
+        self._init_draw()
+
+        # Add our callback for stepping the animation and
+        # actually start the event_source.
+        self.event_source.add_callback(self._step)
+        self.event_source.start()
+
     def _stop(self, *args):
         # On stop we disconnect all of our events.
         if self._blit:
@@ -677,7 +730,7 @@ class Animation(object):
                       "'savefig_kwargs' as it is only currently supported "
                       "with the writers 'ffmpeg_file' and 'mencoder_file' "
                       "(writer used: "
-                      "'{}').".format(
+                      "'{0}').".format(
                           writer if isinstance(writer, six.string_types)
                           else writer.__class__.__name__))
                 savefig_kwargs.pop('bbox_inches')
@@ -702,6 +755,8 @@ class Animation(object):
         # Re-use the savefig DPI for ours if none is given
         if dpi is None:
             dpi = rcParams['savefig.dpi']
+            if dpi == 'figure':
+                dpi = self._fig.dpi
 
         if codec is None:
             codec = rcParams['animation.codec']
@@ -744,10 +799,13 @@ class Animation(object):
         # since GUI widgets are gone. Either need to remove extra code to
         # allow for this non-existant use case or find a way to make it work.
         with writer.saving(self._fig, filename, dpi):
+            for anim in all_anim:
+                # Clear the initial frame
+                anim._init_draw()
             for data in zip(*[a.new_saved_frame_seq()
                               for a in all_anim]):
                 for anim, d in zip(all_anim, data):
-                    #TODO: Need to see if turning off blit is really necessary
+                    # TODO: Need to see if turning off blit is really necessary
                     anim._draw_next_frame(d, blit=False)
                 writer.grab_frame(**savefig_kwargs)
 
@@ -869,6 +927,59 @@ class Animation(object):
         self._resize_id = self._fig.canvas.mpl_connect('resize_event',
                                                        self._handle_resize)
 
+    def to_html5_video(self):
+        r'''Returns animation as an HTML5 video tag.
+
+        This saves the animation as an h264 video, encoded in base64
+        directly into the HTML5 video tag. This respects the rc parameters
+        for the writer as well as the bitrate. This also makes use of the
+        ``interval`` to control the speed, and uses the ``repeat``
+        paramter to decide whether to loop.
+        '''
+        VIDEO_TAG = r'''<video {size} {options}>
+  <source type="video/mp4" src="data:video/mp4;base64,{video}">
+  Your browser does not support the video tag.
+</video>'''
+        # Cache the the rendering of the video as HTML
+        if not hasattr(self, '_base64_video'):
+            # First write the video to a tempfile. Set delete to False
+            # so we can re-open to read binary data.
+            with tempfile.NamedTemporaryFile(suffix='.m4v',
+                                             delete=False) as f:
+                # We create a writer manually so that we can get the
+                # appropriate size for the tag
+                Writer = writers[rcParams['animation.writer']]
+                writer = Writer(codec='h264',
+                                bitrate=rcParams['animation.bitrate'],
+                                fps=1000. / self._interval)
+                self.save(f.name, writer=writer)
+
+            # Now open and base64 encode
+            with open(f.name, 'rb') as video:
+                vid64 = encodebytes(video.read())
+                self._base64_video = vid64.decode('ascii')
+                self._video_size = 'width="{0}" height="{1}"'.format(
+                        *writer.frame_size)
+
+            # Now we can remove
+            os.remove(f.name)
+
+        # Default HTML5 options are to autoplay and to display video controls
+        options = ['controls', 'autoplay']
+
+        # If we're set to repeat, make it loop
+        if self.repeat:
+            options.append('loop')
+        return VIDEO_TAG.format(video=self._base64_video,
+                                size=self._video_size,
+                                options=' '.join(options))
+
+    def _repr_html_(self):
+        r'IPython display hook for rendering.'
+        fmt = rcParams['animation.html']
+        if fmt == 'html5':
+            return self.to_html5_video()
+
 
 class TimedAnimation(Animation):
     '''
@@ -909,6 +1020,7 @@ class TimedAnimation(Animation):
         # back.
         still_going = Animation._step(self, *args)
         if not still_going and self.repeat:
+            self._init_draw()
             self.frame_seq = self.new_frame_seq()
             if self._repeat_delay:
                 self.event_source.remove_callback(self._step)
@@ -955,17 +1067,18 @@ class ArtistAnimation(TimedAnimation):
 
     def _init_draw(self):
         # Make all the artists involved in *any* frame invisible
-        axes = []
+        figs = set()
         for f in self.new_frame_seq():
             for artist in f:
                 artist.set_visible(False)
+                artist.set_animated(self._blit)
                 # Assemble a list of unique axes that need flushing
-                if artist.axes not in axes:
-                    axes.append(artist.axes)
+                if artist.axes.figure not in figs:
+                    figs.add(artist.axes.figure)
 
         # Flush the needed axes
-        for ax in axes:
-            ax.figure.canvas.draw()
+        for fig in figs:
+            fig.canvas.draw_idle()
 
     def _pre_draw(self, framedata, blit):
         '''
@@ -1064,7 +1177,10 @@ class FuncAnimation(TimedAnimation):
         # no saved frames, generate a new frame sequence and take the first
         # save_count entries in it.
         if self._save_seq:
-            return iter(self._save_seq)
+            # While iterating we are going to update _save_seq
+            # so make a copy to safely iterate over
+            self._old_saved_seq = list(self._save_seq)
+            return iter(self._old_saved_seq)
         else:
             return itertools.islice(self.new_frame_seq(), self.save_count)
 
@@ -1075,8 +1191,13 @@ class FuncAnimation(TimedAnimation):
         # artists.
         if self._init_func is None:
             self._draw_frame(next(self.new_frame_seq()))
+
         else:
             self._drawn_artists = self._init_func()
+            if self._blit:
+                for a in self._drawn_artists:
+                    a.set_animated(self._blit)
+        self._save_seq = []
 
     def _draw_frame(self, framedata):
         # Save the data for potential saving of movies.
@@ -1089,3 +1210,6 @@ class FuncAnimation(TimedAnimation):
         # Call the func with framedata and args. If blitting is desired,
         # func needs to return a sequence of any artists that were modified.
         self._drawn_artists = self._func(framedata, *self._args)
+        if self._blit:
+            for a in self._drawn_artists:
+                a.set_animated(self._blit)
