@@ -1,15 +1,21 @@
-from __future__ import division, print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from matplotlib.externals import six
+
 import re
 import warnings
 import inspect
+import numpy as np
 import matplotlib
 import matplotlib.cbook as cbook
+from matplotlib.cbook import mplDeprecation
 from matplotlib import docstring, rcParams
-from transforms import Bbox, IdentityTransform, TransformedBbox, \
-                       TransformedPath, Transform
-from path import Path
+from .transforms import (Bbox, IdentityTransform, TransformedBbox,
+                         TransformedPath, Transform)
+from .path import Path
 
-## Note, matplotlib artists use the doc strings for set and get
+# Note, matplotlib artists use the doc strings for set and get
 # methods to enable the introspection methods of setp and getp.  Every
 # set_* method should have a docstring containing the line
 #
@@ -63,6 +69,11 @@ def allow_rasterization(draw):
     return draw_wrapper
 
 
+def _stale_axes_callback(self, val):
+    if self.axes:
+        self.axes.stale = val
+
+
 class Artist(object):
     """
     Abstract base class for someone who renders into a
@@ -73,6 +84,9 @@ class Artist(object):
     zorder = 0
 
     def __init__(self):
+        self._stale = True
+        self.stale_callback = None
+        self._axes = None
         self.figure = None
 
         self._transform = None
@@ -83,13 +97,12 @@ class Artist(object):
         self.clipbox = None
         self._clippath = None
         self._clipon = True
-        self._lod = False
         self._label = ''
         self._picker = None
         self._contains = None
         self._rasterized = None
         self._agg_filter = None
-
+        self._mouseover = False
         self.eventson = False  # fire events only if eventson
         self._oid = 0  # an observer id
         self._propobservers = {}  # a dict from oids to funcs
@@ -110,6 +123,7 @@ class Artist(object):
         # remove the unpicklable remove method, this will get re-added on load
         # (by the axes) if the artist lives on an axes.
         d['_remove_method'] = None
+        d['stale_callback'] = None
         return d
 
     def remove(self):
@@ -133,6 +147,23 @@ class Artist(object):
         # callback has one parameter, which is the child to be removed.
         if self._remove_method is not None:
             self._remove_method(self)
+            # clear stale callback
+            self.stale_callback = None
+            _ax_flag = False
+            if hasattr(self, 'axes') and self.axes:
+                # remove from the mouse hit list
+                self.axes.mouseover_set.discard(self)
+                # mark the axes as stale
+                self.axes.stale = True
+                # decouple the artist from the axes
+                self.axes = None
+                _ax_flag = True
+
+            if self.figure:
+                self.figure = None
+                if not _ax_flag:
+                    self.figure = True
+
         else:
             raise NotImplementedError('cannot remove artist')
         # TODO: the fix for the collections relim problem is to move the
@@ -154,7 +185,6 @@ class Artist(object):
         """
         ax = getattr(self, 'axes', None)
         if ax is None or ax.xaxis is None:
-            #print 'artist.convert_xunits no conversion: ax=%s'%ax
             return x
         return ax.xaxis.convert_units(x)
 
@@ -172,23 +202,84 @@ class Artist(object):
         Set the :class:`~matplotlib.axes.Axes` instance in which the
         artist resides, if any.
 
+        This has been deprecated in mpl 1.5, please use the
+        axes property.  Will be removed in 1.7 or 2.0.
+
         ACCEPTS: an :class:`~matplotlib.axes.Axes` instance
         """
+        warnings.warn(_get_axes_msg, mplDeprecation, stacklevel=1)
         self.axes = axes
 
     def get_axes(self):
         """
         Return the :class:`~matplotlib.axes.Axes` instance the artist
-        resides in, or *None*
+        resides in, or *None*.
+
+        This has been deprecated in mpl 1.5, please use the
+        axes property.  Will be removed in 1.7 or 2.0.
         """
+        warnings.warn(_get_axes_msg, mplDeprecation, stacklevel=1)
         return self.axes
 
+    @property
+    def axes(self):
+        """
+        The :class:`~matplotlib.axes.Axes` instance the artist
+        resides in, or *None*.
+        """
+        return self._axes
+
+    @axes.setter
+    def axes(self, new_axes):
+
+        if (new_axes is not None and
+                (self._axes is not None and new_axes != self._axes)):
+            raise ValueError("Can not reset the axes.  You are "
+                             "probably trying to re-use an artist "
+                             "in more than one Axes which is not "
+                             "supported")
+
+        self._axes = new_axes
+        if new_axes is not None and new_axes is not self:
+            self.stale_callback = _stale_axes_callback
+
+        return new_axes
+
+    @property
+    def stale(self):
+        """
+        If the artist is 'stale' and needs to be re-drawn for the output to
+        match the internal state of the artist.
+        """
+        return self._stale
+
+    @stale.setter
+    def stale(self, val):
+        self._stale = val
+
+        # if the artist is animated it does not take normal part in the
+        # draw stack and is not expected to be drawn as part of the normal
+        # draw loop (when not saving) so do not propagate this change
+        if self.get_animated():
+            return
+
+        if val and self.stale_callback is not None:
+            self.stale_callback(self, val)
+
     def get_window_extent(self, renderer):
         """
         Get the axes bounding box in display space.
         Subclasses should override for inclusion in the bounding box
         "tight" calculation. Default is to return an empty bounding
         box at 0, 0.
+
+        Be careful when using this function, the results will not update
+        if the artist window extent of the artist changes.  The extent
+        can change due to any changes in the transform stack, such as
+        changing the axes limits, the figure size, or the canvas used
+        (as is done when saving a figure).  This can lead to unexpected
+        behavior where interactive figures will look fine on the screen,
+        but will save incorrectly.
         """
         return Bbox([[0, 0], [0, 0]])
 
@@ -225,7 +316,7 @@ class Artist(object):
         Fire an event when property changed, calling all of the
         registered callbacks.
         """
-        for oid, func in self._propobservers.iteritems():
+        for oid, func in six.iteritems(self._propobservers):
             func(self)
 
     def is_transform_set(self):
@@ -245,6 +336,7 @@ class Artist(object):
         self._transform = t
         self._transformSet = True
         self.pchanged()
+        self.stale = True
 
     def get_transform(self):
         """
@@ -290,7 +382,7 @@ class Artist(object):
         selection, such as which points are contained in the pick radius.  See
         individual artists for details.
         """
-        if callable(self._contains):
+        if six.callable(self._contains):
             return self._contains(self, mouseevent)
         warnings.warn("'%s' needs 'contains' method" % self.__class__.__name__)
         return False, {}
@@ -335,7 +427,7 @@ class Artist(object):
         # Pick self
         if self.pickable():
             picker = self.get_picker()
-            if callable(picker):
+            if six.callable(picker):
                 inside, prop = picker(self, mouseevent)
             else:
                 inside, prop = self.contains(mouseevent)
@@ -346,11 +438,14 @@ class Artist(object):
         for a in self.get_children():
             # make sure the event happened in the same axes
             ax = getattr(a, 'axes', None)
-            if mouseevent.inaxes is None or mouseevent.inaxes == ax:
+            if mouseevent.inaxes is None or ax is None or \
+                    mouseevent.inaxes == ax:
                 # we need to check if mouseevent.inaxes is None
                 # because some objects associated with an axes (e.g., a
                 # tick label) can be outside the bounding box of the
                 # axes and inaxes will be None
+                # also check that ax is None so that it traverse objects
+                # which do no have an axes property but children might
                 a.pick(mouseevent)
 
     def set_picker(self, picker):
@@ -458,6 +553,7 @@ class Artist(object):
         Only supported by the Agg and MacOSX backends.
         """
         self._snap = snap
+        self.stale = True
 
     def get_sketch_params(self):
         """
@@ -483,7 +579,7 @@ class Artist(object):
 
     def set_sketch_params(self, scale=None, length=None, randomness=None):
         """
-        Sets the the sketch parameters.
+        Sets the sketch parameters.
 
         Parameters
         ----------
@@ -505,6 +601,7 @@ class Artist(object):
             self._sketch = None
         else:
             self._sketch = (scale, length or 128.0, randomness or 16.0)
+        self.stale = True
 
     def set_path_effects(self, path_effects):
         """
@@ -512,6 +609,7 @@ class Artist(object):
         matplotlib.patheffect._Base class or its derivatives.
         """
         self._path_effects = path_effects
+        self.stale = True
 
     def get_path_effects(self):
         return self._path_effects
@@ -530,8 +628,21 @@ class Artist(object):
 
         ACCEPTS: a :class:`matplotlib.figure.Figure` instance
         """
+        # if this is a no-op just return
+        if self.figure is fig:
+            return
+        # if we currently have a figure (the case of both `self.figure`
+        # and `fig` being none is taken care of above) we then user is
+        # trying to change the figure an artist is associated with which
+        # is not allowed for the same reason as adding the same instance
+        # to more than one Axes
+        if self.figure is not None:
+            raise RuntimeError("Can not put single artist in "
+                               "more than one figure")
         self.figure = fig
-        self.pchanged()
+        if self.figure and self.figure is not self:
+            self.pchanged()
+        self.stale = True
 
     def set_clip_box(self, clipbox):
         """
@@ -541,6 +652,7 @@ class Artist(object):
         """
         self.clipbox = clipbox
         self.pchanged()
+        self.stale = True
 
     def set_clip_path(self, path, transform=None):
         """
@@ -569,7 +681,7 @@ class Artist(object):
         if transform is None:
             if isinstance(path, Rectangle):
                 self.clipbox = TransformedBbox(Bbox.unit(),
-                                              path.get_transform())
+                                               path.get_transform())
                 self._clippath = None
                 success = True
             elif isinstance(path, Patch):
@@ -593,8 +705,10 @@ class Artist(object):
         if not success:
             print(type(path), type(transform))
             raise TypeError("Invalid arguments to set_clip_path")
-
+        # this may result in the callbacks being hit twice, but grantees they
+        # will be hit at least once
         self.pchanged()
+        self.stale = True
 
     def get_alpha(self):
         """
@@ -637,10 +751,16 @@ class Artist(object):
         """
         Set whether artist uses clipping.
 
+        When False artists will be visible out side of the axes which
+        can lead to unexpected results.
+
         ACCEPTS: [True | False]
         """
         self._clipon = b
+        # This may result in the callbacks being hit twice, but ensures they
+        # are hit at least once
         self.pchanged()
+        self.stale = True
 
     def _set_gc_clip(self, gc):
         'Set the clip properly for the gc'
@@ -679,11 +799,13 @@ class Artist(object):
 
         """
         self._agg_filter = filter_func
+        self.stale = True
 
     def draw(self, renderer, *args, **kwargs):
         'Derived classes drawing method'
         if not self.get_visible():
             return
+        self.stale = False
 
     def set_alpha(self, alpha):
         """
@@ -694,17 +816,7 @@ class Artist(object):
         """
         self._alpha = alpha
         self.pchanged()
-
-    def set_lod(self, on):
-        """
-        Set Level of Detail on or off.  If on, the artists may examine
-        things like the pixel width of the axes and draw a subset of
-        their contents accordingly
-
-        ACCEPTS: [True | False]
-        """
-        self._lod = on
-        self.pchanged()
+        self.stale = True
 
     def set_visible(self, b):
         """
@@ -714,6 +826,7 @@ class Artist(object):
         """
         self._visible = b
         self.pchanged()
+        self.stale = True
 
     def set_animated(self, b):
         """
@@ -721,8 +834,9 @@ class Artist(object):
 
         ACCEPTS: [True | False]
         """
-        self._animated = b
-        self.pchanged()
+        if self._animated != b:
+            self._animated = b
+            self.pchanged()
 
     def update(self, props):
         """
@@ -733,15 +847,19 @@ class Artist(object):
         self.eventson = False
         changed = False
 
-        for k, v in props.iteritems():
-            func = getattr(self, 'set_' + k, None)
-            if func is None or not callable(func):
-                raise AttributeError('Unknown property %s' % k)
-            func(v)
+        for k, v in six.iteritems(props):
+            if k in ['axes']:
+                setattr(self, k, v)
+            else:
+                func = getattr(self, 'set_' + k, None)
+                if func is None or not six.callable(func):
+                    raise AttributeError('Unknown property %s' % k)
+                func(v)
             changed = True
         self.eventson = store
         if changed:
             self.pchanged()
+            self.stale = True
 
     def get_label(self):
         """
@@ -760,6 +878,7 @@ class Artist(object):
         else:
             self._label = None
         self.pchanged()
+        self.stale = True
 
     def get_zorder(self):
         """
@@ -776,6 +895,7 @@ class Artist(object):
         """
         self.zorder = level
         self.pchanged()
+        self.stale = True
 
     def update_from(self, other):
         'Copy properties from *other* to *self*.'
@@ -786,11 +906,11 @@ class Artist(object):
         self.clipbox = other.clipbox
         self._clipon = other._clipon
         self._clippath = other._clippath
-        self._lod = other._lod
         self._label = other._label
         self._sketch = other._sketch
         self._path_effects = other._path_effects
         self.pchanged()
+        self.stale = True
 
     def properties(self):
         """
@@ -800,13 +920,20 @@ class Artist(object):
 
     def set(self, **kwargs):
         """
-        A tkstyle set command, pass *kwargs* to set properties
+        A property batch setter. Pass *kwargs* to set properties.
+        Will handle property name collisions (e.g., if both
+        'color' and 'facecolor' are specified, the property
+        with higher priority gets set last).
+
         """
         ret = []
-        for k, v in kwargs.iteritems():
+        for k, v in sorted(kwargs.items(), reverse=True):
             k = k.lower()
             funcName = "set_%s" % k
-            func = getattr(self, funcName)
+            func = getattr(self, funcName, None)
+            if func is None:
+               raise TypeError('There is no %s property "%s"' %
+                               (self.__class__.__name__, k))
             ret.extend([func(v)])
         return ret
 
@@ -836,7 +963,7 @@ class Artist(object):
         elif cbook.issubclass_safe(match, Artist):
             def matchfunc(x):
                 return isinstance(x, match)
-        elif callable(match):
+        elif six.callable(match):
             matchfunc = match
         else:
             raise ValueError('match must be None, a matplotlib.artist.Artist '
@@ -848,15 +975,47 @@ class Artist(object):
             if matchfunc(c):
                 artists.append(c)
             artists.extend([thisc for thisc in
-                                c.findobj(matchfunc, include_self=False)
-                                                     if matchfunc(thisc)])
+                            c.findobj(matchfunc, include_self=False)
+                            if matchfunc(thisc)])
 
         if include_self and matchfunc(self):
             artists.append(self)
         return artists
 
+    def get_cursor_data(self, event):
+        """
+        Get the cursor data for a given event.
+        """
+        return None
 
-class ArtistInspector:
+    def format_cursor_data(self, data):
+        """
+        Return *cursor data* string formatted.
+        """
+        try:
+            data[0]
+        except (TypeError, IndexError):
+            data = [data]
+        return ', '.join('{:0.3g}'.format(item) for item in data if
+                isinstance(item, (np.floating, np.integer, int, float)))
+
+    @property
+    def mouseover(self):
+        return self._mouseover
+
+    @mouseover.setter
+    def mouseover(self, val):
+        val = bool(val)
+        self._mouseover = val
+        ax = self.axes
+        if ax:
+            if val:
+                ax.mouseover_set.add(self)
+            else:
+                ax.mouseover_set.discard(self)
+
+
+class ArtistInspector(object):
     """
     A helper class to inspect an :class:`~matplotlib.artist.Artist`
     and return information about it's settable properties and their
@@ -894,7 +1053,7 @@ class ArtistInspector:
         """
         names = [name for name in dir(self.o) if
                  (name.startswith('set_') or name.startswith('get_'))
-                 and callable(getattr(self.o, name))]
+                 and six.callable(getattr(self.o, name))]
         aliases = {}
         for name in names:
             func = getattr(self.o, name)
@@ -906,7 +1065,8 @@ class ArtistInspector:
         return aliases
 
     _get_valid_values_regex = re.compile(
-                r"\n\s*ACCEPTS:\s*((?:.|\n)*?)(?:$|(?:\n\n))")
+        r"\n\s*ACCEPTS:\s*((?:.|\n)*?)(?:$|(?:\n\n))"
+    )
 
     def get_valid_values(self, attr):
         """
@@ -916,7 +1076,8 @@ class ArtistInspector:
         for a line that begins with ACCEPTS:
 
         e.g., for a line linestyle, return
-        "[ ``'-'`` | ``'--'`` | ``'-.'`` | ``':'`` | ``'steps'`` | ``'None'`` ]"
+        "[ ``'-'`` | ``'--'`` | ``'-.'`` | ``':'`` | ``'steps'`` | ``'None'``
+        ]"
         """
 
         name = 'set_%s' % attr
@@ -933,7 +1094,7 @@ class ArtistInspector:
 
         match = self._get_valid_values_regex.search(docstring)
         if match is not None:
-            return match.group(1).replace('\n', ' ')
+            return re.sub("\n *", " ", match.group(1))
         return 'unknown'
 
     def _get_setters_and_targets(self):
@@ -947,9 +1108,13 @@ class ArtistInspector:
             if not name.startswith('set_'):
                 continue
             o = getattr(self.o, name)
-            if not callable(o):
+            if not six.callable(o):
                 continue
-            if len(inspect.getargspec(o)[0]) < 2:
+            if six.PY2:
+                nargs = len(inspect.getargspec(o)[0])
+            else:
+                nargs = len(inspect.getfullargspec(o)[0])
+            if nargs < 2:
                 continue
             func = o
             if self.is_alias(func):
@@ -993,7 +1158,7 @@ class ArtistInspector:
         if s in self.aliasd:
             return s + ''.join([' or %s' % x
                                 for x
-                                in self.aliasd[s].iterkeys()])
+                                in six.iterkeys(self.aliasd[s])])
         else:
             return s
 
@@ -1010,7 +1175,7 @@ class ArtistInspector:
         if s in self.aliasd:
             aliases = ''.join([' or %s' % x
                                for x
-                               in self.aliasd[s].iterkeys()])
+                               in six.iterkeys(self.aliasd[s])])
         else:
             aliases = ''
         return ':meth:`%s <%s>`%s' % (s, target, aliases)
@@ -1076,7 +1241,7 @@ class ArtistInspector:
 
         lines.append('')
         lines.append(table_formatstr)
-        lines.append(pad + 'Property'.ljust(col0_len + 3) + \
+        lines.append(pad + 'Property'.ljust(col0_len + 3) +
                      'Description'.ljust(col1_len))
         lines.append(table_formatstr)
 
@@ -1102,8 +1267,7 @@ class ArtistInspector:
         o = self.oorig
         getters = [name for name in dir(o)
                    if name.startswith('get_')
-                   and callable(getattr(o, name))]
-        #print getters
+                   and six.callable(getattr(o, name))]
         getters.sort()
         d = dict()
         for name in getters:
@@ -1112,7 +1276,9 @@ class ArtistInspector:
                 continue
 
             try:
-                val = func()
+                with warnings.catch_warnings():
+                    warnings.simplefilter('ignore')
+                    val = func()
             except:
                 continue
             else:
@@ -1126,7 +1292,7 @@ class ArtistInspector:
         """
 
         d = self.properties()
-        names = d.keys()
+        names = list(six.iterkeys(d))
         names.sort()
         lines = []
         for name in names:
@@ -1161,7 +1327,7 @@ class ArtistInspector:
         elif issubclass(match, Artist):
             def matchfunc(x):
                 return isinstance(x, match)
-        elif callable(match):
+        elif six.callable(match):
             matchfunc = func
         else:
             raise ValueError('match must be None, an '
@@ -1289,14 +1455,17 @@ def setp(obj, *args, **kwargs):
     funcvals = []
     for i in range(0, len(args) - 1, 2):
         funcvals.append((args[i], args[i + 1]))
-    funcvals.extend(kwargs.iteritems())
+    funcvals.extend(sorted(kwargs.items(), reverse=True))
 
     ret = []
     for o in objs:
         for s, val in funcvals:
             s = s.lower()
             funcName = "set_%s" % s
-            func = getattr(o, funcName)
+            func = getattr(o, funcName, None)
+            if func is None:
+                raise TypeError('There is no %s property "%s"' %
+                                (o.__class__.__name__, s))
             ret.extend([func(val)])
     return [x for x in cbook.flatten(ret)]
 
@@ -1305,8 +1474,11 @@ def kwdoc(a):
     hardcopy = matplotlib.rcParams['docstring.hardcopy']
     if hardcopy:
         return '\n'.join(ArtistInspector(a).pprint_setters_rest(
-                                            leadingspace=2))
+                         leadingspace=2))
     else:
         return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=2))
 
 docstring.interpd.update(Artist=kwdoc(Artist))
+
+_get_axes_msg = """This has been deprecated in mpl 1.5, please use the
+axes property.  A removal date has not been set."""
