@@ -1,7 +1,7 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import six
+from matplotlib.externals import six
 
 import math
 import os
@@ -26,7 +26,7 @@ from matplotlib import rcParams
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
      FigureManagerBase, FigureCanvasBase
-from matplotlib.cbook import is_string_like
+from matplotlib.cbook import is_string_like, restrict_dict
 from matplotlib.figure import Figure
 from matplotlib.mathtext import MathTextParser
 from matplotlib.transforms import Affine2D
@@ -109,7 +109,6 @@ class RendererGDK(RendererBase):
             #             int(w), int(h))
             # set clip rect?
 
-        im.flipud_out()
         rows, cols, image_str = im.as_rgba_str()
 
         image_array = np.fromstring(image_str, np.uint8)
@@ -120,7 +119,7 @@ class RendererGDK(RendererBase):
                                 width=cols, height=rows)
 
         array = pixbuf_get_pixels_array(pixbuf)
-        array[:,:,:] = image_array
+        array[:,:,:] = image_array[::-1]
 
         gc = self.new_gc()
 
@@ -138,9 +137,6 @@ class RendererGDK(RendererBase):
                                   int(x), int(y), cols, rows,
                                   gdk.RGB_DITHER_NONE, 0, 0)
 
-        # unflip
-        im.flipud_out()
-
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath=False, mtext=None):
         x, y = int(x), int(y)
@@ -477,7 +473,7 @@ class FigureCanvasGDK (FigureCanvasBase):
 
         # set the default quality, if we are writing a JPEG.
         # http://www.pygtk.org/docs/pygtk/class-gdkpixbuf.html#method-gdkpixbuf--save
-        options = cbook.restrict_dict(kwargs, ['quality'])
+        options = restrict_dict(kwargs, ['quality'])
         if format in ['jpg','jpeg']:
            if 'quality' not in options:
               options['quality'] = rcParams['savefig.jpeg_quality']
