@@ -1,12 +1,14 @@
 # Todd Miller   jmiller@stsci.edu
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
-from __future__ import division, print_function
+from matplotlib.externals import six
+from matplotlib.externals.six.moves import tkinter as Tk
+from matplotlib.externals.six.moves import tkinter_filedialog as FileDialog
 
 import os, sys, math
 import os.path
 
-import Tkinter as Tk, FileDialog
-
 # Paint image to Tk photo blitter extension
 import matplotlib.backends.tkagg as tkagg
 
@@ -18,7 +20,10 @@ from matplotlib.cbook import is_string_like
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase
 from matplotlib.backend_bases import FigureManagerBase, FigureCanvasBase
 from matplotlib.backend_bases import NavigationToolbar2, cursors, TimerBase
-from matplotlib.backend_bases import ShowBase
+from matplotlib.backend_bases import (ShowBase, ToolContainerBase,
+                                      StatusbarBase)
+from matplotlib.backend_managers import ToolManager
+from matplotlib import backend_tools
 from matplotlib._pylab_helpers import Gcf
 
 from matplotlib.figure import Figure
@@ -55,7 +60,7 @@ def raise_msg_to_str(msg):
     return msg
 
 def error_msg_tkpaint(msg, parent=None):
-    import tkMessageBox
+    from matplotlib.externals.six.moves import tkinter_messagebox as tkMessageBox
     tkMessageBox.showerror("matplotlib", msg)
 
 def draw_if_interactive():
@@ -107,6 +112,7 @@ def new_figure_manager_given_figure(num, figure):
     figManager = FigureManagerTkAgg(canvas, num, window)
     if matplotlib.is_interactive():
         figManager.show()
+        canvas.draw_idle()
     return figManager
 
 
@@ -220,7 +226,8 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
         t1,t2,w,h = self.figure.bbox.bounds
         w, h = int(w), int(h)
         self._tkcanvas = Tk.Canvas(
-            master=master, width=w, height=h, borderwidth=4)
+            master=master, width=w, height=h, borderwidth=0,
+            highlightthickness=0)
         self._tkphoto = Tk.PhotoImage(
             master=self._tkcanvas, width=w, height=h)
         self._tkcanvas.create_image(w//2, h//2, image=self._tkphoto)
@@ -244,14 +251,14 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
         # event to the window containing the canvas instead.
         # See http://wiki.tcl.tk/3893 (mousewheel) for details
         root = self._tkcanvas.winfo_toplevel()
-        root.bind("<MouseWheel>", self.scroll_event_windows)
+        root.bind("<MouseWheel>", self.scroll_event_windows, "+")
 
         # Can't get destroy events by binding to _tkcanvas. Therefore, bind
         # to the window and filter.
         def filter_destroy(evt):
             if evt.widget is self._tkcanvas:
                 self.close_event()
-        root.bind("<Destroy>", filter_destroy)
+        root.bind("<Destroy>", filter_destroy, "+")
 
         self._master = master
         self._tkcanvas.focus_set()
@@ -343,7 +350,6 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
             else:
                 self.leave_notify_event(guiEvent)
 
-
     def draw(self):
         FigureCanvasAgg.draw(self)
         tkagg.blit(self._tkphoto, self.renderer._renderer, colormode=2)
@@ -357,14 +363,18 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
 
     def draw_idle(self):
         'update drawing area only if idle'
-        d = self._idle
-        self._idle = False
-        def idle_draw(*args):
-            self.draw()
-            self._idle = True
+        if self._idle is False:
+            return
 
-        if d:
-            self._idle_callback = self._tkcanvas.after_idle(idle_draw)
+        self._idle = False
+
+        def idle_draw(*args):
+            try:
+                self.draw()
+            finally:
+                self._idle = True
+
+        self._idle_callback = self._tkcanvas.after_idle(idle_draw)
 
     def get_tk_widget(self):
         """returns the Tk widget used to implement FigureCanvasTkAgg.
@@ -453,15 +463,20 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
         # In general, the modifier key is excluded from the modifier flag,
         # however this is not the case on "darwin", so double check that
         # we aren't adding repeat modifier flags to a modifier key.
-        modifiers = [(6, 'super', 'super'),
-                     (3, 'alt', 'alt'),
-                     (2, 'ctrl', 'control'),
-                    ]
-        if sys.platform == 'darwin':
+        if sys.platform == 'win32':
+            modifiers = [(17, 'alt', 'alt'),
+                         (2, 'ctrl', 'control'),
+                         ]
+        elif sys.platform == 'darwin':
             modifiers = [(3, 'super', 'super'),
                          (4, 'alt', 'alt'),
                          (2, 'ctrl', 'control'),
-                        ]
+                         ]
+        else:
+            modifiers = [(6, 'super', 'super'),
+                         (3, 'alt', 'alt'),
+                         (2, 'ctrl', 'control'),
+                         ]
 
         if key is not None:
             # note, shift is not added to the keys as this is already accounted for
@@ -521,26 +536,45 @@ class FigureManagerTkAgg(FigureManagerBase):
         self.window.withdraw()
         self.set_window_title("Figure %d" % num)
         self.canvas = canvas
-        self._num =  num
-        _, _, w, h = canvas.figure.bbox.bounds
-        w, h = int(w), int(h)
-        self.window.minsize(int(w*3/4),int(h*3/4))
-        if matplotlib.rcParams['toolbar']=='classic':
-            self.toolbar = NavigationToolbar( canvas, self.window )
-        elif matplotlib.rcParams['toolbar']=='toolbar2':
-            self.toolbar = NavigationToolbar2TkAgg( canvas, self.window )
-        else:
-            self.toolbar = None
-        if self.toolbar is not None:
-            self.toolbar.update()
         self.canvas._tkcanvas.pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)
+        self._num = num
+
+        self.toolmanager = self._get_toolmanager()
+        self.toolbar = self._get_toolbar()
+        self.statusbar = None
+
+        if self.toolmanager:
+            backend_tools.add_tools_to_manager(self.toolmanager)
+            if self.toolbar:
+                backend_tools.add_tools_to_container(self.toolbar)
+                self.statusbar = StatusbarTk(self.window, self.toolmanager)
+
         self._shown = False
 
         def notify_axes_change(fig):
             'this will be called whenever the current axes is changed'
-            if self.toolbar != None: self.toolbar.update()
+            if self.toolmanager is not None:
+                pass
+            elif self.toolbar is not None:
+                self.toolbar.update()
         self.canvas.figure.add_axobserver(notify_axes_change)
 
+    def _get_toolbar(self):
+        if matplotlib.rcParams['toolbar'] == 'toolbar2':
+            toolbar = NavigationToolbar2TkAgg(self.canvas, self.window)
+        elif matplotlib.rcParams['toolbar'] == 'toolmanager':
+            toolbar = ToolbarTk(self.toolmanager, self.window)
+        else:
+            toolbar = None
+        return toolbar
+
+    def _get_toolmanager(self):
+        if rcParams['toolbar'] != 'toolbar2':
+            toolmanager = ToolManager(self.canvas)
+        else:
+            toolmanager = None
+        return toolmanager
+
     def resize(self, width, height=None):
         # before 09-12-22, the resize method takes a single *event*
         # parameter. On the other hand, the resize method of other
@@ -556,7 +590,8 @@ class FigureManagerTkAgg(FigureManagerBase):
         else:
             self.canvas._tkcanvas.master.geometry("%dx%d" % (width, height))
 
-        self.toolbar.configure(width=width)
+        if self.toolbar is not None:
+            self.toolbar.configure(width=width)
 
     def show(self):
         """
@@ -598,7 +633,7 @@ class FigureManagerTkAgg(FigureManagerBase):
         self.window.attributes('-fullscreen', not is_fullscreen)
 
 
-class AxisMenu:
+class AxisMenu(object):
     def __init__(self, master, naxes):
         self._master = master
         self._naxes = naxes
@@ -661,131 +696,6 @@ class AxisMenu:
         self.set_active()
 
 
-class NavigationToolbar(Tk.Frame):
-    """
-    Public attributes
-
-      canvas   - the FigureCanvas  (gtk.DrawingArea)
-      win   - the gtk.Window
-
-    """
-    def _Button(self, text, file, command):
-        file = os.path.join(rcParams['datapath'], 'images', file)
-        im = Tk.PhotoImage(master=self, file=file)
-        b = Tk.Button(
-            master=self, text=text, padx=2, pady=2, image=im, command=command)
-        b._ntimage = im
-        b.pack(side=Tk.LEFT)
-        return b
-
-    def __init__(self, canvas, window):
-        self.canvas = canvas
-        self.window = window
-
-        xmin, xmax = canvas.figure.bbox.intervalx
-        height, width = 50, xmax-xmin
-        Tk.Frame.__init__(self, master=self.window,
-                          width=int(width), height=int(height),
-                          borderwidth=2)
-
-        self.update()  # Make axes menu
-
-        self.bLeft = self._Button(
-            text="Left", file="stock_left",
-            command=lambda x=-1: self.panx(x))
-
-        self.bRight = self._Button(
-            text="Right", file="stock_right",
-            command=lambda x=1: self.panx(x))
-
-        self.bZoomInX = self._Button(
-            text="ZoomInX",file="stock_zoom-in",
-            command=lambda x=1: self.zoomx(x))
-
-        self.bZoomOutX = self._Button(
-            text="ZoomOutX", file="stock_zoom-out",
-            command=lambda x=-1: self.zoomx(x))
-
-        self.bUp = self._Button(
-            text="Up", file="stock_up",
-            command=lambda y=1: self.pany(y))
-
-        self.bDown = self._Button(
-            text="Down", file="stock_down",
-            command=lambda y=-1: self.pany(y))
-
-        self.bZoomInY = self._Button(
-            text="ZoomInY", file="stock_zoom-in",
-            command=lambda y=1: self.zoomy(y))
-
-        self.bZoomOutY = self._Button(
-            text="ZoomOutY",file="stock_zoom-out",
-            command=lambda y=-1: self.zoomy(y))
-
-        self.bSave = self._Button(
-            text="Save", file="stock_save_as",
-            command=self.save_figure)
-
-        self.pack(side=Tk.BOTTOM, fill=Tk.X)
-
-
-    def set_active(self, ind):
-        self._ind = ind
-        self._active = [ self._axes[i] for i in self._ind ]
-
-    def panx(self, direction):
-        for a in self._active:
-            a.xaxis.pan(direction)
-        self.canvas.draw()
-
-    def pany(self, direction):
-        for a in self._active:
-            a.yaxis.pan(direction)
-        self.canvas.draw()
-
-    def zoomx(self, direction):
-
-        for a in self._active:
-            a.xaxis.zoom(direction)
-        self.canvas.draw()
-
-    def zoomy(self, direction):
-
-        for a in self._active:
-            a.yaxis.zoom(direction)
-        self.canvas.draw()
-
-    def save_figure(self, *args):
-        fs = FileDialog.SaveFileDialog(master=self.window,
-                                       title='Save the figure')
-        try:
-            self.lastDir
-        except AttributeError:
-            self.lastDir = os.curdir
-
-        fname = fs.go(dir_or_file=self.lastDir) # , pattern="*.png")
-        if fname is None: # Cancel
-            return
-
-        self.lastDir = os.path.dirname(fname)
-        try:
-            self.canvas.print_figure(fname)
-        except IOError as msg:
-            err = '\n'.join(map(str, msg))
-            msg = 'Failed to save %s: Error msg was\n\n%s' % (
-                fname, err)
-            error_msg_tkpaint(msg)
-
-    def update(self):
-        _focus = windowing.FocusManager()
-        self._axes = self.canvas.figure.axes
-        naxes = len(self._axes)
-        if not hasattr(self, "omenu"):
-            self.set_active(range(naxes))
-            self.omenu = AxisMenu(master=self, naxes=naxes)
-        else:
-            self.omenu.adjust(naxes)
-
 class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
     """
     Public attributes
@@ -872,8 +782,7 @@ class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
         canvas.get_tk_widget().pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)
 
     def save_figure(self, *args):
-        from tkFileDialog import asksaveasfilename
-        from tkMessageBox import showerror
+        from matplotlib.externals.six.moves import tkinter_tkfiledialog, tkinter_messagebox
         filetypes = self.canvas.get_supported_filetypes().copy()
         default_filetype = self.canvas.get_default_filetype()
 
@@ -882,7 +791,7 @@ class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
         default_filetype_name = filetypes[default_filetype]
         del filetypes[default_filetype]
 
-        sorted_filetypes = filetypes.items()
+        sorted_filetypes = list(six.iteritems(filetypes))
         sorted_filetypes.sort()
         sorted_filetypes.insert(0, (default_filetype, default_filetype_name))
 
@@ -892,13 +801,13 @@ class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
         # adding a default extension seems to break the
         # asksaveasfilename dialog when you choose various save types
         # from the dropdown.  Passing in the empty string seems to
-        # work - JDH
+        # work - JDH!
         #defaultextension = self.canvas.get_default_filetype()
         defaultextension = ''
         initialdir = rcParams.get('savefig.directory', '')
         initialdir = os.path.expanduser(initialdir)
         initialfile = self.canvas.get_default_filename()
-        fname = asksaveasfilename(
+        fname = tkinter_tkfiledialog.asksaveasfilename(
             master=self.window,
             title='Save the figure',
             filetypes=tk_filetypes,
@@ -915,12 +824,12 @@ class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
                 rcParams['savefig.directory'] = initialdir
             else:
                 # save dir for next time
-                rcParams['savefig.directory'] = os.path.dirname(unicode(fname))
+                rcParams['savefig.directory'] = os.path.dirname(six.text_type(fname))
             try:
                 # This method will handle the delegation to the correct type
                 self.canvas.print_figure(fname)
             except Exception as e:
-                showerror("Error saving file", str(e))
+                tkinter_messagebox.showerror("Error saving file", str(e))
 
     def set_active(self, ind):
         self._ind = ind
@@ -943,9 +852,6 @@ class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
         self.canvas.draw_idle()
 
 
-FigureManager = FigureManagerTkAgg
-
-
 class ToolTip(object):
     """
     Tooltip recipe from
@@ -995,3 +901,210 @@ class ToolTip(object):
         self.tipwindow = None
         if tw:
             tw.destroy()
+
+
+class RubberbandTk(backend_tools.RubberbandBase):
+    def __init__(self, *args, **kwargs):
+        backend_tools.RubberbandBase.__init__(self, *args, **kwargs)
+
+    def draw_rubberband(self, x0, y0, x1, y1):
+        height = self.figure.canvas.figure.bbox.height
+        y0 = height - y0
+        y1 = height - y1
+        try:
+            self.lastrect
+        except AttributeError:
+            pass
+        else:
+            self.figure.canvas._tkcanvas.delete(self.lastrect)
+        self.lastrect = self.figure.canvas._tkcanvas.create_rectangle(x0, y0, x1, y1)
+
+    def remove_rubberband(self):
+        try:
+            self.lastrect
+        except AttributeError:
+            pass
+        else:
+            self.figure.canvas._tkcanvas.delete(self.lastrect)
+            del self.lastrect
+
+
+class SetCursorTk(backend_tools.SetCursorBase):
+    def set_cursor(self, cursor):
+        self.figure.canvas.manager.window.configure(cursor=cursord[cursor])
+
+
+class ToolbarTk(ToolContainerBase, Tk.Frame):
+    def __init__(self, toolmanager, window):
+        ToolContainerBase.__init__(self, toolmanager)
+        xmin, xmax = self.toolmanager.canvas.figure.bbox.intervalx
+        height, width = 50, xmax - xmin
+        Tk.Frame.__init__(self, master=window,
+                          width=int(width), height=int(height),
+                          borderwidth=2)
+        self._toolitems = {}
+        self.pack(side=Tk.TOP, fill=Tk.X)
+        self._groups = {}
+
+    def add_toolitem(self, name, group, position, image_file, description,
+                     toggle):
+        frame = self._get_groupframe(group)
+        button = self._Button(name, image_file, toggle, frame)
+        if description is not None:
+            ToolTip.createToolTip(button, description)
+        self._toolitems.setdefault(name, [])
+        self._toolitems[name].append(button)
+
+    def _get_groupframe(self, group):
+        if group not in self._groups:
+            if self._groups:
+                self._add_separator()
+            frame = Tk.Frame(master=self, borderwidth=0)
+            frame.pack(side=Tk.LEFT, fill=Tk.Y)
+            self._groups[group] = frame
+        return self._groups[group]
+
+    def _add_separator(self):
+        separator = Tk.Frame(master=self, bd=5, width=1, bg='black')
+        separator.pack(side=Tk.LEFT, fill=Tk.Y, padx=2)
+
+    def _Button(self, text, image_file, toggle, frame):
+        if image_file is not None:
+            im = Tk.PhotoImage(master=self, file=image_file)
+        else:
+            im = None
+
+        if not toggle:
+            b = Tk.Button(master=frame, text=text, padx=2, pady=2, image=im,
+                          command=lambda: self._button_click(text))
+        else:
+            b = Tk.Checkbutton(master=frame, text=text, padx=2, pady=2,
+                               image=im, indicatoron=False,
+                               command=lambda: self._button_click(text))
+        b._ntimage = im
+        b.pack(side=Tk.LEFT)
+        return b
+
+    def _button_click(self, name):
+        self.trigger_tool(name)
+
+    def toggle_toolitem(self, name, toggled):
+        if name not in self._toolitems:
+            return
+        for toolitem in self._toolitems[name]:
+            if toggled:
+                toolitem.select()
+            else:
+                toolitem.deselect()
+
+    def remove_toolitem(self, name):
+        for toolitem in self._toolitems[name]:
+            toolitem.pack_forget()
+        del self._toolitems[name]
+
+
+class StatusbarTk(StatusbarBase, Tk.Frame):
+    def __init__(self, window, *args, **kwargs):
+        StatusbarBase.__init__(self, *args, **kwargs)
+        xmin, xmax = self.toolmanager.canvas.figure.bbox.intervalx
+        height, width = 50, xmax - xmin
+        Tk.Frame.__init__(self, master=window,
+                          width=int(width), height=int(height),
+                          borderwidth=2)
+        self._message = Tk.StringVar(master=self)
+        self._message_label = Tk.Label(master=self, textvariable=self._message)
+        self._message_label.pack(side=Tk.RIGHT)
+        self.pack(side=Tk.TOP, fill=Tk.X)
+
+    def set_message(self, s):
+        self._message.set(s)
+
+
+class SaveFigureTk(backend_tools.SaveFigureBase):
+    def trigger(self, *args):
+        from matplotlib.externals.six.moves import tkinter_tkfiledialog, tkinter_messagebox
+        filetypes = self.figure.canvas.get_supported_filetypes().copy()
+        default_filetype = self.figure.canvas.get_default_filetype()
+
+        # Tk doesn't provide a way to choose a default filetype,
+        # so we just have to put it first
+        default_filetype_name = filetypes[default_filetype]
+        del filetypes[default_filetype]
+
+        sorted_filetypes = list(six.iteritems(filetypes))
+        sorted_filetypes.sort()
+        sorted_filetypes.insert(0, (default_filetype, default_filetype_name))
+
+        tk_filetypes = [
+            (name, '*.%s' % ext) for (ext, name) in sorted_filetypes]
+
+        # adding a default extension seems to break the
+        # asksaveasfilename dialog when you choose various save types
+        # from the dropdown.  Passing in the empty string seems to
+        # work - JDH!
+        # defaultextension = self.figure.canvas.get_default_filetype()
+        defaultextension = ''
+        initialdir = rcParams.get('savefig.directory', '')
+        initialdir = os.path.expanduser(initialdir)
+        initialfile = self.figure.canvas.get_default_filename()
+        fname = tkinter_tkfiledialog.asksaveasfilename(
+            master=self.figure.canvas.manager.window,
+            title='Save the figure',
+            filetypes=tk_filetypes,
+            defaultextension=defaultextension,
+            initialdir=initialdir,
+            initialfile=initialfile,
+            )
+
+        if fname == "" or fname == ():
+            return
+        else:
+            if initialdir == '':
+                # explicitly missing key or empty str signals to use cwd
+                rcParams['savefig.directory'] = initialdir
+            else:
+                # save dir for next time
+                rcParams['savefig.directory'] = os.path.dirname(
+                    six.text_type(fname))
+            try:
+                # This method will handle the delegation to the correct type
+                self.figure.canvas.print_figure(fname)
+            except Exception as e:
+                tkinter_messagebox.showerror("Error saving file", str(e))
+
+
+class ConfigureSubplotsTk(backend_tools.ConfigureSubplotsBase):
+    def __init__(self, *args, **kwargs):
+        backend_tools.ConfigureSubplotsBase.__init__(self, *args, **kwargs)
+        self.window = None
+
+    def trigger(self, *args):
+        self.init_window()
+        self.window.lift()
+
+    def init_window(self):
+        if self.window:
+            return
+
+        toolfig = Figure(figsize=(6, 3))
+        self.window = Tk.Tk()
+
+        canvas = FigureCanvasTkAgg(toolfig, master=self.window)
+        toolfig.subplots_adjust(top=0.9)
+        _tool = SubplotTool(self.figure, toolfig)
+        canvas.show()
+        canvas.get_tk_widget().pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)
+        self.window.protocol("WM_DELETE_WINDOW", self.destroy)
+
+    def destroy(self, *args, **kwargs):
+        self.window.destroy()
+        self.window = None
+
+
+backend_tools.ToolSaveFigure = SaveFigureTk
+backend_tools.ToolConfigureSubplots = ConfigureSubplotsTk
+backend_tools.ToolSetCursor = SetCursorTk
+backend_tools.ToolRubberband = RubberbandTk
+Toolbar = ToolbarTk
+FigureCanvas = FigureCanvasTkAgg
+FigureManager = FigureManagerTkAgg
