@@ -11,7 +11,11 @@ contains all the plot elements.  The following classes are defined
 
 """
 
-from __future__ import print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from matplotlib.externals import six
+
 import warnings
 from operator import itemgetter
 
@@ -47,6 +51,11 @@ from matplotlib.backend_bases import NonGuiException
 docstring.interpd.update(projection_names=get_projection_names())
 
 
+def _stale_figure_callback(self, val):
+    if self.figure:
+        self.figure.stale = val
+
+
 class AxesStack(Stack):
     """
     Specialization of the Stack to handle all tracking of Axes in a Figure.
@@ -147,7 +156,7 @@ class AxesStack(Stack):
         return a in self.as_list()
 
 
-class SubplotParams:
+class SubplotParams(object):
     """
     A class to hold the parameters for a subplot
     """
@@ -294,7 +303,11 @@ class Figure(Artist):
             Defaults to rc ``figure.autolayout``.
         """
         Artist.__init__(self)
-
+        # remove the non-figure artist _axes property
+        # as it makes no sense for a figure to be _in_ an axes
+        # this is used by the property methods in the artist base class
+        # which are over-ridden in this class
+        del self._axes
         self.callbacks = cbook.CallbackRegistry()
 
         if figsize is None:
@@ -322,6 +335,7 @@ class Figure(Artist):
             xy=(0, 0), width=1, height=1,
             facecolor=facecolor, edgecolor=edgecolor,
             linewidth=linewidth)
+
         self._set_artist_props(self.patch)
         self.patch.set_aa(False)
 
@@ -339,6 +353,18 @@ class Figure(Artist):
         self.clf()
         self._cachedRenderer = None
 
+    # TODO: I'd like to dynamically add the _repr_html_ method
+    # to the figure in the right context, but then IPython doesn't
+    # use it, for some reason.
+
+    def _repr_html_(self):
+        # We can't use "isinstance" here, because then we'd end up importing
+        # webagg unconditiionally.
+        if (self.canvas is not None and
+            'WebAgg' in self.canvas.__class__.__name__):
+            from matplotlib.backends import backend_webagg
+            return backend_webagg.ipython_inline_display(self)
+
     def show(self, warn=True):
         """
         If using a GUI backend with pyplot, display the figure window.
@@ -406,6 +432,7 @@ class Figure(Artist):
             tight = rcParams['figure.autolayout']
         self._tight = bool(tight)
         self._tight_parameters = tight if isinstance(tight, dict) else {}
+        self.stale = True
 
     def autofmt_xdate(self, bottom=0.2, rotation=30, ha='right'):
         """
@@ -445,6 +472,7 @@ class Figure(Artist):
 
         if allsubplots:
             self.subplots_adjust(bottom=bottom)
+        self.stale = True
 
     def get_children(self):
         'get a list of artists contained in the figure'
@@ -464,7 +492,7 @@ class Figure(Artist):
 
         Returns True,{}
         """
-        if callable(self._contains):
+        if six.callable(self._contains):
             return self._contains(self, mouseevent)
         # inside = mouseevent.x >= 0 and mouseevent.y >= 0
         inside = self.bbox.contains(mouseevent.x, mouseevent.y)
@@ -502,24 +530,32 @@ class Figure(Artist):
         """
         x = kwargs.pop('x', 0.5)
         y = kwargs.pop('y', 0.98)
+
         if ('horizontalalignment' not in kwargs) and ('ha' not in kwargs):
             kwargs['horizontalalignment'] = 'center'
-
         if ('verticalalignment' not in kwargs) and ('va' not in kwargs):
             kwargs['verticalalignment'] = 'top'
 
+        if 'fontsize' not in kwargs and 'size' not in kwargs:
+            kwargs['size'] = rcParams['figure.titlesize']
+        if 'fontweight' not in kwargs and 'weight' not in kwargs:
+            kwargs['weight'] = rcParams['figure.titleweight']
+
         sup = self.text(x, y, t, **kwargs)
         if self._suptitle is not None:
             self._suptitle.set_text(t)
             self._suptitle.set_position((x, y))
             self._suptitle.update_from(sup)
+            sup.remove()
         else:
             self._suptitle = sup
+
+        self.stale = True
         return self._suptitle
 
     def set_canvas(self, canvas):
         """
-        Set the canvas the contains the figure
+        Set the canvas that contains the figure
 
         ACCEPTS: a FigureCanvas instance
         """
@@ -550,6 +586,7 @@ class Figure(Artist):
                  vmin=None,
                  vmax=None,
                  origin=None,
+                 resize=False,
                  **kwargs):
         """
         Adds a non-resampled image to the figure.
@@ -577,8 +614,10 @@ class Figure(Artist):
           =========   =========================================================
           Keyword     Description
           =========   =========================================================
+          resize      a boolean, True or False. If "True", then re-size the
+                      Figure to match the given image size.
           xo or yo    An integer, the *x* and *y* image offset in pixels
-          cmap        a :class:`matplotlib.colors.Colormap` instance, eg
+          cmap        a :class:`matplotlib.colors.Colormap` instance, e.g.,
                       cm.jet. If *None*, default to the rc ``image.cmap``
                       value
           norm        a :class:`matplotlib.colors.Normalize` instance. The
@@ -611,19 +650,28 @@ class Figure(Artist):
         if not self._hold:
             self.clf()
 
+        if resize:
+            dpi = self.get_dpi()
+            figsize = [x / float(dpi) for x in (X.shape[1], X.shape[0])]
+            self.set_size_inches(figsize, forward=True)
+
         im = FigureImage(self, cmap, norm, xo, yo, origin, **kwargs)
+        im.stale_callback = _stale_figure_callback
+
         im.set_array(X)
         im.set_alpha(alpha)
         if norm is None:
             im.set_clim(vmin, vmax)
         self.images.append(im)
+        im._remove_method = lambda h: self.images.remove(h)
+        self.stale = True
         return im
 
-    def set_size_inches(self, *args, **kwargs):
+    def set_size_inches(self, w, h=None, forward=False):
         """
         set_size_inches(w,h, forward=False)
 
-        Set the figure size in inches
+        Set the figure size in inches (1in == 2.54cm)
 
         Usage::
 
@@ -631,17 +679,21 @@ class Figure(Artist):
              fig.set_size_inches((w,h) )
 
         optional kwarg *forward=True* will cause the canvas size to be
-        automatically updated; eg you can resize the figure window
+        automatically updated; e.g., you can resize the figure window
         from the shell
 
         ACCEPTS: a w,h tuple with w,h in inches
+
+        See Also
+        --------
+
+        matplotlib.Figure.get_size_inches
         """
 
-        forward = kwargs.get('forward', False)
-        if len(args) == 1:
-            w, h = args[0]
-        else:
-            w, h = args
+        # the width and height have been passed in as a tuple to the first
+        # argument, so unpack them
+        if h is None:
+            w, h = w
 
         dpival = self.dpi
         self.bbox_inches.p1 = w, h
@@ -653,9 +705,24 @@ class Figure(Artist):
             manager = getattr(self.canvas, 'manager', None)
             if manager is not None:
                 manager.resize(int(canvasw), int(canvash))
+        self.stale = True
 
     def get_size_inches(self):
-        return self.bbox_inches.p1
+        """
+        Returns the current size of the figure in inches (1in == 2.54cm)
+        as an numpy array.
+
+        Returns
+        -------
+        size : ndarray
+           The size of the figure in inches
+
+        See Also
+        --------
+
+        matplotlib.Figure.set_size_inches
+        """
+        return np.array(self.bbox_inches.p1)
 
     def get_edgecolor(self):
         'Get the edge color of the Figure rectangle'
@@ -704,22 +771,23 @@ class Figure(Artist):
         ACCEPTS: float
         """
         self.dpi = val
+        self.stale = True
 
-    def set_figwidth(self, val):
+    def set_figwidth(self, val, forward=False):
         """
         Set the width of the figure in inches
 
         ACCEPTS: float
         """
-        self.bbox_inches.x1 = val
+        self.set_size_inches(val, self.get_figheight(), forward=forward)
 
-    def set_figheight(self, val):
+    def set_figheight(self, val, forward=False):
         """
         Set the height of the figure in inches
 
         ACCEPTS: float
         """
-        self.bbox_inches.y1 = val
+        self.set_size_inches(self.get_figwidth(), val, forward=forward)
 
     def set_frameon(self, b):
         """
@@ -728,12 +796,14 @@ class Figure(Artist):
         ACCEPTS: boolean
         """
         self.frameon = b
+        self.stale = True
 
     def delaxes(self, a):
         'remove a from the figure and update the current axes'
         self._axstack.remove(a)
         for func in self._axobservers:
             func(self)
+        self.stale = True
 
     def _make_key(self, *args, **kwargs):
         'make a hashable key out of args and kwargs'
@@ -743,8 +813,14 @@ class Figure(Artist):
             # to tuples for the key
             ret = []
             for k, v in items:
-                if iterable(v):
+                # some objects can define __getitem__ without being
+                # iterable and in those cases the conversion to tuples
+                # will fail. So instead of using the iterable(v) function
+                # we simply try and convert to a tuple, and proceed if not.
+                try:
                     v = tuple(v)
+                except Exception:
+                    pass
                 ret.append((k, v))
             return tuple(ret)
 
@@ -756,7 +832,7 @@ class Figure(Artist):
                 ret.append(a)
             return tuple(ret)
 
-        key = fixlist(args), fixitems(kwargs.iteritems())
+        key = fixlist(args), fixitems(six.iteritems(kwargs))
         return key
 
     @docstring.dedent_interpd
@@ -820,7 +896,9 @@ class Figure(Artist):
 
         if isinstance(args[0], Axes):
             a = args[0]
-            assert(a.get_figure() is self)
+            if a.get_figure() is not self:
+                msg = "The Axes must have been created in the present figure"
+                raise ValueError(msg)
         else:
             rect = args[0]
             projection_class, kwargs, key = process_projection_requirements(
@@ -838,6 +916,9 @@ class Figure(Artist):
 
         self._axstack.add(key, a)
         self.sca(a)
+        a._remove_method = self.__remove_ax
+        self.stale = True
+        a.stale_callback = _stale_figure_callback
         return a
 
     @docstring.dedent_interpd
@@ -874,6 +955,9 @@ class Figure(Artist):
         *kwargs*) then it will simply make that subplot current and
         return it.
 
+        .. seealso:: :meth:`~matplotlib.pyplot.subplot` for an
+           explanation of the args.
+
         The following kwargs are supported:
 
         %(Axes)s
@@ -883,11 +967,18 @@ class Figure(Artist):
 
         if len(args) == 1 and isinstance(args[0], int):
             args = tuple([int(c) for c in str(args[0])])
+            if len(args) != 3:
+                raise ValueError("Integer subplot specification must " +
+                                 "be a three digit number.  " +
+                                 "Not {n:d}".format(n=len(args)))
 
         if isinstance(args[0], SubplotBase):
 
             a = args[0]
-            assert(a.get_figure() is self)
+            if a.get_figure() is not self:
+                msg = ("The Subplot must have been created in the present"
+                       " figure")
+                raise ValueError(msg)
             # make a key for the subplot (which includes the axes object id
             # in the hash)
             key = self._make_key(*args, **kwargs)
@@ -915,8 +1006,37 @@ class Figure(Artist):
 
         self._axstack.add(key, a)
         self.sca(a)
+        a._remove_method = self.__remove_ax
+        self.stale = True
+        a.stale_callback = _stale_figure_callback
         return a
 
+    def __remove_ax(self, ax):
+        def _reset_loc_form(axis):
+            axis.set_major_formatter(axis.get_major_formatter())
+            axis.set_major_locator(axis.get_major_locator())
+            axis.set_minor_formatter(axis.get_minor_formatter())
+            axis.set_minor_locator(axis.get_minor_locator())
+
+        def _break_share_link(ax, grouper):
+            siblings = grouper.get_siblings(ax)
+            if len(siblings) > 1:
+                grouper.remove(ax)
+                for last_ax in siblings:
+                    if ax is last_ax:
+                        continue
+                    return last_ax
+            return None
+
+        self.delaxes(ax)
+        last_ax = _break_share_link(ax, ax._shared_y_axes)
+        if last_ax is not None:
+            _reset_loc_form(last_ax.yaxis)
+
+        last_ax = _break_share_link(ax, ax._shared_x_axes)
+        if last_ax is not None:
+            _reset_loc_form(last_ax.xaxis)
+
     def clf(self, keep_observers=False):
         """
         Clear the figure.
@@ -943,6 +1063,8 @@ class Figure(Artist):
         self.legends = []
         if not keep_observers:
             self._axobservers = []
+        self._suptitle = None
+        self.stale = True
 
     def clear(self):
         """
@@ -956,11 +1078,14 @@ class Figure(Artist):
         Render the figure using :class:`matplotlib.backend_bases.RendererBase`
         instance *renderer*.
         """
+
         # draw the figure bounding box, perhaps none for white figure
         if not self.get_visible():
             return
-        renderer.open_group('figure')
 
+        renderer.open_group('figure')
+        # prevent triggering call backs during the draw process
+        self._stale = True
         if self.get_tight_layout() and self.axes:
             try:
                 self.tight_layout(renderer, **self._tight_parameters)
@@ -1000,8 +1125,8 @@ class Figure(Artist):
             ims = [(im.make_image(mag), im.ox, im.oy, im.get_alpha())
                    for im in self.images]
 
-            im = _image.from_images(self.bbox.height * mag,
-                                    self.bbox.width * mag,
+            im = _image.from_images(int(self.bbox.height * mag),
+                                    int(self.bbox.width * mag),
                                     ims)
 
             im.is_grayscale = False
@@ -1034,9 +1159,9 @@ class Figure(Artist):
             func(*args)
 
         renderer.close_group('figure')
+        self.stale = False
 
         self._cachedRenderer = renderer
-
         self.canvas.draw_event(renderer)
 
     def draw_artist(self, a):
@@ -1044,7 +1169,10 @@ class Figure(Artist):
         draw :class:`matplotlib.artist.Artist` instance *a* only --
         this is available only after the figure is drawn
         """
-        assert self._cachedRenderer is not None
+        if self._cachedRenderer is None:
+            msg = ('draw_artist can only be used after an initial draw which'
+                   ' caches the render')
+            raise AttributeError(msg)
         a.draw(self._cachedRenderer)
 
     def get_axes(self):
@@ -1103,6 +1231,11 @@ class Figure(Artist):
             The relative size of legend markers vs. original. If *None*, use rc
             settings.
 
+          *markerfirst*: [ *True* | *False* ]
+            if *True*, legend marker is placed to the left of the legend label
+            if *False*, legend marker is placed to the right of the legend
+            label
+
           *fancybox*: [ *None* | *False* | *True* ]
             if *True*, draw a frame with a round fancybox.  If *None*, use rc
 
@@ -1143,6 +1276,8 @@ class Figure(Artist):
         """
         l = Legend(self, handles, labels, *args, **kwargs)
         self.legends.append(l)
+        l._remove_method = lambda h: self.legends.remove(h)
+        self.stale = True
         return l
 
     @docstring.dedent_interpd
@@ -1169,17 +1304,20 @@ class Figure(Artist):
         t.update(override)
         self._set_artist_props(t)
         self.texts.append(t)
+        t._remove_method = lambda h: self.texts.remove(h)
+        self.stale = True
         return t
 
     def _set_artist_props(self, a):
         if a != self:
             a.set_figure(self)
+        a.stale_callback = _stale_figure_callback
         a.set_transform(self.transFigure)
 
     @docstring.dedent_interpd
     def gca(self, **kwargs):
         """
-        Return the current axes, creating one if necessary
+        Get the current axes, creating one if necessary
 
         The following kwargs are supported for ensuring the returned axes
         adheres to the given projection etc., and for axes creation if
@@ -1250,7 +1388,7 @@ class Figure(Artist):
         return None
 
     def __getstate__(self):
-        state = self.__dict__.copy()
+        state = super(Figure, self).__getstate__()
         # the axobservers cannot currently be pickled.
         # Additionally, the canvas cannot currently be pickled, but this has
         # the benefit of meaning that a figure can be detached from one canvas,
@@ -1267,7 +1405,8 @@ class Figure(Artist):
         if getattr(self.canvas, 'manager', None) is not None:
             manager = self.canvas.manager
             import matplotlib._pylab_helpers
-            if manager in matplotlib._pylab_helpers.Gcf.figs.values():
+            if manager in list(six.itervalues(
+                    matplotlib._pylab_helpers.Gcf.figs)):
                 state['_restore_to_pylab'] = True
 
         return state
@@ -1313,6 +1452,7 @@ class Figure(Artist):
             self.number = num
 
             plt.draw_if_interactive()
+        self.stale = True
 
     def add_axobserver(self, func):
         'whenever the axes state change, ``func(self)`` will be called'
@@ -1348,9 +1488,10 @@ class Figure(Artist):
 
         Keyword arguments:
 
-          *dpi*: [ *None* | ``scalar > 0`` ]
+          *dpi*: [ *None* | ``scalar > 0`` | 'figure']
             The resolution in dots per inch.  If *None* it will default to
-            the value ``savefig.dpi`` in the matplotlibrc file.
+            the value ``savefig.dpi`` in the matplotlibrc file. If 'figure'
+            it will set the dpi to be the value of the figure.
 
           *facecolor*, *edgecolor*:
             the colors of the figure rectangle
@@ -1397,8 +1538,11 @@ class Figure(Artist):
         """
 
         kwargs.setdefault('dpi', rcParams['savefig.dpi'])
+        if kwargs['dpi'] == 'figure':
+            kwargs['dpi'] = self.get_dpi()
         frameon = kwargs.pop('frameon', rcParams['savefig.frameon'])
-        transparent = kwargs.pop('transparent', False)
+        transparent = kwargs.pop('transparent',
+                                 rcParams['savefig.transparent'])
 
         if transparent:
             kwargs.setdefault('facecolor', 'none')
@@ -1451,6 +1595,7 @@ class Figure(Artist):
         cb = cbar.colorbar_factory(cax, mappable, **kw)
 
         self.sca(current_ax)
+        self.stale = True
         return cb
 
     def subplots_adjust(self, *args, **kwargs):
@@ -1479,6 +1624,7 @@ class Figure(Artist):
             else:
                 ax.update_params()
                 ax.set_position(ax.figbox)
+        self.stale = True
 
     def ginput(self, n=1, timeout=30, show_clicks=True, mouse_add=1,
                mouse_pop=3, mouse_stop=2):
@@ -1561,6 +1707,9 @@ class Figure(Artist):
             if ax.get_visible():
                 bb.append(ax.get_tightbbox(renderer))
 
+        if len(bb) == 0:
+            return self.bbox_inches
+
         _bbox = Bbox.union([b for b in bb if b.width != 0 or b.height != 0])
 
         bbox_inches = TransformedBbox(_bbox,
@@ -1587,8 +1736,8 @@ class Figure(Artist):
             labels) will fit into. Default is (0, 0, 1, 1).
         """
 
-        from tight_layout import (get_renderer, get_tight_layout_figure,
-                                  get_subplotspec_list)
+        from .tight_layout import (get_renderer, get_tight_layout_figure,
+                                   get_subplotspec_list)
 
         subplotspec_list = get_subplotspec_list(self.axes)
         if None in subplotspec_list:
@@ -1614,7 +1763,7 @@ def figaspect(arg):
     determine the width and height for a figure that would fit array
     preserving aspect ratio.  The figure width, height in inches are
     returned.  Be sure to create an axes with equal with and height,
-    eg
+    e.g.,
 
     Example usage::
 
@@ -1635,7 +1784,7 @@ def figaspect(arg):
     Thanks to Fernando Perez for this function
     """
 
-    isarray = hasattr(arg, 'shape')
+    isarray = hasattr(arg, 'shape') and not np.isscalar(arg)
 
     # min/max sizes to respect when autoscaling.  If John likes the idea, they
     # could become rc parameters, for now they're hardwired.
