@@ -2,11 +2,11 @@
 """
 Module for creating Sankey diagrams using matplotlib
 """
-from __future__ import print_function
-__author__ = "Kevin L. Davies"
-__credits__ = ["Yannick Copin"]
-__license__ = "BSD"
-__version__ = "2011/09/16"
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from matplotlib.externals import six
+from matplotlib.externals.six.moves import zip
 
 # Original version by Yannick Copin (ycopin@ipnl.in2p3.fr) 10/2/2010, available
 # at:
@@ -29,7 +29,8 @@ __version__ = "2011/09/16"
 #     argument specifies the direction of the arrows.  The "main"
 #     inputs/outputs are now specified via an orientation of 0, and there may
 #     be several of each.
-#   --Added assertions to catch common calling errors
+#   --Changed assertions to ValueError to catch common calling errors (by
+#     Francesco Montesano, franz.bergesung@gmail.com)
 #   --Added the physical unit as a string argument to be used in the labels, so
 #     that the values of the flows can usually be applied automatically
 #   --Added an argument for a minimum magnitude below which flows are not shown
@@ -45,6 +46,11 @@ from matplotlib.transforms import Affine2D
 from matplotlib import verbose
 from matplotlib import docstring
 
+__author__ = "Kevin L. Davies"
+__credits__ = ["Yannick Copin"]
+__license__ = "BSD"
+__version__ = "2011/09/16"
+
 # Angles [deg/90]
 RIGHT = 0
 UP = 1
@@ -52,7 +58,7 @@ UP = 1
 DOWN = 3
 
 
-class Sankey:
+class Sankey(object):
     """
     Sankey diagram in matplotlib
 
@@ -145,17 +151,21 @@ class Sankey:
             .. plot:: mpl_examples/api/sankey_demo_basics.py
         """
         # Check the arguments.
-        assert gap >= 0, (
+        if gap < 0:
+            raise ValueError(
             "The gap is negative.\nThis isn't allowed because it "
             "would cause the paths to overlap.")
-        assert radius <= gap, (
+        if radius > gap:
+            raise ValueError(
             "The inner radius is greater than the path spacing.\n"
             "This isn't allowed because it would cause the paths to overlap.")
-        assert head_angle >= 0, (
+        if head_angle < 0:
+            raise ValueError(
             "The angle is negative.\nThis isn't allowed "
             "because it would cause inputs to look like "
             "outputs and vice versa.")
-        assert tolerance >= 0, (
+        if tolerance < 0:
+            raise ValueError(
             "The tolerance is negative.\nIt must be a magnitude.")
 
         # Create axes if necessary.
@@ -220,7 +230,7 @@ class Sankey:
                                  [5.19642327e-01, 8.94571235e-01],
                                  [2.65114773e-01, 1.00000000e+00],
                                  # Insignificant
-                                 #[6.12303177e-17, 1.00000000e+00]])
+                                 # [6.12303177e-17, 1.00000000e+00]])
                                  [0.00000000e+00, 1.00000000e+00]])
         if quadrant == 0 or quadrant == 2:
             if cw:
@@ -238,8 +248,8 @@ class Sankey:
                                              ARC_VERTICES[:, 1]))
         if quadrant > 1:
             radius = -radius  # Rotate 180 deg.
-        return zip(ARC_CODES, radius * vertices +
-                              np.tile(center, (ARC_VERTICES.shape[0], 1)))
+        return list(zip(ARC_CODES, radius * vertices +
+                        np.tile(center, (ARC_VERTICES.shape[0], 1))))
 
     def _add_input(self, path, angle, flow, length):
         """
@@ -364,10 +374,10 @@ class Sankey:
         return reverse_path
         # This might be more efficient, but it fails because 'tuple' object
         # doesn't support item assignment:
-        #path[1] = path[1][-1:0:-1]
-        #path[1][0] = first_action
-        #path[2] = path[2][::-1]
-        #return path
+        # path[1] = path[1][-1:0:-1]
+        # path[1][0] = first_action
+        # path[2] = path[2][::-1]
+        # return path
 
     @docstring.dedent_interpd
     def add(self, patchlabel='', flows=None, orientations=None, labels='',
@@ -444,7 +454,7 @@ class Sankey:
         placed along the sides of the diagram from the top down and along the
         bottom from the outside in.
 
-        If the the sum of the inputs and outputs is nonzero, the discrepancy
+        If the sum of the inputs and outputs is nonzero, the discrepancy
         will appear as a cubic Bezier curve along the top and bottom edges of
         the trunk.
 
@@ -465,20 +475,23 @@ class Sankey:
             rotation /= 90.0
         if orientations is None:
             orientations = [0, 0]
-        assert len(orientations) == n, (
+        if len(orientations) != n:
+            raise ValueError(
             "orientations and flows must have the same length.\n"
             "orientations has length %d, but flows has length %d."
             % (len(orientations), n))
         if labels != '' and getattr(labels, '__iter__', False):
-        # iterable() isn't used because it would give True if labels is a
-        # string
-            assert len(labels) == n, (
+            # iterable() isn't used because it would give True if labels is a
+            # string
+            if len(labels) != n:
+                raise ValueError(
                 "If labels is a list, then labels and flows must have the "
                 "same length.\nlabels has length %d, but flows has length %d."
                 % (len(labels), n))
         else:
             labels = [labels] * n
-        assert trunklength >= 0, (
+        if trunklength < 0:
+            raise ValueError(
             "trunklength is negative.\nThis isn't allowed, because it would "
             "cause poor layout.")
         if np.absolute(np.sum(flows)) > self.tolerance:
@@ -499,28 +512,35 @@ class Sankey:
                 "cause poor layout.\nConsider changing the scale so"
                 " that the scaled sum is approximately 1.0." % gain, 'helpful')
         if prior is not None:
-            assert prior >= 0, "The index of the prior diagram is negative."
-            assert min(connect) >= 0, (
+            if prior < 0:
+                raise ValueError("The index of the prior diagram is negative.")
+            if min(connect) < 0:
+                raise ValueError(
                 "At least one of the connection indices is negative.")
-            assert prior < len(self.diagrams), (
+            if prior >= len(self.diagrams):
+                raise ValueError(
                 "The index of the prior diagram is %d, but there are "
                 "only %d other diagrams.\nThe index is zero-based."
                 % (prior, len(self.diagrams)))
-            assert connect[0] < len(self.diagrams[prior].flows), (
+            if connect[0] >= len(self.diagrams[prior].flows):
+                raise ValueError(
                 "The connection index to the source diagram is %d, but "
                 "that diagram has only %d flows.\nThe index is zero-based."
                 % (connect[0], len(self.diagrams[prior].flows)))
-            assert connect[1] < n, (
+            if connect[1] >= n:
+                raise ValueError(
                 "The connection index to this diagram is %d, but this diagram"
                 "has only %d flows.\n The index is zero-based."
                 % (connect[1], n))
-            assert self.diagrams[prior].angles[connect[0]] is not None, (
+            if self.diagrams[prior].angles[connect[0]] is None:
+                raise ValueError(
                 "The connection cannot be made.  Check that the magnitude "
                 "of flow %d of diagram %d is greater than or equal to the "
                 "specified tolerance." % (connect[0], prior))
             flow_error = (self.diagrams[prior].flows[connect[0]] +
                           flows[connect[1]])
-            assert abs(flow_error) < self.tolerance, (
+            if abs(flow_error) >= self.tolerance:
+                raise ValueError(
                 "The scaled sum of the connected flows is %f, which is not "
                 "within the tolerance (%f)." % (flow_error, self.tolerance))
 
@@ -551,9 +571,10 @@ class Sankey:
                 if is_input is not None:
                     angles[i] = RIGHT
             else:
-                assert orient == -1, (
+                if orient != -1:
+                    raise ValueError(
                     "The value of orientations[%d] is %d, "
-                    "but it must be -1, 0, or 1." % (i, orient))
+                    "but it must be [ -1 | 0 | 1 ]." % (i, orient))
                 if is_input:
                     angles[i] = UP
                 elif not is_input:
@@ -561,7 +582,8 @@ class Sankey:
 
         # Justify the lengths of the paths.
         if iterable(pathlengths):
-            assert len(pathlengths) == n, (
+            if len(pathlengths) != n:
+                raise ValueError(
                 "If pathlengths is a list, then pathlengths and flows must "
                 "have the same length.\npathlengths has length %d, but flows "
                 "has length %d." % (len(pathlengths), n))
@@ -584,8 +606,8 @@ class Sankey:
                     urlength -= flow  # Flow is negative for outputs.
             # Determine the lengths of the bottom-side arrows
             # from the middle outwards.
-            for i, (angle, is_input, flow) in enumerate(reversed(zip(
-                angles, are_inputs, scaled_flows))):
+            for i, (angle, is_input, flow) in enumerate(reversed(list(zip(
+                  angles, are_inputs, scaled_flows)))):
                 if angle == UP and is_input:
                     pathlengths[n - i - 1] = lllength
                     lllength += flow
@@ -595,8 +617,8 @@ class Sankey:
             # Determine the lengths of the left-side arrows
             # from the bottom upwards.
             has_left_input = False
-            for i, (angle, is_input, spec) in enumerate(reversed(zip(
-                angles, are_inputs, zip(scaled_flows, pathlengths)))):
+            for i, (angle, is_input, spec) in enumerate(reversed(list(zip(
+                  angles, are_inputs, zip(scaled_flows, pathlengths))))):
                 if angle == RIGHT:
                     if is_input:
                         if has_left_input:
@@ -607,7 +629,7 @@ class Sankey:
             # from the top downwards.
             has_right_output = False
             for i, (angle, is_input, spec) in enumerate(zip(
-                angles, are_inputs, zip(scaled_flows, pathlengths))):
+                  angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
                 if angle == RIGHT:
                     if not is_input:
                         if has_right_output:
@@ -651,7 +673,7 @@ class Sankey:
         label_locations = np.zeros((n, 2))
         # Add the top-side inputs and outputs from the middle outwards.
         for i, (angle, is_input, spec) in enumerate(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths))):
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
             if angle == DOWN and is_input:
                 tips[i, :], label_locations[i, :] = self._add_input(
                     ulpath, angle, *spec)
@@ -659,8 +681,8 @@ class Sankey:
                 tips[i, :], label_locations[i, :] = self._add_output(
                     urpath, angle, *spec)
         # Add the bottom-side inputs and outputs from the middle outwards.
-        for i, (angle, is_input, spec) in enumerate(reversed(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths)))):
+        for i, (angle, is_input, spec) in enumerate(reversed(list(zip(
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))))):
             if angle == UP and is_input:
                 tip, label_location = self._add_input(llpath, angle, *spec)
                 tips[n - i - 1, :] = tip
@@ -671,8 +693,8 @@ class Sankey:
                 label_locations[n - i - 1, :] = label_location
         # Add the left-side inputs from the bottom upwards.
         has_left_input = False
-        for i, (angle, is_input, spec) in enumerate(reversed(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths)))):
+        for i, (angle, is_input, spec) in enumerate(reversed(list(zip(
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))))):
             if angle == RIGHT and is_input:
                 if not has_left_input:
                     # Make sure the lower path extends
@@ -687,7 +709,7 @@ class Sankey:
         # Add the right-side outputs from the top downwards.
         has_right_output = False
         for i, (angle, is_input, spec) in enumerate(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths))):
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
             if angle == RIGHT and not is_input:
                 if not has_right_output:
                     # Make sure the upper path extends
@@ -711,7 +733,7 @@ class Sankey:
                 [(Path.CLOSEPOLY, urpath[0][1])])
 
         # Create a patch with the Sankey outline.
-        codes, vertices = zip(*path)
+        codes, vertices = list(zip(*path))
         vertices = np.array(vertices)
 
         def _get_angle(a, r):
@@ -723,7 +745,7 @@ class Sankey:
         if prior is None:
             if rotation != 0:  # By default, none of this is needed.
                 angles = [_get_angle(angle, rotation) for angle in angles]
-                rotate = Affine2D().rotate_deg(rotation * 90).transform_point
+                rotate = Affine2D().rotate_deg(rotation * 90).transform_affine
                 tips = rotate(tips)
                 label_locations = rotate(label_locations)
                 vertices = rotate(vertices)
@@ -732,10 +754,10 @@ class Sankey:
             rotation = (self.diagrams[prior].angles[connect[0]] -
                         angles[connect[1]])
             angles = [_get_angle(angle, rotation) for angle in angles]
-            rotate = Affine2D().rotate_deg(rotation * 90).transform_point
+            rotate = Affine2D().rotate_deg(rotation * 90).transform_affine
             tips = rotate(tips)
             offset = self.diagrams[prior].tips[connect[0]] - tips[connect[1]]
-            translate = Affine2D().translate(*offset).transform_point
+            translate = Affine2D().translate(*offset).transform_affine
             tips = translate(tips)
             label_locations = translate(rotate(label_locations))
             vertices = translate(rotate(vertices))
@@ -746,7 +768,7 @@ class Sankey:
             print("ulpath\n", self._revert(ulpath))
             print("urpath\n", urpath)
             print("lrpath\n", self._revert(lrpath))
-            xs, ys = zip(*vertices)
+            xs, ys = list(zip(*vertices))
             self.ax.plot(xs, ys, 'go-')
         patch = PathPatch(Path(vertices, codes),
                           fc=kwargs.pop('fc', kwargs.pop('facecolor',
