@@ -1,18 +1,22 @@
-from __future__ import print_function
-
-import copy
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import warnings
+from matplotlib.externals import six
 
+import io
 
 import numpy as np
 
-
 import matplotlib.pyplot as plt
 import matplotlib.patches as mpatches
+import matplotlib.lines as mlines
 import matplotlib.path as mpath
 import matplotlib.transforms as mtrans
 import matplotlib.collections as mcollections
 from matplotlib.testing.decorators import image_comparison, cleanup
 
+from nose.tools import (assert_true, assert_false)
+
 
 @cleanup
 def test_patch_transform_of_none():
@@ -93,16 +97,6 @@ def test_collection_transform_of_none():
     assert isinstance(c._transOffset, mtrans.IdentityTransform)
 
 
-def test_point_in_path():
-    # Test #1787
-    verts2 = [(0,0), (0,1), (1,1), (1,0), (0,0)]
-
-    path = mpath.Path(verts2, closed=True)
-    points = [(0.5,0.5), (1.5,0.5)]
-
-    assert np.all(path.contains_points(points) == [True, False])
-
-
 @image_comparison(baseline_images=["clip_path_clipping"], remove_text=True)
 def test_clipping():
     exterior = mpath.Path.unit_rectangle().deepcopy()
@@ -134,6 +128,65 @@ def test_clipping():
     ax1.set_ylim([-3, 3])
 
 
-if __name__=='__main__':
+@cleanup
+def test_cull_markers():
+    x = np.random.random(20000)
+    y = np.random.random(20000)
+
+    fig = plt.figure()
+    ax = fig.add_subplot(111)
+    ax.plot(x, y, 'k.')
+    ax.set_xlim(2, 3)
+
+    pdf = io.BytesIO()
+    fig.savefig(pdf, format="pdf")
+    assert len(pdf.getvalue()) < 8000
+
+    svg = io.BytesIO()
+    fig.savefig(svg, format="svg")
+    assert len(svg.getvalue()) < 20000
+
+
+@cleanup
+def test_remove():
+    fig, ax = plt.subplots()
+    im = ax.imshow(np.arange(36).reshape(6, 6))
+    ln, = ax.plot(range(5))
+
+    assert_true(fig.stale)
+    assert_true(ax.stale)
+
+    fig.canvas.draw()
+    assert_false(fig.stale)
+    assert_false(ax.stale)
+    assert_false(ln.stale)
+
+    assert_true(im in ax.mouseover_set)
+    assert_true(ln not in ax.mouseover_set)
+    assert_true(im.axes is ax)
+
+    im.remove()
+    ln.remove()
+
+    for art in [im, ln]:
+        assert_true(art.axes is None)
+        assert_true(art.figure is None)
+
+    assert_true(im not in ax.mouseover_set)
+    assert_true(fig.stale)
+    assert_true(ax.stale)
+
+
+@cleanup
+def test_properties():
+    ln = mlines.Line2D([], [])
+    with warnings.catch_warnings(record=True) as w:
+        # Cause all warnings to always be triggered.
+        warnings.simplefilter("always")
+        ln.properties()
+        assert len(w) == 0
+
+
+if __name__ == '__main__':
     import nose
-    nose.runmodule(argv=['-s','--with-doctest'], exit=False)
+    nose.runmodule(argv=['-s', '--with-doctest'], exit=False)
