@@ -3,14 +3,14 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import six
+from matplotlib.externals import six
 
 import io
 import os
 
 import numpy as np
-
 from matplotlib import cm, rcParams
+from matplotlib.backends.backend_pdf import PdfPages
 from matplotlib import pyplot as plt
 from matplotlib.testing.decorators import (image_comparison, knownfailureif,
                                            cleanup)
@@ -42,7 +42,6 @@ def test_type42():
 
 @cleanup
 def test_multipage_pagecount():
-    from matplotlib.backends.backend_pdf import PdfPages
     with PdfPages(io.BytesIO()) as pdf:
         assert pdf.get_pagecount() == 0
         fig = plt.figure()
@@ -58,7 +57,7 @@ def test_multipage_pagecount():
 def test_multipage_keep_empty():
     from matplotlib.backends.backend_pdf import PdfPages
     from tempfile import NamedTemporaryFile
-    ### test empty pdf files
+    # test empty pdf files
     # test that an empty pdf is left behind with keep_empty=True (default)
     with NamedTemporaryFile(delete=False) as tmp:
         with PdfPages(tmp) as pdf:
@@ -69,7 +68,7 @@ def test_multipage_keep_empty():
     with PdfPages(filename, keep_empty=False) as pdf:
         pass
     assert not os.path.exists(filename)
-    ### test pdf files with content, they should never be deleted
+    # test pdf files with content, they should never be deleted
     fig = plt.figure()
     ax = fig.add_subplot(111)
     ax.plot([1, 2, 3])
@@ -87,3 +86,49 @@ def test_multipage_keep_empty():
             pdf.savefig()
         assert os.path.exists(filename)
     os.remove(filename)
+
+
+@cleanup
+def test_composite_image():
+    #Test that figures can be saved with and without combining multiple images
+    #(on a single set of axes) into a single composite image.
+    X, Y = np.meshgrid(np.arange(-5, 5, 1), np.arange(-5, 5, 1))
+    Z = np.sin(Y ** 2)
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1)
+    ax.set_xlim(0, 3)
+    ax.imshow(Z, extent=[0, 1, 0, 1])
+    ax.imshow(Z[::-1], extent=[2, 3, 0, 1])
+    plt.rcParams['image.composite_image'] = True
+    with PdfPages(io.BytesIO()) as pdf:
+        fig.savefig(pdf, format="pdf")
+        assert len(pdf._file.images.keys()) == 1
+    plt.rcParams['image.composite_image'] = False
+    with PdfPages(io.BytesIO()) as pdf:
+        fig.savefig(pdf, format="pdf")
+        assert len(pdf._file.images.keys()) == 2
+
+
+@image_comparison(baseline_images=['hatching_legend'],
+                  extensions=['pdf'])
+def test_hatching_legend():
+    """Test for correct hatching on patches in legend"""
+    fig = plt.figure(figsize=(1, 2))
+
+    a = plt.Rectangle([0, 0], 0, 0, facecolor="green", hatch="XXXX")
+    b = plt.Rectangle([0, 0], 0, 0, facecolor="blue", hatch="XXXX")
+
+    fig.legend([a, b, a, b], ["", "", "", ""])
+
+
+@image_comparison(baseline_images=['grayscale_alpha'],
+                  extensions=['pdf'], tol=1e-3)
+def test_grayscale_alpha():
+    """Masking images with NaN did not work for grayscale images"""
+    x, y = np.ogrid[-2:2:.1, -2:2:.1]
+    dd = np.exp(-(x**2 + y**2))
+    dd[dd < .1] = np.nan
+    fig, ax = plt.subplots()
+    ax.imshow(dd, interpolation='none', cmap='gray_r')
+    ax.set_xticks([])
+    ax.set_yticks([])
