@@ -1,9 +1,9 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import six
-from six.moves import cPickle as pickle
-from six.moves import xrange
+from matplotlib.externals import six
+from matplotlib.externals.six.moves import cPickle as pickle
+from matplotlib.externals.six.moves import xrange
 
 from io import BytesIO
 
@@ -12,6 +12,7 @@ import numpy as np
 
 from matplotlib.testing.decorators import cleanup, image_comparison
 import matplotlib.pyplot as plt
+import matplotlib.transforms as mtransforms
 
 
 def depth_getter(obj,
@@ -83,7 +84,6 @@ def recursive_pickle(top_obj):
     objs = sorted(six.itervalues(objs), key=lambda val: (-val[0], val[2]))
 
     for _, obj, location in objs:
-#        print('trying %s' % location)
         try:
             pickle.dump(obj, BytesIO(), pickle.HIGHEST_PROTOCOL)
         except Exception as err:
@@ -107,6 +107,8 @@ def test_simple():
     plt.plot(list(xrange(10)), label='foobar')
     plt.legend()
 
+    # Uncomment to debug any unpicklable objects. This is slow so is not
+    # uncommented by default.
 #    recursive_pickle(fig)
     pickle.dump(ax, BytesIO(), pickle.HIGHEST_PROTOCOL)
 
@@ -125,6 +127,7 @@ def test_simple():
     pickle.dump(fig, BytesIO(), pickle.HIGHEST_PROTOCOL)
 
 
+@cleanup
 @image_comparison(baseline_images=['multi_pickle'],
                   extensions=['png'], remove_text=True)
 def test_complete():
@@ -193,19 +196,94 @@ def test_complete():
     assert_equal(fig.get_label(), 'Figure with a label?')
 
 
+@cleanup
 def test_no_pyplot():
     # tests pickle-ability of a figure not created with pyplot
-
-    import pickle as p
     from matplotlib.backends.backend_pdf import FigureCanvasPdf as fc
     from matplotlib.figure import Figure
 
     fig = Figure()
-    can = fc(fig)
+    _ = fc(fig)
     ax = fig.add_subplot(1, 1, 1)
     ax.plot([1, 2, 3], [1, 2, 3])
-
-    # Uncomment to debug any unpicklable objects. This is slow so is not
-    # uncommented by default.
-#    recursive_pickle(fig)
     pickle.dump(fig, BytesIO(), pickle.HIGHEST_PROTOCOL)
+
+
+@cleanup
+def test_renderer():
+    from matplotlib.backends.backend_agg import RendererAgg
+    renderer = RendererAgg(10, 20, 30)
+    pickle.dump(renderer, BytesIO())
+
+
+@cleanup
+def test_image():
+    # Prior to v1.4.0 the Image would cache data which was not picklable
+    # once it had been drawn.
+    from matplotlib.backends.backend_agg import new_figure_manager
+    manager = new_figure_manager(1000)
+    fig = manager.canvas.figure
+    ax = fig.add_subplot(1, 1, 1)
+    ax.imshow(np.arange(12).reshape(3, 4))
+    manager.canvas.draw()
+    pickle.dump(fig, BytesIO())
+
+
+@cleanup
+def test_grid():
+    from matplotlib.backends.backend_agg import new_figure_manager
+    manager = new_figure_manager(1000)
+    fig = manager.canvas.figure
+    ax = fig.add_subplot(1, 1, 1)
+    ax.grid()
+    # Drawing the grid triggers instance methods to be attached
+    # to the Line2D object (_lineFunc).
+    manager.canvas.draw()
+
+    pickle.dump(ax, BytesIO())
+
+
+@cleanup
+def test_polar():
+    ax = plt.subplot(111, polar=True)
+    fig = plt.gcf()
+    result = BytesIO()
+    pf = pickle.dumps(fig)
+    pickle.loads(pf)
+    plt.draw()
+
+
+class TransformBlob(object):
+    def __init__(self):
+        self.identity = mtransforms.IdentityTransform()
+        self.identity2 = mtransforms.IdentityTransform()
+        # Force use of the more complex composition.
+        self.composite = mtransforms.CompositeGenericTransform(
+            self.identity,
+            self.identity2)
+        # Check parent -> child links of TransformWrapper.
+        self.wrapper = mtransforms.TransformWrapper(self.composite)
+        # Check child -> parent links of TransformWrapper.
+        self.composite2 = mtransforms.CompositeGenericTransform(
+            self.wrapper,
+            self.identity)
+
+
+def test_transform():
+    obj = TransformBlob()
+    pf = pickle.dumps(obj)
+    del obj
+
+    obj = pickle.loads(pf)
+    # Check parent -> child links of TransformWrapper.
+    assert_equal(obj.wrapper._child, obj.composite)
+    # Check child -> parent links of TransformWrapper.
+    assert_equal(list(obj.wrapper._parents.values()), [obj.composite2])
+    # Check input and output dimensions are set as expected.
+    assert_equal(obj.wrapper.input_dims, obj.composite.input_dims)
+    assert_equal(obj.wrapper.output_dims, obj.composite.output_dims)
+
+
+if __name__ == '__main__':
+    import nose
+    nose.runmodule(argv=['-s'])
