@@ -1,3 +1,8 @@
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from matplotlib.externals import six
+
 import matplotlib
 from matplotlib.font_manager import FontProperties
 from matplotlib.testing.decorators import image_comparison
