@@ -1,9 +1,14 @@
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from matplotlib.externals import six
+
 from mpl_toolkits.axes_grid1.parasite_axes import \
      subplot_class_factory, \
      parasite_axes_class_factory, parasite_axes_auxtrans_class_factory, \
      host_axes_class_factory
 
-from axislines import Axes
+from .axislines import Axes
 
 
 ParasiteAxes = parasite_axes_class_factory(Axes)
@@ -13,4 +18,3 @@ ParasiteAxesAuxTrans = parasite_axes_auxtrans_class_factory(axes_class=ParasiteA
 HostAxes = host_axes_class_factory(axes_class=Axes)
 
 SubplotHost = subplot_class_factory(HostAxes)
-
