@@ -1,3 +1,7 @@
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from matplotlib.externals import six
 
 from matplotlib.patches import Rectangle, Ellipse
 
@@ -20,7 +24,7 @@ class AnchoredDrawingArea(AnchoredOffsetbox):
         *prop* : font property. This is only used for scaling the paddings.
         """
 
-        self.da = DrawingArea(width, height, xdescent, ydescent, clip=True)
+        self.da = DrawingArea(width, height, xdescent, ydescent)
         self.drawing_area = self.da
 
         super(AnchoredDrawingArea, self).__init__(loc, pad=pad, borderpad=borderpad,
@@ -62,30 +66,96 @@ class AnchoredEllipse(AnchoredOffsetbox):
                                    frameon=frameon, **kwargs)
 
 
-
 class AnchoredSizeBar(AnchoredOffsetbox):
     def __init__(self, transform, size, label, loc,
-                 pad=0.1, borderpad=0.1, sep=2, prop=None, frameon=True,
+                 pad=0.1, borderpad=0.1, sep=2,
+                 frameon=True, size_vertical=0, color='black',
+                 label_top=False, fontproperties=None,
                  **kwargs):
         """
-        Draw a horizontal bar with the size in data coordinate of the give axes.
+        Draw a horizontal bar with the size in data coordinate of the given axes.
         A label will be drawn underneath (center-aligned).
 
-        pad, borderpad in fraction of the legend font size (or prop)
-        sep in points.
+        Parameters:
+        -----------
+        transform : matplotlib transformation object
+        size : int or float
+          horizontal length of the size bar, given in data coordinates
+        label : str
+        loc : int
+        pad : int or float, optional
+          in fraction of the legend font size (or prop)
+        borderpad : int or float, optional
+          in fraction of the legend font size (or prop)
+        sep : int or float, optional
+          in points
+        frameon : bool, optional
+          if True, will draw a box around the horizontal bar and label
+        size_vertical : int or float, optional
+          vertical length of the size bar, given in data coordinates
+        color : str, optional
+          color for the size bar and label
+        label_top : bool, optional
+          if True, the label will be over the rectangle
+        fontproperties: a matplotlib.font_manager.FontProperties instance, optional
+          sets the font properties for the label text
+
+        Returns:
+        --------
+        AnchoredSizeBar object
+
+        Example:
+        --------
+        >>> import matplotlib.pyplot as plt
+        >>> import numpy as np
+        >>> from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
+        >>> fig, ax = plt.subplots()
+        >>> ax.imshow(np.random.random((10,10)))
+        >>> bar = AnchoredSizeBar(ax.transData, 3, '3 units', 4)
+        >>> ax.add_artist(bar)
+        >>> fig.show()
+
+        Using all the optional parameters
+
+        >>> import matplotlib.font_manager as fm
+        >>> fontprops = fm.FontProperties(size=14, family='monospace')
+        >>> bar = AnchoredSizeBar(ax.transData, 3, '3 units', 4, pad=0.5, sep=5, borderpad=0.5, frameon=False, size_vertical=0.5, color='white', fontproperties=fontprops)
+
         """
+
         self.size_bar = AuxTransformBox(transform)
-        self.size_bar.add_artist(Rectangle((0,0), size, 0, fc="none"))
+        self.size_bar.add_artist(Rectangle((0, 0), size, size_vertical,
+                                           fill=True, facecolor=color,
+                                           edgecolor=color))
 
-        self.txt_label = TextArea(label, minimumdescent=False)
+        # if fontproperties is None, but `prop` is not, assume that
+        # prop should be used to set the font properties. This is
+        # questionable behavior
+        if fontproperties is None and 'prop' in kwargs:
+            fontproperties = kwargs.pop('prop')
 
-        self._box = VPacker(children=[self.size_bar, self.txt_label],
+        if fontproperties is None:
+            textprops = {'color': color}
+        else:
+            textprops = {'color': color, 'fontproperties': fontproperties}
+
+        self.txt_label = TextArea(
+            label,
+            minimumdescent=False,
+            textprops=textprops)
+
+        if label_top:
+            _box_children = [self.txt_label, self.size_bar]
+        else:
+            _box_children = [self.size_bar, self.txt_label]
+
+        self._box = VPacker(children=_box_children,
                             align="center",
                             pad=0, sep=sep)
 
         AnchoredOffsetbox.__init__(self, loc, pad=pad, borderpad=borderpad,
                                    child=self._box,
-                                   prop=prop,
+                                   prop=fontproperties,
                                    frameon=frameon, **kwargs)
 
 
@@ -157,4 +227,3 @@ if __name__ == "__main__":
 
     plt.draw()
     plt.show()
-
