@@ -1,19 +1,30 @@
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from matplotlib.externals import six
+
 import matplotlib.cbook as cbook
 
-import matplotlib.pyplot as plt
 import matplotlib.axes as maxes
 #import matplotlib.colorbar as mcolorbar
-import colorbar as mcolorbar
+from . import colorbar as mcolorbar
 import matplotlib as mpl
 import matplotlib.patches as mpatches
 import matplotlib.lines as mlines
 import matplotlib.ticker as ticker
 
-from matplotlib.gridspec import SubplotSpec, GridSpec
+from matplotlib.gridspec import SubplotSpec
 
-from axes_divider import Size, SubplotDivider, LocatableAxes, Divider
+from .axes_divider import Size, SubplotDivider, LocatableAxes, Divider
+
+
+def _extend_axes_pad(value):
+    # Check whether a list/tuple/array or scalar has been passed
+    ret = value
+    if not hasattr(ret, "__getitem__"):
+        ret = (value, value)
+    return ret
 
-#import numpy as np
 
 def _tick_only(ax, bottom_on, left_on):
     bottom_off = not bottom_on
@@ -25,6 +36,7 @@ def _tick_only(ax, bottom_on, left_on):
     ax.axis["bottom"].toggle(ticklabels=bottom_off, label=bottom_off)
     ax.axis["left"].toggle(ticklabels=left_off, label=left_off)
 
+
 class Colorbar(mcolorbar.Colorbar):
     def _config_axes_deprecated(self, X, Y):
         '''
@@ -37,7 +49,8 @@ class Colorbar(mcolorbar.Colorbar):
         ax.update_datalim(xy)
         ax.set_xlim(*ax.dataLim.intervalx)
         ax.set_ylim(*ax.dataLim.intervaly)
-        self.outline = mlines.Line2D(xy[:, 0], xy[:, 1], color=mpl.rcParams['axes.edgecolor'],
+        self.outline = mlines.Line2D(xy[:, 0], xy[:, 1],
+                                     color=mpl.rcParams['axes.edgecolor'],
                                      linewidth=mpl.rcParams['axes.linewidth'])
         ax.add_artist(self.outline)
         self.outline.set_clip_box(None)
@@ -61,26 +74,26 @@ class Colorbar(mcolorbar.Colorbar):
             ax.xaxis.get_major_formatter().set_offset_string(offset_string)
 
 
-
 class CbarAxesBase(object):
 
     def colorbar(self, mappable, **kwargs):
-        locator=kwargs.pop("locator", None)
+        locator = kwargs.pop("locator", None)
 
         if locator is None:
             if "ticks" not in kwargs:
                 kwargs["ticks"] = ticker.MaxNLocator(5)
         if locator is not None:
             if "ticks" in kwargs:
-                raise ValueError("Either *locator* or *ticks* need to be given, not both")
+                raise ValueError("Either *locator* or *ticks* need" +
+                                 " to be given, not both")
             else:
                 kwargs["ticks"] = locator
 
         self.hold(True)
-        if self.orientation in  ["top", "bottom"]:
-            orientation="horizontal"
+        if self.orientation in ["top", "bottom"]:
+            orientation = "horizontal"
         else:
-            orientation="vertical"
+            orientation = "vertical"
 
         cb = Colorbar(self, mappable, orientation=orientation, **kwargs)
         self._config_axes()
@@ -120,15 +133,14 @@ class CbarAxesBase(object):
         # axis.major_ticks.set_visible(True)
         # axis.minor_ticks.set_visible(True)
 
-
-        #axis.major_ticklabels.set_size(int(axis.major_ticklabels.get_size()*.9))
+        #axis.major_ticklabels.set_size(
+        #    int(axis.major_ticklabels.get_size()*.9))
         #axis.major_tick_pad = 3
 
         # axis.major_ticklabels.set_visible(b)
         # axis.minor_ticklabels.set_visible(b)
         # axis.label.set_visible(b)
 
-
     def toggle_label(self, b):
         self._default_label_on = b
         axis = self.axis[self.orientation]
@@ -138,10 +150,9 @@ class CbarAxesBase(object):
         #axis.label.set_visible(b)
 
 
-
 class CbarAxes(CbarAxesBase, LocatableAxes):
     def __init__(self, *kl, **kwargs):
-        orientation=kwargs.pop("orientation", None)
+        orientation = kwargs.pop("orientation", None)
         if orientation is None:
             raise ValueError("orientation must be specified")
         self.orientation = orientation
@@ -170,9 +181,9 @@ class Grid(object):
     def __init__(self, fig,
                  rect,
                  nrows_ncols,
-                 ngrids = None,
+                 ngrids=None,
                  direction="row",
-                 axes_pad = 0.02,
+                 axes_pad=0.02,
                  add_all=True,
                  share_all=False,
                  share_x=True,
@@ -196,6 +207,8 @@ class Grid(object):
           ================  ========  =========================================
           direction         "row"     [ "row" | "column" ]
           axes_pad          0.02      float| pad between axes given in inches
+                                      or tuple-like of floats,
+                                      (horizontal padding, vertical padding)
           add_all           True      [ True | False ]
           share_all         False     [ True | False ]
           share_x           True      [ True | False ]
@@ -210,7 +223,7 @@ class Grid(object):
         if ngrids is None:
             ngrids = self._nrows * self._ncols
         else:
-            if (ngrids > self._nrows * self._ncols) or  (ngrids <= 0):
+            if (ngrids > self._nrows * self._ncols) or (ngrids <= 0):
                 raise Exception("")
 
         self.ngrids = ngrids
@@ -222,21 +235,20 @@ class Grid(object):
 
         self._direction = direction
 
-
         if axes_class is None:
             axes_class = self._defaultLocatableAxesClass
             axes_class_args = {}
         else:
             if (type(axes_class)) == type and \
-                   issubclass(axes_class, self._defaultLocatableAxesClass.Axes):
+                   issubclass(axes_class,
+                              self._defaultLocatableAxesClass.Axes):
                 axes_class_args = {}
             else:
                 axes_class, axes_class_args = axes_class
 
         self.axes_all = []
-        self.axes_column = [[] for i in range(self._ncols)]
-        self.axes_row = [[] for i in range(self._nrows)]
-
+        self.axes_column = [[] for _ in range(self._ncols)]
+        self.axes_row = [[] for _ in range(self._nrows)]
 
         h = []
         v = []
@@ -255,12 +267,11 @@ class Grid(object):
         else:
             raise Exception("")
 
-
         rect = self._divider.get_position()
 
         # reference axes
-        self._column_refax = [None for i in range(self._ncols)]
-        self._row_refax = [None for i in range(self._nrows)]
+        self._column_refax = [None for _ in range(self._ncols)]
+        self._row_refax = [None for _ in range(self._nrows)]
         self._refax = None
 
         for i in range(self.ngrids):
@@ -307,24 +318,23 @@ class Grid(object):
 
         self.set_label_mode(label_mode)
 
-
     def _init_axes_pad(self, axes_pad):
+        axes_pad = _extend_axes_pad(axes_pad)
         self._axes_pad = axes_pad
 
-        self._horiz_pad_size = Size.Fixed(axes_pad)
-        self._vert_pad_size = Size.Fixed(axes_pad)
-
+        self._horiz_pad_size = Size.Fixed(axes_pad[0])
+        self._vert_pad_size = Size.Fixed(axes_pad[1])
 
     def _update_locators(self):
 
         h = []
 
         h_ax_pos = []
-        h_cb_pos = []
 
-        for ax in self._column_refax:
+        for _ in self._column_refax:
             #if h: h.append(Size.Fixed(self._axes_pad))
-            if h: h.append(self._horiz_pad_size)
+            if h:
+                h.append(self._horiz_pad_size)
 
             h_ax_pos.append(len(h))
 
@@ -334,27 +344,24 @@ class Grid(object):
         v = []
 
         v_ax_pos = []
-        v_cb_pos = []
-        for ax in self._row_refax[::-1]:
+        for _ in self._row_refax[::-1]:
             #if v: v.append(Size.Fixed(self._axes_pad))
-            if v: v.append(self._vert_pad_size)
+            if v:
+                v.append(self._vert_pad_size)
 
             v_ax_pos.append(len(v))
             sz = Size.Scaled(1)
             v.append(sz)
 
-
         for i in range(self.ngrids):
             col, row = self._get_col_row(i)
             locator = self._divider.new_locator(nx=h_ax_pos[col],
-                                                ny=v_ax_pos[self._nrows -1 - row])
+                                ny=v_ax_pos[self._nrows - 1 - row])
             self.axes_all[i].set_axes_locator(locator)
 
         self._divider.set_horizontal(h)
         self._divider.set_vertical(v)
 
-
-
     def _get_col_row(self, n):
         if self._direction == "column":
             col, row = divmod(n, self._nrows)
@@ -363,11 +370,13 @@ class Grid(object):
 
         return col, row
 
+    # Good to propagate __len__ if we have __getitem__
+    def __len__(self):
+        return len(self.axes_all)
 
     def __getitem__(self, i):
         return self.axes_all[i]
 
-
     def get_geometry(self):
         """
         get geometry of the grid. Returns a tuple of two integer,
@@ -379,12 +388,19 @@ class Grid(object):
         "set axes_pad"
         self._axes_pad = axes_pad
 
-        self._horiz_pad_size.fixed_size = axes_pad
-        self._vert_pad_size.fixed_size = axes_pad
-
+        # These two lines actually differ from ones in _init_axes_pad
+        self._horiz_pad_size.fixed_size = axes_pad[0]
+        self._vert_pad_size.fixed_size = axes_pad[1]
 
     def get_axes_pad(self):
-        "get axes_pad"
+        """
+        get axes_pad
+
+        Returns
+        -------
+        tuple
+            Padding in inches, (horizontal pad, vertical pad)
+        """
         return self._axes_pad
 
     def set_aspect(self, aspect):
@@ -426,7 +442,7 @@ class Grid(object):
 
     def get_divider(self):
         return self._divider
-        
+
     def set_axes_locator(self, locator):
         self._divider.set_locator(locator)
 
@@ -444,8 +460,6 @@ class Grid(object):
 #         return vsize, hsize
 
 
-
-
 class ImageGrid(Grid):
     """
     A class that creates a grid of Axes. In matplotlib, the axes
@@ -461,9 +475,9 @@ class ImageGrid(Grid):
     def __init__(self, fig,
                  rect,
                  nrows_ncols,
-                 ngrids = None,
+                 ngrids=None,
                  direction="row",
-                 axes_pad = 0.02,
+                 axes_pad=0.02,
                  add_all=True,
                  share_all=False,
                  aspect=True,
@@ -490,6 +504,8 @@ class ImageGrid(Grid):
           ================  ========  =========================================
           direction         "row"     [ "row" | "column" ]
           axes_pad          0.02      float| pad between axes given in inches
+                                      or tuple-like of floats,
+                                      (horizontal padding, vertical padding)
           add_all           True      [ True | False ]
           share_all         False     [ True | False ]
           aspect            True      [ True | False ]
@@ -500,7 +516,8 @@ class ImageGrid(Grid):
           cbar_size         "5%"
           cbar_set_cax      True      [ True | False ]
           axes_class        None      a type object which must be a subclass
-                                      of :class:`~matplotlib.axes.Axes`
+                                      of axes_grid's subclass of
+                                      :class:`~matplotlib.axes.Axes`
           ================  ========  =========================================
 
         *cbar_set_cax* : if True, each axes in the grid has a cax
@@ -511,17 +528,22 @@ class ImageGrid(Grid):
         if ngrids is None:
             ngrids = self._nrows * self._ncols
         else:
-            if (ngrids > self._nrows * self._ncols) or  (ngrids <= 0):
+            if (ngrids > self._nrows * self._ncols) or (ngrids <= 0):
                 raise Exception("")
 
         self.ngrids = ngrids
 
+        axes_pad = _extend_axes_pad(axes_pad)
         self._axes_pad = axes_pad
 
         self._colorbar_mode = cbar_mode
         self._colorbar_location = cbar_location
         if cbar_pad is None:
-            self._colorbar_pad = axes_pad
+            # horizontal or vertical arrangement?
+            if cbar_location in ("left", "right"):
+                self._colorbar_pad = axes_pad[0]
+            else:
+                self._colorbar_pad = axes_pad[1]
         else:
             self._colorbar_pad = cbar_pad
 
@@ -534,7 +556,6 @@ class ImageGrid(Grid):
 
         self._direction = direction
 
-
         if axes_class is None:
             axes_class = self._defaultLocatableAxesClass
             axes_class_args = {}
@@ -544,11 +565,9 @@ class ImageGrid(Grid):
             else:
                 axes_class, axes_class_args = axes_class
 
-
-
         self.axes_all = []
-        self.axes_column = [[] for i in range(self._ncols)]
-        self.axes_row = [[] for i in range(self._nrows)]
+        self.axes_column = [[] for _ in range(self._ncols)]
+        self.axes_row = [[] for _ in range(self._nrows)]
 
         self.cbar_axes = []
 
@@ -569,12 +588,11 @@ class ImageGrid(Grid):
         else:
             raise Exception("")
 
-
         rect = self._divider.get_position()
 
         # reference axes
-        self._column_refax = [None for i in range(self._ncols)]
-        self._row_refax = [None for i in range(self._nrows)]
+        self._column_refax = [None for _ in range(self._ncols)]
+        self._row_refax = [None for _ in range(self._nrows)]
         self._refax = None
 
         for i in range(self.ngrids):
@@ -582,8 +600,12 @@ class ImageGrid(Grid):
             col, row = self._get_col_row(i)
 
             if share_all:
-                sharex = self._refax
-                sharey = self._refax
+                if self.axes_all:
+                    sharex = self.axes_all[0]
+                    sharey = self.axes_all[0]
+                else:
+                    sharex = None
+                    sharey = None
             else:
                 sharex = self._column_refax[col]
                 sharey = self._row_refax[row]
@@ -591,21 +613,20 @@ class ImageGrid(Grid):
             ax = axes_class(fig, rect, sharex=sharex, sharey=sharey,
                             **axes_class_args)
 
-            if share_all:
-                if self._refax is None:
-                    self._refax = ax
-            else:
-                if sharex is None:
-                    self._column_refax[col] = ax
-                if sharey is None:
-                    self._row_refax[row] = ax
-
             self.axes_all.append(ax)
             self.axes_column[col].append(ax)
             self.axes_row[row].append(ax)
 
+            if share_all:
+                if self._refax is None:
+                    self._refax = ax
+            if sharex is None:
+                self._column_refax[col] = ax
+            if sharey is None:
+                self._row_refax[row] = ax
+
             cax = self._defaultCbarAxesClass(fig, rect,
-                                             orientation=self._colorbar_location)
+                                        orientation=self._colorbar_location)
             self.cbar_axes.append(cax)
 
         self.axes_llc = self.axes_column[0][-1]
@@ -620,14 +641,19 @@ class ImageGrid(Grid):
             if self._colorbar_mode == "single":
                 for ax in self.axes_all:
                     ax.cax = self.cbar_axes[0]
+            elif self._colorbar_mode == "edge":
+                for index, ax in enumerate(self.axes_all):
+                    col, row = self._get_col_row(index)
+                    if self._colorbar_location in ("left", "right"):
+                        ax.cax = self.cbar_axes[row]
+                    else:
+                        ax.cax = self.cbar_axes[col]
             else:
                 for ax, cax in zip(self.axes_all, self.cbar_axes):
                     ax.cax = cax
 
         self.set_label_mode(label_mode)
 
-
-
     def _update_locators(self):
 
         h = []
@@ -635,7 +661,8 @@ class ImageGrid(Grid):
 
         h_ax_pos = []
         h_cb_pos = []
-        if self._colorbar_mode == "single" and self._colorbar_location in ('left', 'bottom'):
+        if (self._colorbar_mode == "single" and
+             self._colorbar_location in ('left', 'bottom')):
             if self._colorbar_location == "left":
                 #sz = Size.Fraction(Size.AxesX(self.axes_llc), self._nrows)
                 sz = Size.Fraction(self._nrows, Size.AxesX(self.axes_llc))
@@ -653,13 +680,15 @@ class ImageGrid(Grid):
             self.cbar_axes[0].set_axes_locator(locator)
             self.cbar_axes[0].set_visible(True)
 
-        for col,ax in enumerate(self._column_refax):
-            if h: h.append(self._horiz_pad_size) #Size.Fixed(self._axes_pad))
+        for col, ax in enumerate(self.axes_row[0]):
+            if h:
+                h.append(self._horiz_pad_size)  # Size.Fixed(self._axes_pad))
 
             if ax:
-                sz = Size.AxesX(ax)
+                sz = Size.AxesX(ax, aspect="axes", ref_ax=self.axes_all[0])
             else:
-                sz = Size.AxesX(self.axes_llc)
+                sz = Size.AxesX(self.axes_all[0],
+                                aspect="axes", ref_ax=self.axes_all[0])
 
             if (self._colorbar_mode == "each" or
                     (self._colorbar_mode == 'edge' and
@@ -672,23 +701,25 @@ class ImageGrid(Grid):
 
             h.append(sz)
 
-            if (self._colorbar_mode == "each" or
+            if ((self._colorbar_mode == "each" or
                     (self._colorbar_mode == 'edge' and
-                        col == self._ncols - 1)) and self._colorbar_location == "right":
+                        col == self._ncols - 1)) and
+                    self._colorbar_location == "right"):
                 h.append(Size.from_any(self._colorbar_pad, sz))
                 h_cb_pos.append(len(h))
                 h.append(Size.from_any(self._colorbar_size, sz))
 
-
         v_ax_pos = []
         v_cb_pos = []
-        for row,ax in enumerate(self._row_refax[::-1]):
-            if v: v.append(self._horiz_pad_size) #Size.Fixed(self._axes_pad))
+        for row, ax in enumerate(self.axes_column[0][::-1]):
+            if v:
+                v.append(self._vert_pad_size)  # Size.Fixed(self._axes_pad))
 
             if ax:
-                sz = Size.AxesY(ax)
+                sz = Size.AxesY(ax, aspect="axes", ref_ax=self.axes_all[0])
             else:
-                sz = Size.AxesY(self.axes_llc)
+                sz = Size.AxesY(self.axes_all[0],
+                                aspect="axes", ref_ax=self.axes_all[0])
 
             if (self._colorbar_mode == "each" or
                     (self._colorbar_mode == 'edge' and
@@ -700,42 +731,46 @@ class ImageGrid(Grid):
             v_ax_pos.append(len(v))
             v.append(sz)
 
-            if (self._colorbar_mode == "each" or
+            if ((self._colorbar_mode == "each" or
                     (self._colorbar_mode == 'edge' and
-                        row == self._nrows - 1)) and self._colorbar_location == "top":
+                        row == self._nrows - 1)) and
+                        self._colorbar_location == "top"):
                 v.append(Size.from_any(self._colorbar_pad, sz))
                 v_cb_pos.append(len(v))
                 v.append(Size.from_any(self._colorbar_size, sz))
 
-
         for i in range(self.ngrids):
             col, row = self._get_col_row(i)
-            #locator = self._divider.new_locator(nx=4*col, ny=2*(self._nrows - row - 1))
+            #locator = self._divider.new_locator(nx=4*col,
+            #                                    ny=2*(self._nrows - row - 1))
             locator = self._divider.new_locator(nx=h_ax_pos[col],
-                                                ny=v_ax_pos[self._nrows -1 - row])
+                                                ny=v_ax_pos[self._nrows-1-row])
             self.axes_all[i].set_axes_locator(locator)
 
             if self._colorbar_mode == "each":
                 if self._colorbar_location in ("right", "left"):
-                    locator = self._divider.new_locator(nx=h_cb_pos[col],
-                                                        ny=v_ax_pos[self._nrows -1 - row])
+                    locator = self._divider.new_locator(
+                        nx=h_cb_pos[col], ny=v_ax_pos[self._nrows - 1 - row])
+
                 elif self._colorbar_location in ("top", "bottom"):
-                    locator = self._divider.new_locator(nx=h_ax_pos[col],
-                                                        ny=v_cb_pos[self._nrows -1 - row])
+                    locator = self._divider.new_locator(
+                        nx=h_ax_pos[col], ny=v_cb_pos[self._nrows - 1 - row])
+
                 self.cbar_axes[i].set_axes_locator(locator)
             elif self._colorbar_mode == 'edge':
                 if ((self._colorbar_location == 'left' and col == 0) or
-                        (self._colorbar_location == 'right' and col == self._ncols-1)):
-                    locator = self._divider.new_locator(nx=h_cb_pos[0],
-                                                        ny=v_ax_pos[self._nrows -1 - row])
+                        (self._colorbar_location == 'right'
+                         and col == self._ncols-1)):
+                    locator = self._divider.new_locator(
+                        nx=h_cb_pos[0], ny=v_ax_pos[self._nrows -1 - row])
                     self.cbar_axes[row].set_axes_locator(locator)
-                elif ((self._colorbar_location == 'bottom' and row == self._nrows - 1) or
+                elif ((self._colorbar_location == 'bottom' and
+                       row == self._nrows - 1) or
                         (self._colorbar_location == 'top' and row == 0)):
                     locator = self._divider.new_locator(nx=h_ax_pos[col],
                                                         ny=v_cb_pos[0])
                     self.cbar_axes[col].set_axes_locator(locator)
 
-
         if self._colorbar_mode == "single":
             if self._colorbar_location == "right":
                 #sz = Size.Fraction(Size.AxesX(self.axes_llc), self._nrows)
@@ -778,102 +813,3 @@ class ImageGrid(Grid):
 
 AxesGrid = ImageGrid
 
-
-
-#if __name__ == "__main__":
-if 0:
-    F = plt.figure(1, (7, 6))
-    F.clf()
-
-    F.subplots_adjust(left=0.15, right=0.9)
-
-    grid = Grid(F, 111, # similar to subplot(111)
-                nrows_ncols = (2, 2),
-                direction="row",
-                axes_pad = 0.05,
-                add_all=True,
-                label_mode = "1",
-                )
-
-
-
-#if __name__ == "__main__":
-if 0:
-    from axes_divider import get_demo_image
-    F = plt.figure(1, (9, 3.5))
-    F.clf()
-
-    F.subplots_adjust(left=0.05, right=0.98)
-
-    grid = ImageGrid(F, 131, # similar to subplot(111)
-                    nrows_ncols = (2, 2),
-                    direction="row",
-                    axes_pad = 0.05,
-                    add_all=True,
-                    label_mode = "1",
-                    )
-
-    Z, extent = get_demo_image()
-    plt.ioff()
-    for i in range(4):
-        im = grid[i].imshow(Z, extent=extent, interpolation="nearest")
-
-    # This only affects axes in first column and second row as share_all = False.
-    grid.axes_llc.set_xticks([-2, 0, 2])
-    grid.axes_llc.set_yticks([-2, 0, 2])
-    plt.ion()
-
-
-    grid = ImageGrid(F, 132, # similar to subplot(111)
-                    nrows_ncols = (2, 2),
-                    direction="row",
-                    axes_pad = 0.0,
-                    add_all=True,
-                    share_all=True,
-                    label_mode = "1",
-                    cbar_mode="single",
-                    )
-
-    Z, extent = get_demo_image()
-    plt.ioff()
-    for i in range(4):
-        im = grid[i].imshow(Z, extent=extent, interpolation="nearest")
-    plt.colorbar(im, cax = grid.cbar_axes[0])
-    plt.setp(grid.cbar_axes[0].get_yticklabels(), visible=False)
-
-    # This affects all axes as share_all = True.
-    grid.axes_llc.set_xticks([-2, 0, 2])
-    grid.axes_llc.set_yticks([-2, 0, 2])
-
-    plt.ion()
-
-
-
-    grid = ImageGrid(F, 133, # similar to subplot(122)
-                    nrows_ncols = (2, 2),
-                    direction="row",
-                    axes_pad = 0.1,
-                    add_all=True,
-                    label_mode = "1",
-                    share_all = True,
-                    cbar_location="top",
-                    cbar_mode="each",
-                    cbar_size="7%",
-                    cbar_pad="2%",
-                    )
-    plt.ioff()
-    for i in range(4):
-        im = grid[i].imshow(Z, extent=extent, interpolation="nearest")
-        plt.colorbar(im, cax = grid.cbar_axes[i],
-                     orientation="horizontal")
-        grid.cbar_axes[i].xaxis.set_ticks_position("top")
-        plt.setp(grid.cbar_axes[i].get_xticklabels(), visible=False)
-
-    # This affects all axes as share_all = True.
-    grid.axes_llc.set_xticks([-2, 0, 2])
-    grid.axes_llc.set_yticks([-2, 0, 2])
-
-    plt.ion()
-    plt.draw()
-
-
