@@ -9,6 +9,12 @@
 Module containing Axes3D, an object which can plot 3D objects on a
 2D matplotlib figure.
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+import math
+
+from matplotlib.externals import six
+from matplotlib.externals.six.moves import map, xrange, zip, reduce
 
 import warnings
 from operator import itemgetter
@@ -25,9 +31,9 @@ from matplotlib.tri.triangulation import Triangulation
 import numpy as np
 from matplotlib.colors import Normalize, colorConverter, LightSource
 
-import art3d
-import proj3d
-import axis3d
+from . import art3d
+from . import proj3d
+from . import axis3d
 
 def unit_bbox():
     box = Bbox(np.array([[0, 0], [1, 1]]))
@@ -120,9 +126,11 @@ class Axes3D(Axes):
 
     def set_axis_off(self):
         self._axis3don = False
+        self.stale = True
 
     def set_axis_on(self):
         self._axis3don = True
+        self.stale = True
 
     def have_units(self):
         """
@@ -184,27 +192,30 @@ class Axes3D(Axes):
     def _init_axis(self):
         '''Init 3D axes; overrides creation of regular X/Y axes'''
         self.w_xaxis = axis3d.XAxis('x', self.xy_viewLim.intervalx,
-                            self.xy_dataLim.intervalx, self)
+                                    self.xy_dataLim.intervalx, self)
         self.xaxis = self.w_xaxis
         self.w_yaxis = axis3d.YAxis('y', self.xy_viewLim.intervaly,
-                            self.xy_dataLim.intervaly, self)
+                                    self.xy_dataLim.intervaly, self)
         self.yaxis = self.w_yaxis
         self.w_zaxis = axis3d.ZAxis('z', self.zz_viewLim.intervalx,
-                            self.zz_dataLim.intervalx, self)
+                                    self.zz_dataLim.intervalx, self)
         self.zaxis = self.w_zaxis
 
         for ax in self.xaxis, self.yaxis, self.zaxis:
             ax.init3d()
 
     def get_children(self):
-        return [self.zaxis,] + Axes.get_children(self)
+        return [self.zaxis, ] + Axes.get_children(self)
+
+    def _get_axis_list(self):
+        return super(Axes3D, self)._get_axis_list() + (self.zaxis, )
 
     def unit_cube(self, vals=None):
         minx, maxx, miny, maxy, minz, maxz = vals or self.get_w_lims()
         xs, ys, zs = ([minx, maxx, maxx, minx, minx, maxx, maxx, minx],
-                    [miny, miny, maxy, maxy, miny, miny, maxy, maxy],
-                    [minz, minz, minz, minz, maxz, maxz, maxz, maxz])
-        return zip(xs, ys, zs)
+                      [miny, miny, maxy, maxy, miny, miny, maxy, maxy],
+                      [minz, minz, minz, minz, maxz, maxz, maxz, maxz])
+        return list(zip(xs, ys, zs))
 
     def tunit_cube(self, vals=None, M=None):
         if M is None:
@@ -236,6 +247,18 @@ class Axes3D(Axes):
         self.axesPatch.draw(renderer)
         self._frameon = False
 
+        # first, set the aspect
+        # this is duplicated from `axes._base._AxesBase.draw`
+        # but must be called before any of the artist are drawn as
+        # it adjusts the view limits and the size of the bounding box
+        # of the axes
+        locator = self.get_axes_locator()
+        if locator:
+            pos = locator(self, renderer)
+            self.apply_aspect(pos)
+        else:
+            self.apply_aspect()
+
         # add the projection matrix to the renderer
         self.M = self.get_proj()
         renderer.M = self.M
@@ -336,6 +359,7 @@ class Axes3D(Axes):
         if m < 0 or m > 1 :
             raise ValueError("margin must be in range 0 to 1")
         self._zmargin = m
+        self.stale = True
 
     def margins(self, *args, **kw) :
         """
@@ -435,7 +459,7 @@ class Axes3D(Axes):
                                          scalez=scalez)
 
     def auto_scale_xyz(self, X, Y, Z=None, had_data=None):
-        x, y, z = map(np.asarray, (X, Y, Z))
+        x, y, z = list(map(np.asarray, (X, Y, Z)))
         try:
             x, y = x.flatten(), y.flatten()
             if Z is not None:
@@ -604,7 +628,7 @@ class Axes3D(Axes):
                     if (other.figure != self.figure and
                         other.figure.canvas is not None):
                         other.figure.canvas.draw_idle()
-
+        self.stale = True
         return left, right
     set_xlim = set_xlim3d
 
@@ -659,7 +683,7 @@ class Axes3D(Axes):
                     if (other.figure != self.figure and
                         other.figure.canvas is not None):
                         other.figure.canvas.draw_idle()
-
+        self.stale = True
         return bottom, top
     set_ylim = set_ylim3d
 
@@ -713,7 +737,7 @@ class Axes3D(Axes):
                     if (other.figure != self.figure and
                         other.figure.canvas is not None):
                         other.figure.canvas.draw_idle()
-
+        self.stale = True
         return bottom, top
     set_zlim = set_zlim3d
 
@@ -765,6 +789,7 @@ class Axes3D(Axes):
         self.yaxis._set_scale(value, **kwargs)
         self.autoscale_view(scalex=False, scalez=False)
         self._update_transScale()
+        self.stale = True
     set_yscale.__doc__ = maxes.Axes.set_yscale.__doc__ + """
 
         .. versionadded :: 1.1.0
@@ -796,6 +821,7 @@ class Axes3D(Axes):
         self.zaxis._set_scale(value, **kwargs)
         self.autoscale_view(scalex=False, scaley=False)
         self._update_transScale()
+        self.stale = True
 
     def set_zticks(self, *args, **kwargs):
         """
@@ -1002,8 +1028,11 @@ class Axes3D(Axes):
         else:
             warnings.warn('Axes3D.figure.canvas is \'None\', mouse rotation disabled.  Set canvas then call Axes3D.mouse_init().')
 
-        self._rotate_btn = np.atleast_1d(rotate_btn)
-        self._zoom_btn = np.atleast_1d(zoom_btn)
+        # coerce scalars into array-like, then convert into
+        # a regular list to avoid comparisons against None
+        # which breaks in recent versions of numpy.
+        self._rotate_btn = np.atleast_1d(rotate_btn).tolist()
+        self._zoom_btn = np.atleast_1d(zoom_btn).tolist()
 
     def can_zoom(self) :
         """
@@ -1174,11 +1203,7 @@ class Axes3D(Axes):
         '''
         Set zlabel.  See doc for :meth:`set_ylabel` for description.
 
-        .. note::
-            Currently, *labelpad* does not have an effect on the labels.
         '''
-        # FIXME: With a rework of axis3d.py, the labelpad should work again
-        #        At that point, remove the above message in the docs.
         if labelpad is not None : self.zaxis.labelpad = labelpad
         return self.zaxis.set_label_text(zlabel, fontdict, **kwargs)
 
@@ -1211,6 +1236,7 @@ class Axes3D(Axes):
         .. versionadded :: 1.1.0
         """
         self._frameon = bool(b)
+        self.stale = True
 
     def get_axisbelow(self):
         """
@@ -1236,6 +1262,7 @@ class Axes3D(Axes):
             This function was added for completeness.
         """
         self._axisbelow = True
+        self.stale = True
 
     def grid(self, b=True, **kwargs):
         '''
@@ -1253,7 +1280,8 @@ class Axes3D(Axes):
         # TODO: Operate on each axes separately
         if len(kwargs) :
             b = True
-        self._draw_grid = maxes._string_to_bool(b)
+        self._draw_grid = cbook._string_to_bool(b)
+        self.stale = True
 
     def ticklabel_format(self, **kwargs) :
         """
@@ -1287,11 +1315,11 @@ class Axes3D(Axes):
                 cb = False
             else:
                 cb = True
-                raise NotImplementedError, "comma style remains to be added"
+                raise NotImplementedError("comma style remains to be added")
         elif style == '':
             sb = None
         else:
-            raise ValueError, "%s is not a valid style value"
+            raise ValueError("%s is not a valid style value")
         try:
             if sb is not None:
                 if axis in ['both', 'z']:
@@ -1466,7 +1494,7 @@ class Axes3D(Axes):
         ==========  ================================================
         Argument    Description
         ==========  ================================================
-        *xs*, *ys*  X, y coordinates of vertices
+        *xs*, *ys*  x, y coordinates of vertices
 
         *zs*        z value(s), either one for all points or one for
                     each point.
@@ -1527,12 +1555,17 @@ class Axes3D(Axes):
         but it also supports color mapping by supplying the *cmap*
         argument.
 
+        The `rstride` and `cstride` kwargs set the stride used to
+        sample the input data to generate the graph.  If 1k by 1k
+        arrays are passed in the default values for the strides will
+        result in a 100x100 grid being plotted.
+
         ============= ================================================
         Argument      Description
         ============= ================================================
         *X*, *Y*, *Z* Data values as 2D arrays
-        *rstride*     Array row stride (step size)
-        *cstride*     Array column stride (step size)
+        *rstride*     Array row stride (step size), defaults to 10
+        *cstride*     Array column stride (step size), defaults to 10
         *color*       Color of the surface patches
         *cmap*        A colormap for the surface patches.
         *facecolors*  Face colors for the individual patches
@@ -1601,7 +1634,7 @@ class Axes3D(Axes):
 
                 # The construction leaves the array with duplicate points, which
                 # are removed here.
-                ps = zip(*ps)
+                ps = list(zip(*ps))
                 lastp = np.array([])
                 ps2 = [ps[0]] + [ps[i] for i in xrange(1, len(ps)) if ps[i] != ps[i-1]]
                 avgzsum = sum(p[2] for p in ps2)
@@ -1670,11 +1703,13 @@ class Axes3D(Axes):
         '''
 
         shade = np.array([np.dot(n / proj3d.mod(n), [-1, -1, 0.5])
+                          if proj3d.mod(n) else np.nan
                           for n in normals])
         mask = ~np.isnan(shade)
 
         if len(shade[mask]) > 0:
             norm = Normalize(min(shade[mask]), max(shade[mask]))
+            shade[~mask] = min(shade[mask])
             color = colorConverter.to_rgba_array(color)
             # shape of color should be (M, 4) (where M is number of faces)
             # shape of shade should be (M,)
@@ -1696,13 +1731,18 @@ class Axes3D(Axes):
         '''
         Plot a 3D wireframe.
 
+        The `rstride` and `cstride` kwargs set the stride used to
+        sample the input data to generate the graph. If either is 0
+        the input data in not sampled along this direction producing a
+        3D line plot rather than a wireframe plot.
+
         ==========  ================================================
         Argument    Description
         ==========  ================================================
         *X*, *Y*,   Data values as 2D arrays
         *Z*
-        *rstride*   Array row stride (step size)
-        *cstride*   Array column stride (step size)
+        *rstride*   Array row stride (step size), defaults to 1
+        *cstride*   Array column stride (step size), defaults to 1
         ==========  ================================================
 
         Keyword arguments are passed on to
@@ -1725,14 +1765,23 @@ class Axes3D(Axes):
         # This transpose will make it easy to obtain the columns.
         tX, tY, tZ = np.transpose(X), np.transpose(Y), np.transpose(Z)
 
-        rii = range(0, rows, rstride)
-        cii = range(0, cols, cstride)
+        if rstride:
+            rii = list(xrange(0, rows, rstride))
+            # Add the last index only if needed
+            if rows > 0 and rii[-1] != (rows - 1) :
+                rii += [rows-1]
+        else:
+            rii = []
+        if cstride:
+            cii = list(xrange(0, cols, cstride))
+            # Add the last index only if needed
+            if cols > 0 and cii[-1] != (cols - 1) :
+                cii += [cols-1]
+        else:
+            cii = []
 
-        # Add the last index only if needed
-        if rows > 0 and rii[-1] != (rows - 1) :
-            rii += [rows-1]
-        if cols > 0 and cii[-1] != (cols - 1) :
-            cii += [cols-1]
+        if rstride == 0 and cstride == 0:
+            raise ValueError("Either rstride or cstride must be non zero")
 
         # If the inputs were empty, then just
         # reset everything.
@@ -1748,10 +1797,10 @@ class Axes3D(Axes):
         tylines = [tY[i] for i in cii]
         tzlines = [tZ[i] for i in cii]
 
-        lines = [zip(xl, yl, zl) for xl, yl, zl in \
-                zip(xlines, ylines, zlines)]
-        lines += [zip(xl, yl, zl) for xl, yl, zl in \
-                zip(txlines, tylines, tzlines)]
+        lines = [list(zip(xl, yl, zl)) for xl, yl, zl in \
+                 zip(xlines, ylines, zlines)]
+        lines += [list(zip(xl, yl, zl)) for xl, yl, zl in \
+                  zip(txlines, tylines, tzlines)]
 
         linec = art3d.Line3DCollection(lines, *args, **kwargs)
         self.add_collection(linec)
@@ -1822,12 +1871,17 @@ class Axes3D(Axes):
         lightsource = kwargs.pop('lightsource', None)
 
         tri, args, kwargs = Triangulation.get_from_args_and_kwargs(*args, **kwargs)
-        z = np.asarray(args[0])
+        if 'Z' in kwargs:
+            z = np.asarray(kwargs.pop('Z'))
+        else:
+            z = np.asarray(args[0])
+            # We do this so Z doesn't get passed as an arg to PolyCollection
+            args = args[1:]
 
         triangles = tri.get_masked_triangles()
-        xt = tri.x[triangles][...,np.newaxis]
-        yt = tri.y[triangles][...,np.newaxis]
-        zt = np.array(z)[triangles][...,np.newaxis]
+        xt = tri.x[triangles][..., np.newaxis]
+        yt = tri.y[triangles][..., np.newaxis]
+        zt = z[triangles][..., np.newaxis]
 
         verts = np.concatenate((xt, yt, zt), axis=2)
 
@@ -1980,8 +2034,8 @@ class Axes3D(Axes):
 
     contour3D = contour
 
-    def tricontour(self, X, Y, Z, *args, **kwargs):
-        '''
+    def tricontour(self, *args, **kwargs):
+        """
         Create a 3D contour plot.
 
         ==========  ================================================
@@ -2001,8 +2055,12 @@ class Axes3D(Axes):
 
         Returns a :class:`~matplotlib.axes.Axes.contour`
 
-        .. versionadded:: 1.1.0
-        '''
+        .. versionchanged:: 1.3.0
+            Added support for custom triangulations
+
+        EXPERIMENTAL:  This method currently produces incorrect output due to a
+        longstanding bug in 3D PolyCollection rendering.
+        """
 
         extend3d = kwargs.pop('extend3d', False)
         stride = kwargs.pop('stride', 5)
@@ -2011,8 +2069,21 @@ class Axes3D(Axes):
 
         had_data = self.has_data()
 
+        tri, args, kwargs = Triangulation.get_from_args_and_kwargs(
+                *args, **kwargs)
+        X = tri.x
+        Y = tri.y
+        if 'Z' in kwargs:
+            Z = kwargs.pop('Z')
+        else:
+            Z = args[0]
+            # We do this so Z doesn't get passed as an arg to Axes.tricontour
+            args = args[1:]
+
         jX, jY, jZ = art3d.rotate_axes(X, Y, Z, zdir)
-        cset = Axes.tricontour(self, jX, jY, jZ, *args, **kwargs)
+        tri = Triangulation(jX, jY, tri.triangles, tri.mask)
+
+        cset = Axes.tricontour(self, tri, jZ, *args, **kwargs)
         self.add_contour_set(cset, extend3d, stride, zdir, offset)
 
         self.auto_scale_xyz(X, Y, Z, had_data)
@@ -2055,8 +2126,8 @@ class Axes3D(Axes):
 
     contourf3D = contourf
 
-    def tricontourf(self, X, Y, Z, offset=None, zdir='z', *args, **kwargs):
-        '''
+    def tricontourf(self, *args, **kwargs):
+        """
         Create a 3D contourf plot.
 
         ==========  ================================================
@@ -2074,15 +2145,32 @@ class Axes3D(Axes):
 
         Returns a :class:`~matplotlib.axes.Axes.contour`
 
-        .. versionadded :: 1.1.0
-        '''
+        .. versionchanged :: 1.3.0
+            Added support for custom triangulations
+
+        EXPERIMENTAL:  This method currently produces incorrect output due to a
+        longstanding bug in 3D PolyCollection rendering.
+        """
         zdir = kwargs.pop('zdir', 'z')
         offset = kwargs.pop('offset', None)
 
         had_data = self.has_data()
 
+        tri, args, kwargs = Triangulation.get_from_args_and_kwargs(
+                *args, **kwargs)
+        X = tri.x
+        Y = tri.y
+        if 'Z' in kwargs:
+            Z = kwargs.pop('Z')
+        else:
+            Z = args[0]
+            # We do this so Z doesn't get passed as an arg to Axes.tricontourf
+            args = args[1:]
+
         jX, jY, jZ = art3d.rotate_axes(X, Y, Z, zdir)
-        cset = Axes.tricontourf(self, X, Y, Z, *args, **kwargs)
+        tri = Triangulation(jX, jY, tri.triangles, tri.mask)
+
+        cset = Axes.tricontourf(self, tri, jZ, *args, **kwargs)
         self.add_contourf_set(cset, zdir, offset)
 
         self.auto_scale_xyz(X, Y, Z, had_data)
@@ -2121,31 +2209,36 @@ class Axes3D(Axes):
 
         Axes.add_collection(self, col)
 
-    def scatter(self, xs, ys, zs=0, zdir='z', s=20, c='b', *args, **kwargs):
+    def scatter(self, xs, ys, zs=0, zdir='z', s=20, c='b', depthshade=True,
+                *args, **kwargs):
         '''
         Create a scatter plot.
 
-        ==========  ==========================================================
-        Argument    Description
-        ==========  ==========================================================
-        *xs*, *ys*  Positions of data points.
-        *zs*        Either an array of the same length as *xs* and
-                    *ys* or a single value to place all points in
-                    the same plane. Default is 0.
-        *zdir*      Which direction to use as z ('x', 'y' or 'z')
-                    when plotting a 2D set.
-        *s*         size in points^2.  It is a scalar or an array of the same
-                    length as *x* and *y*.
+        ============  ========================================================
+        Argument      Description
+        ============  ========================================================
+        *xs*, *ys*    Positions of data points.
+        *zs*          Either an array of the same length as *xs* and
+                      *ys* or a single value to place all points in
+                      the same plane. Default is 0.
+        *zdir*        Which direction to use as z ('x', 'y' or 'z')
+                      when plotting a 2D set.
+        *s*           Size in points^2.  It is a scalar or an array of the
+                      same length as *x* and *y*.
 
-        *c*         a color. *c* can be a single color format string, or a
-                    sequence of color specifications of length *N*, or a
-                    sequence of *N* numbers to be mapped to colors using the
-                    *cmap* and *norm* specified via kwargs (see below). Note
-                    that *c* should not be a single numeric RGB or RGBA
-                    sequence because that is indistinguishable from an array
-                    of values to be colormapped.  *c* can be a 2-D array in
-                    which the rows are RGB or RGBA, however.
-        ==========  ==========================================================
+        *c*           A color. *c* can be a single color format string, or a
+                      sequence of color specifications of length *N*, or a
+                      sequence of *N* numbers to be mapped to colors using the
+                      *cmap* and *norm* specified via kwargs (see below). Note
+                      that *c* should not be a single numeric RGB or RGBA
+                      sequence because that is indistinguishable from an array
+                      of values to be colormapped.  *c* can be a 2-D array in
+                      which the rows are RGB or RGBA, however.
+
+        *depthshade*
+                      Whether or not to shade the scatter markers to give
+                      the appearance of depth. Default is *True*.
+        ============  ========================================================
 
         Keyword arguments are passed on to
         :func:`~matplotlib.axes.Axes.scatter`.
@@ -2183,7 +2276,8 @@ class Axes3D(Axes):
             zs = np.ones(len(xs)) * zs
         else:
             is_2d = False
-        art3d.patch_collection_2d_to_3d(patches, zs=zs, zdir=zdir)
+        art3d.patch_collection_2d_to_3d(patches, zs=zs, zdir=zdir,
+                                        depthshade=depthshade)
 
         if self._zmargin < 0.05 and xs.size > 0:
             self.set_zmargin(0.05)
@@ -2237,7 +2331,7 @@ class Axes3D(Axes):
             # the following has to be skipped if verts is empty
             # NOTE: Bugs could still occur if len(verts) > 0,
             #       but the "2nd dimension" is empty.
-            xs, ys = zip(*verts)
+            xs, ys = list(zip(*verts))
         else :
             xs, ys = [], []
 
@@ -2360,6 +2454,8 @@ class Axes3D(Axes):
 
         self.auto_scale_xyz((minx, maxx), (miny, maxy), (minz, maxz), had_data)
 
+        return col
+
     def set_title(self, label, fontdict=None, loc='center', **kwargs):
         ret = Axes.set_title(self, label, fontdict=fontdict, loc=loc, **kwargs)
         (x, y) = self.title.get_position()
@@ -2367,6 +2463,170 @@ class Axes3D(Axes):
         return ret
     set_title.__doc__ = maxes.Axes.set_title.__doc__
 
+    def quiver(self, *args, **kwargs):
+        """
+        Plot a 3D field of arrows.
+
+        call signatures::
+
+            quiver(X, Y, Z, U, V, W, **kwargs)
+
+        Arguments:
+
+            *X*, *Y*, *Z*:
+                The x, y and z coordinates of the arrow locations (default is
+                tip of arrow; see *pivot* kwarg)
+
+            *U*, *V*, *W*:
+                The x, y and z components of the arrow vectors
+
+        The arguments could be array-like or scalars, so long as they
+        they can be broadcast together. The arguments can also be
+        masked arrays. If an element in any of argument is masked, then
+        that corresponding quiver element will not be plotted.
+
+        Keyword arguments:
+
+            *length*: [1.0 | float]
+                The length of each quiver, default to 1.0, the unit is
+                the same with the axes
+
+            *arrow_length_ratio*: [0.3 | float]
+                The ratio of the arrow head with respect to the quiver,
+                default to 0.3
+
+            *pivot*: [ 'tail' | 'middle' | 'tip' ]
+                The part of the arrow that is at the grid point; the arrow
+                rotates about this point, hence the name *pivot*.
+
+        Any additional keyword arguments are delegated to
+        :class:`~matplotlib.collections.LineCollection`
+
+        """
+        def calc_arrow(uvw, angle=15):
+            """
+            To calculate the arrow head. uvw should be a unit vector.
+            """
+            # get unit direction vector perpendicular to (u,v,w)
+            norm = np.linalg.norm(uvw[:2])
+            if norm > 0:
+                x = uvw[1] / norm
+                y = -uvw[0] / norm
+            else:
+                x, y = 0, 1
+
+            # compute the two arrowhead direction unit vectors
+            ra = math.radians(angle)
+            c = math.cos(ra)
+            s = math.sin(ra)
+
+            # construct the rotation matrices
+            Rpos = np.array([[c+(x**2)*(1-c), x*y*(1-c), y*s],
+                             [y*x*(1-c), c+(y**2)*(1-c), -x*s],
+                             [-y*s, x*s, c]])
+            # opposite rotation negates everything but the diagonal
+            Rneg = Rpos * (np.eye(3)*2 - 1)
+
+            # multiply them to get the rotated vector
+            return Rpos.dot(uvw), Rneg.dot(uvw)
+
+        had_data = self.has_data()
+
+        # handle kwargs
+        # shaft length
+        length = kwargs.pop('length', 1)
+        # arrow length ratio to the shaft length
+        arrow_length_ratio = kwargs.pop('arrow_length_ratio', 0.3)
+        # pivot point
+        pivot = kwargs.pop('pivot', 'tip')
+
+        # handle args
+        argi = 6
+        if len(args) < argi:
+            ValueError('Wrong number of arguments. Expected %d got %d' %
+                       (argi, len(args)))
+
+        # first 6 arguments are X, Y, Z, U, V, W
+        input_args = args[:argi]
+        # if any of the args are scalar, convert into list
+        input_args = [[k] if isinstance(k, (int, float)) else k
+                      for k in input_args]
+
+        # extract the masks, if any
+        masks = [k.mask for k in input_args if isinstance(k, np.ma.MaskedArray)]
+        # broadcast to match the shape
+        bcast = np.broadcast_arrays(*(input_args + masks))
+        input_args = bcast[:argi]
+        masks = bcast[argi:]
+        if masks:
+            # combine the masks into one
+            mask = reduce(np.logical_or, masks)
+            # put mask on and compress
+            input_args = [np.ma.array(k, mask=mask).compressed()
+                          for k in input_args]
+        else:
+            input_args = [k.flatten() for k in input_args]
+
+        if any(len(v) == 0 for v in input_args):
+            # No quivers, so just make an empty collection and return early
+            linec = art3d.Line3DCollection([], *args[argi:], **kwargs)
+            self.add_collection(linec)
+            return linec
+
+        # Following assertions must be true before proceeding
+        # must all be ndarray
+        assert all(isinstance(k, np.ndarray) for k in input_args)
+        # must all in same shape
+        assert len(set([k.shape for k in input_args])) == 1
+
+        shaft_dt = np.linspace(0, length, num=2)
+        arrow_dt = shaft_dt * arrow_length_ratio
+
+        if pivot == 'tail':
+            shaft_dt -= length
+        elif pivot == 'middle':
+            shaft_dt -= length/2.
+        elif pivot != 'tip':
+            raise ValueError('Invalid pivot argument: ' + str(pivot))
+
+        XYZ = np.column_stack(input_args[:3])
+        UVW = np.column_stack(input_args[3:argi]).astype(float)
+
+        # Normalize rows of UVW
+        # Note: with numpy 1.9+, could use np.linalg.norm(UVW, axis=1)
+        norm = np.sqrt(np.sum(UVW**2, axis=1))
+
+        # If any row of UVW is all zeros, don't make a quiver for it
+        mask = norm > 1e-10
+        XYZ = XYZ[mask]
+        UVW = UVW[mask] / norm[mask].reshape((-1, 1))
+
+        if len(XYZ) > 0:
+            # compute the shaft lines all at once with an outer product
+            shafts = (XYZ - np.multiply.outer(shaft_dt, UVW)).swapaxes(0, 1)
+            # compute head direction vectors, n heads by 2 sides by 3 dimensions
+            head_dirs = np.array([calc_arrow(d) for d in UVW])
+            # compute all head lines at once, starting from where the shaft ends
+            heads = shafts[:, :1] - np.multiply.outer(arrow_dt, head_dirs)
+            # stack left and right head lines together
+            heads.shape = (len(arrow_dt), -1, 3)
+            # transpose to get a list of lines
+            heads = heads.swapaxes(0, 1)
+
+            lines = list(shafts) + list(heads)
+        else:
+            lines = []
+
+        linec = art3d.Line3DCollection(lines, *args[argi:], **kwargs)
+        self.add_collection(linec)
+
+        self.auto_scale_xyz(XYZ[:, 0], XYZ[:, 1], XYZ[:, 2], had_data)
+
+        return linec
+
+    quiver3D = quiver
+
+
 def get_test_data(delta=0.05):
     '''
     Return a tuple X, Y, Z with a test data set.
