@@ -3,12 +3,12 @@
 #ifndef __PATH_CONVERTERS_H__
 #define __PATH_CONVERTERS_H__
 
-#include "CXX/Objects.hxx"
-#include "numpy/arrayobject.h"
+#include <cmath>
+#include <stdint.h>
 #include "agg_path_storage.h"
 #include "agg_clip_liang_barsky.h"
-#include "MPL_isnan.h"
 #include "mplutils.h"
+#include "agg_conv_segmentator.h"
 
 /*
  This file contains a number of vertex converters that modify
@@ -45,12 +45,11 @@
  output.  It is designed to be as fast as possible vs. the STL's queue
  which is more flexible.
  */
-template<int QueueSize>
+template <int QueueSize>
 class EmbeddedQueue
 {
-protected:
-    EmbeddedQueue() :
-        m_queue_read(0), m_queue_write(0)
+  protected:
+    EmbeddedQueue() : m_queue_read(0), m_queue_write(0)
     {
         // empty
     }
@@ -59,11 +58,9 @@ protected:
     {
         item()
         {
-
         }
 
-        inline void
-        set(const unsigned cmd_, const double& x_, const double& y_)
+        inline void set(const unsigned cmd_, const double x_, const double y_)
         {
             cmd = cmd_;
             x = x_;
@@ -73,28 +70,24 @@ protected:
         double x;
         double y;
     };
-    int  m_queue_read;
-    int  m_queue_write;
+    int m_queue_read;
+    int m_queue_write;
     item m_queue[QueueSize];
 
-    inline void
-    queue_push(const unsigned cmd, const double& x, const double& y)
+    inline void queue_push(const unsigned cmd, const double x, const double y)
     {
         m_queue[m_queue_write++].set(cmd, x, y);
     }
 
-    inline bool
-    queue_nonempty()
+    inline bool queue_nonempty()
     {
         return m_queue_read < m_queue_write;
     }
 
-    inline bool
-    queue_pop(unsigned *cmd, double *x, double *y)
+    inline bool queue_pop(unsigned *cmd, double *x, double *y)
     {
-        if (queue_nonempty())
-        {
-            const item& front = m_queue[m_queue_read++];
+        if (queue_nonempty()) {
+            const item &front = m_queue[m_queue_read++];
             *cmd = front.cmd;
             *x = front.x;
             *y = front.y;
@@ -108,97 +101,129 @@ protected:
         return false;
     }
 
-    inline void
-    queue_clear()
+    inline void queue_clear()
     {
         m_queue_read = 0;
         m_queue_write = 0;
     }
 };
 
+/* Defines when path segment types have more than one vertex */
+static const size_t num_extra_points_map[] =
+    {0, 0, 0, 1,
+     2, 0, 0, 0,
+     0, 0, 0, 0,
+     0, 0, 0, 0
+    };
+
+/* An implementation of a simple linear congruential random number
+   generator.  This is a "classic" and fast RNG which works fine for
+   our purposes of sketching lines, but should not be used for things
+   that matter, like crypto.  We are implementing this ourselves
+   rather than using the C stdlib so that the seed state is not shared
+   with other third-party code. There are recent C++ options, but we
+   still require nothing later than C++98 for compatibility
+   reasons. */
+class RandomNumberGenerator
+{
+private:
+    /* These are the same constants from MS Visual C++, which
+       has the nice property that the modulus is 2^32, thus
+       saving an explicit modulo operation
+    */
+    static const uint32_t a = 214013;
+    static const uint32_t c = 2531011;
+    uint32_t m_seed;
+
+public:
+    RandomNumberGenerator() : m_seed(0) {}
+    RandomNumberGenerator(int seed) : m_seed(seed) {}
+
+    void seed(int seed)
+    {
+        m_seed = seed;
+    }
+
+    double get_double()
+    {
+        m_seed = (a * m_seed + c);
+        return (double)m_seed / (double)(1LL << 32);
+    }
+};
+
 /*
   PathNanRemover is a vertex converter that removes non-finite values
   from the vertices list, and inserts MOVETO commands as necessary to
   skip over them.  If a curve segment contains at least one non-finite
   value, the entire curve segment will be skipped.
  */
-template<class VertexSource>
+template <class VertexSource>
 class PathNanRemover : protected EmbeddedQueue<4>
 {
-    VertexSource* m_source;
+    VertexSource *m_source;
     bool m_remove_nans;
     bool m_has_curves;
-    static const unsigned char num_extra_points_map[16];
 
-public:
+  public:
     /* has_curves should be true if the path contains bezier curve
        segments, as this requires a slower algorithm to remove the
        NaNs.  When in doubt, set to true.
      */
-    PathNanRemover(VertexSource& source, bool remove_nans, bool has_curves) :
-        m_source(&source), m_remove_nans(remove_nans), m_has_curves(has_curves)
+    PathNanRemover(VertexSource &source, bool remove_nans, bool has_curves)
+        : m_source(&source), m_remove_nans(remove_nans), m_has_curves(has_curves)
     {
         // empty
     }
 
-    inline void
-    rewind(unsigned path_id)
+    inline void rewind(unsigned path_id)
     {
         queue_clear();
         m_source->rewind(path_id);
     }
 
-    inline unsigned
-    vertex(double* x, double *y)
+    inline unsigned vertex(double *x, double *y)
     {
         unsigned code;
 
-        if (!m_remove_nans)
-        {
+        if (!m_remove_nans) {
             return m_source->vertex(x, y);
         }
 
-        if (m_has_curves)
-        {
+        if (m_has_curves) {
             /* This is the slow method for when there might be curves. */
-            if (queue_pop(&code, x, y))
-            {
+            if (queue_pop(&code, x, y)) {
                 return code;
             }
 
             bool needs_move_to = false;
-            while (true)
-            {
+            while (true) {
                 /* The approach here is to push each full curve
                    segment into the queue.  If any non-finite values
                    are found along the way, the queue is emptied, and
                    the next curve segment is handled. */
                 code = m_source->vertex(x, y);
                 if (code == agg::path_cmd_stop ||
-                        code == (agg::path_cmd_end_poly | agg::path_flags_close))
-                {
+                    code == (agg::path_cmd_end_poly | agg::path_flags_close)) {
                     return code;
                 }
 
-                if (needs_move_to)
-                {
+                if (needs_move_to) {
                     queue_push(agg::path_cmd_move_to, *x, *y);
                 }
 
                 size_t num_extra_points = num_extra_points_map[code & 0xF];
-                bool has_nan = (MPL_notisfinite64(*x) || MPL_notisfinite64(*y));
+                bool has_nan = (!(std::isfinite(*x) && std::isfinite(*y)));
                 queue_push(code, *x, *y);
+
                 /* Note: this test can not be short-circuited, since we need to
                    advance through the entire curve no matter what */
-                for (size_t i = 0; i < num_extra_points; ++i)
-                {
+                for (size_t i = 0; i < num_extra_points; ++i) {
                     m_source->vertex(x, y);
-                    has_nan |= (MPL_notisfinite64(*x) || MPL_notisfinite64(*y));
+                    has_nan = has_nan || !(std::isfinite(*x) && std::isfinite(*y));
                     queue_push(code, *x, *y);
                 }
 
-                if (!has_nan)
-                {
+                if (!has_nan) {
                     break;
                 }
 
@@ -207,49 +232,37 @@ public:
                 /* If the last point is finite, we use that for the
                    moveto, otherwise, we'll use the first vertex of
                    the next curve. */
-                if (!(MPL_notisfinite64(*x) || MPL_notisfinite64(*y)))
-                {
+                if (std::isfinite(*x) && std::isfinite(*y)) {
                     queue_push(agg::path_cmd_move_to, *x, *y);
                     needs_move_to = false;
-                }
-                else
-                {
+                } else {
                     needs_move_to = true;
                 }
             }
 
-            if (queue_pop(&code, x, y))
-            {
+            if (queue_pop(&code, x, y)) {
                 return code;
-            }
-            else
-            {
+            } else {
                 return agg::path_cmd_stop;
             }
-        }
-        else // !m_has_curves
+        } else // !m_has_curves
         {
             /* This is the fast path for when we know we have no curves */
             code = m_source->vertex(x, y);
 
             if (code == agg::path_cmd_stop ||
-                code == (agg::path_cmd_end_poly | agg::path_flags_close))
-            {
+                code == (agg::path_cmd_end_poly | agg::path_flags_close)) {
                 return code;
             }
 
-            if (MPL_notisfinite64(*x) || MPL_notisfinite64(*y))
-            {
-                do
-                {
+            if (!(std::isfinite(*x) && std::isfinite(*y))) {
+                do {
                     code = m_source->vertex(x, y);
                     if (code == agg::path_cmd_stop ||
-                        code == (agg::path_cmd_end_poly | agg::path_flags_close))
-                    {
+                        code == (agg::path_cmd_end_poly | agg::path_flags_close)) {
                         return code;
                     }
-                }
-                while (MPL_notisfinite64(*x) || MPL_notisfinite64(*y));
+                } while (!(std::isfinite(*x) && std::isfinite(*y)));
                 return agg::path_cmd_move_to;
             }
 
@@ -258,53 +271,53 @@ public:
     }
 };
 
-/* Defines when path segment types have more than one vertex */
-template<class VertexSource>
-const unsigned char PathNanRemover<VertexSource>::num_extra_points_map[] =
-    {0, 0, 0, 1,
-     2, 0, 0, 0,
-     0, 0, 0, 0,
-     0, 0, 0, 0
-    };
-
 /************************************************************
  PathClipper uses the Liang-Barsky line clipping algorithm (as
  implemented in Agg) to clip the path to a given rectangle.  Lines
  will never extend outside of the rectangle.  Curve segments are not
  clipped, but are always included in their entirety.
  */
-template<class VertexSource>
+template <class VertexSource>
 class PathClipper
 {
-    VertexSource*          m_source;
-    bool                   m_do_clipping;
+    VertexSource *m_source;
+    bool m_do_clipping;
     agg::rect_base<double> m_cliprect;
-    double                 m_lastX;
-    double                 m_lastY;
-    bool                   m_moveto;
-    double                 m_nextX;
-    double                 m_nextY;
-    bool                   m_has_next;
-    double                 m_initX;
-    double                 m_initY;
-    bool                   m_has_init;
-    bool                   m_broke_path;
+    double m_lastX;
+    double m_lastY;
+    bool m_moveto;
+    double m_nextX;
+    double m_nextY;
+    bool m_has_next;
+    bool m_end_poly;
+    double m_initX;
+    double m_initY;
+    bool m_has_init;
+    bool m_broke_path;
 
-public:
-    PathClipper(VertexSource& source, bool do_clipping,
-                double width, double height) :
-        m_source(&source), m_do_clipping(do_clipping),
-        m_cliprect(-1.0, -1.0, width + 1.0, height + 1.0), m_moveto(true),
-        m_has_next(false), m_has_init(false), m_broke_path(false)
+  public:
+    PathClipper(VertexSource &source, bool do_clipping, double width, double height)
+        : m_source(&source),
+          m_do_clipping(do_clipping),
+          m_cliprect(-1.0, -1.0, width + 1.0, height + 1.0),
+          m_moveto(true),
+          m_has_next(false),
+          m_end_poly(false),
+          m_has_init(false),
+          m_broke_path(false)
     {
         // empty
     }
 
-    PathClipper(VertexSource& source, bool do_clipping,
-                const agg::rect_base<double>& rect) :
-        m_source(&source), m_do_clipping(do_clipping),
-        m_cliprect(rect), m_moveto(true), m_has_next(false),
-        m_has_init(false), m_broke_path(false)
+    PathClipper(VertexSource &source, bool do_clipping, const agg::rect_base<double> &rect)
+        : m_source(&source),
+          m_do_clipping(do_clipping),
+          m_cliprect(rect),
+          m_moveto(true),
+          m_has_next(false),
+          m_end_poly(false),
+          m_has_init(false),
+          m_broke_path(false)
     {
         m_cliprect.x1 -= 1.0;
         m_cliprect.y1 -= 1.0;
@@ -312,48 +325,55 @@ public:
         m_cliprect.y2 += 1.0;
     }
 
-    inline void
-    rewind(unsigned path_id)
+    inline void rewind(unsigned path_id)
     {
         m_has_next = false;
         m_moveto = true;
         m_source->rewind(path_id);
     }
 
-    unsigned
-    vertex(double* x, double* y)
+    unsigned vertex(double *x, double *y)
     {
         unsigned code;
 
-        if (m_do_clipping)
-        {
+        if (m_do_clipping) {
             /* This is the slow path where we actually do clipping */
 
-            if (m_has_next)
-            {
+            if (m_end_poly) {
+                m_end_poly = false;
+                return (agg::path_cmd_end_poly | agg::path_flags_close);
+            }
+
+            if (m_has_next) {
                 m_has_next = false;
                 *x = m_nextX;
                 *y = m_nextY;
                 return agg::path_cmd_line_to;
             }
 
-            while ((code = m_source->vertex(x, y)) != agg::path_cmd_stop)
-            {
-                if (code == agg::path_cmd_move_to)
-                {
+            while ((code = m_source->vertex(x, y)) != agg::path_cmd_stop) {
+                if (code == (agg::path_cmd_end_poly | agg::path_flags_close)) {
+                    if (m_has_init) {
+                        *x = m_initX;
+                        *y = m_initY;
+                        code = agg::path_cmd_line_to;
+                        m_end_poly = true;
+                    } else {
+                        continue;
+                    }
+                }
+
+                if (code == agg::path_cmd_move_to) {
                     m_initX = *x;
                     m_initY = *y;
                     m_has_init = true;
                     m_moveto = true;
                 }
-                if (m_moveto)
-                {
+                if (m_moveto) {
                     m_moveto = false;
                     code = agg::path_cmd_move_to;
                     break;
-                }
-                else if (code == agg::path_cmd_line_to)
-                {
+                } else if (code == agg::path_cmd_line_to) {
                     double x0, y0, x1, y1;
                     x0 = m_lastX;
                     y0 = m_lastY;
@@ -365,10 +385,8 @@ public:
                     // moved >= 4 - Fully clipped
                     // moved & 1 != 0 - First point has been moved
                     // moved & 2 != 0 - Second point has been moved
-                    if (moved < 4)
-                    {
-                        if (moved & 1)
-                        {
+                    if (moved < 4) {
+                        if (moved & 1) {
                             *x = x0;
                             *y = y0;
                             m_nextX = x1;
@@ -381,16 +399,7 @@ public:
                         *y = y1;
                         return code;
                     }
-                }
-                else if (code == (agg::path_cmd_end_poly | agg::path_flags_close)
-                         && m_broke_path && m_has_init)
-                {
-                    *x = m_initX;
-                    *y = m_initY;
-                    return agg::path_cmd_line_to;
-                }
-                else
-                {
+                } else {
                     break;
                 }
             }
@@ -398,9 +407,7 @@ public:
             m_lastX = *x;
             m_lastY = *y;
             return code;
-        }
-        else
-        {
+        } else {
             // If not doing any clipping, just pass along the vertices
             // verbatim
             return m_source->vertex(x, y);
@@ -413,55 +420,45 @@ public:
  makes rectilinear paths (rectangles, horizontal and vertical lines
  etc.) look much cleaner.
 */
-enum e_snap_mode
-{
+enum e_snap_mode {
     SNAP_AUTO,
     SNAP_FALSE,
     SNAP_TRUE
 };
 
-template<class VertexSource>
+template <class VertexSource>
 class PathSnapper
 {
-private:
-    VertexSource* m_source;
-    bool          m_snap;
-    double        m_snap_value;
+  private:
+    VertexSource *m_source;
+    bool m_snap;
+    double m_snap_value;
 
-    static bool
-    should_snap(VertexSource& path,
-                e_snap_mode snap_mode,
-                unsigned total_vertices)
+    static bool should_snap(VertexSource &path, e_snap_mode snap_mode, unsigned total_vertices)
     {
         // If this contains only straight horizontal or vertical lines, it should be
         // snapped to the nearest pixels
-        double x0, y0, x1, y1;
+        double x0 = 0, y0 = 0, x1 = 0, y1 = 0;
         unsigned code;
 
-        switch (snap_mode)
-        {
+        switch (snap_mode) {
         case SNAP_AUTO:
-            if (total_vertices > 1024)
-            {
+            if (total_vertices > 1024) {
                 return false;
             }
 
             code = path.vertex(&x0, &y0);
-            if (code == agg::path_cmd_stop)
-            {
+            if (code == agg::path_cmd_stop) {
                 return false;
             }
 
-            while ((code = path.vertex(&x1, &y1)) != agg::path_cmd_stop)
-            {
-                switch (code)
-                {
+            while ((code = path.vertex(&x1, &y1)) != agg::path_cmd_stop) {
+                switch (code) {
                 case agg::path_cmd_curve3:
                 case agg::path_cmd_curve4:
                     return false;
                 case agg::path_cmd_line_to:
-                    if (!(fabs(x0 - x1) < 1e-4 || fabs(y0 - y1) < 1e-4))
-                    {
+                    if (!(fabs(x0 - x1) < 1e-4 || fabs(y0 - y1) < 1e-4)) {
                         return false;
                     }
                 }
@@ -479,21 +476,22 @@ private:
         return false;
     }
 
-public:
+  public:
     /*
       snap_mode should be one of:
         - SNAP_AUTO: Examine the path to determine if it should be snapped
         - SNAP_TRUE: Force snapping
         - SNAP_FALSE: No snapping
     */
-    PathSnapper(VertexSource& source, e_snap_mode snap_mode,
-                unsigned total_vertices = 15, double stroke_width = 0.0) :
-        m_source(&source)
+    PathSnapper(VertexSource &source,
+                e_snap_mode snap_mode,
+                unsigned total_vertices = 15,
+                double stroke_width = 0.0)
+        : m_source(&source)
     {
         m_snap = should_snap(source, snap_mode, total_vertices);
 
-        if (m_snap)
-        {
+        if (m_snap) {
             int is_odd = (int)mpl_round(stroke_width) % 2;
             m_snap_value = (is_odd) ? 0.5 : 0.0;
         }
@@ -501,27 +499,23 @@ public:
         source.rewind(0);
     }
 
-    inline void
-    rewind(unsigned path_id)
+    inline void rewind(unsigned path_id)
     {
         m_source->rewind(path_id);
     }
 
-    inline unsigned
-    vertex(double* x, double* y)
+    inline unsigned vertex(double *x, double *y)
     {
         unsigned code;
         code = m_source->vertex(x, y);
-        if (m_snap && agg::is_vertex(code))
-        {
+        if (m_snap && agg::is_vertex(code)) {
             *x = floor(*x + 0.5) + m_snap_value;
             *y = floor(*y + 0.5) + m_snap_value;
         }
         return code;
     }
 
-    inline bool
-    is_snapping()
+    inline bool is_snapping()
     {
         return m_snap;
     }
@@ -531,41 +525,47 @@ public:
  PathSimplifier reduces the number of vertices in a dense path without
  changing its appearance.
 */
-template<class VertexSource>
+template <class VertexSource>
 class PathSimplifier : protected EmbeddedQueue<9>
 {
-public:
+  public:
     /* Set simplify to true to perform simplification */
-    PathSimplifier(VertexSource& source, bool do_simplify, double simplify_threshold) :
-        m_source(&source), m_simplify(do_simplify),
-        m_simplify_threshold(simplify_threshold*simplify_threshold),
-        m_moveto(true), m_after_moveto(false),
-        m_lastx(0.0), m_lasty(0.0), m_clipped(false),
-        m_origdx(0.0), m_origdy(0.0),
-        m_origdNorm2(0.0), m_dnorm2Max(0.0),
-        m_lastMax(false), m_nextX(0.0), m_nextY(0.0),
-        m_lastWrittenX(0.0), m_lastWrittenY(0.0)
+    PathSimplifier(VertexSource &source, bool do_simplify, double simplify_threshold)
+        : m_source(&source),
+          m_simplify(do_simplify),
+          m_simplify_threshold(simplify_threshold * simplify_threshold),
+          m_moveto(true),
+          m_after_moveto(false),
+          m_lastx(0.0),
+          m_lasty(0.0),
+          m_clipped(false),
+          m_origdx(0.0),
+          m_origdy(0.0),
+          m_origdNorm2(0.0),
+          m_dnorm2Max(0.0),
+          m_lastMax(false),
+          m_nextX(0.0),
+          m_nextY(0.0),
+          m_lastWrittenX(0.0),
+          m_lastWrittenY(0.0)
     {
         // empty
     }
 
-    inline void
-    rewind(unsigned path_id)
+    inline void rewind(unsigned path_id)
     {
         queue_clear();
         m_moveto = true;
         m_source->rewind(path_id);
     }
 
-    unsigned
-    vertex(double* x, double* y)
+    unsigned vertex(double *x, double *y)
     {
         unsigned cmd;
 
         /* The simplification algorithm doesn't support curves or compound paths
            so we just don't do it at all in that case... */
-        if (!m_simplify)
-        {
+        if (!m_simplify) {
             return m_source->vertex(x, y);
         }
 
@@ -589,8 +589,7 @@ public:
            the queue before proceeding to the main loop below.
            -- Michael Droettboom */
 
-        if (queue_pop(&cmd, x, y))
-        {
+        if (queue_pop(&cmd, x, y)) {
             return cmd;
         }
 
@@ -599,19 +598,16 @@ public:
            to the outbound queue, not to run through the entire path
            in one go.  This eliminates the need to allocate and fill
            an entire additional path array on each draw. */
-        while ((cmd = m_source->vertex(x, y)) != agg::path_cmd_stop)
-        {
+        while ((cmd = m_source->vertex(x, y)) != agg::path_cmd_stop) {
             /* if we are starting a new path segment, move to the first point
                + init */
 
-            if (m_moveto || cmd == agg::path_cmd_move_to)
-            {
+            if (m_moveto || cmd == agg::path_cmd_move_to) {
                 /* m_moveto check is not generally needed because
                    m_source generates an initial moveto; but it is
                    retained for safety in case circumstances arise
                    where this is not true. */
-                if (m_origdNorm2 != 0.0 && !m_after_moveto)
-                {
+                if (m_origdNorm2 != 0.0 && !m_after_moveto) {
                     /* m_origdNorm2 is nonzero only if we have a
                        vector; the m_after_moveto check ensures we
                        push this vector to the queue only once. */
@@ -623,8 +619,7 @@ public:
                 m_moveto = false;
                 m_origdNorm2 = 0.0;
                 m_clipped = true;
-                if (queue_nonempty())
-                {
+                if (queue_nonempty()) {
                     /* If we did a push, empty the queue now. */
                     break;
                 }
@@ -645,10 +640,8 @@ public:
             /* if we have no orig vector, set it to this vector and
                continue.  this orig vector is the reference vector we
                will build up the line to */
-            if (m_origdNorm2 == 0.0)
-            {
-                if (m_clipped)
-                {
+            if (m_origdNorm2 == 0.0) {
+                if (m_clipped) {
                     queue_push(agg::path_cmd_move_to, m_lastx, m_lasty);
                     m_clipped = false;
                 }
@@ -657,7 +650,7 @@ public:
                 m_origdy = *y - m_lasty;
                 m_origdNorm2 = m_origdx * m_origdx + m_origdy * m_origdy;
 
-                //set all the variables to reflect this new orig vector
+                // set all the variables to reflect this new orig vector
                 m_dnorm2Max = m_origdNorm2;
                 m_lastMax = true;
 
@@ -693,8 +686,7 @@ public:
 
             /* If the perp vector is less than some number of (squared)
                pixels in size, then merge the current vector */
-            if (perpdNorm2 < m_simplify_threshold)
-            {
+            if (perpdNorm2 < m_simplify_threshold) {
                 /* check if the current vector is parallel or
                    anti-parallel to the orig vector. If it is
                    parallel, test if it is the longest of the vectors
@@ -702,18 +694,14 @@ public:
                 double paradNorm2 = paradx * paradx + parady * parady;
 
                 m_lastMax = false;
-                if (totdot > 0.0)
-                {
-                    if (paradNorm2 > m_dnorm2Max)
-                    {
+                if (totdot > 0.0) {
+                    if (paradNorm2 > m_dnorm2Max) {
                         m_lastMax = true;
                         m_dnorm2Max = paradNorm2;
                         m_nextX = *x;
                         m_nextY = *y;
                     }
-                }
-                else
-                {
+                } else {
                     _push(&m_lastx, &m_lasty);
                     _push(x, y);
                     break;
@@ -738,67 +726,59 @@ public:
 
         /* Fill the queue with the remaining vertices if we've finished the
            path in the above loop. */
-        if (cmd == agg::path_cmd_stop)
-        {
-            if (m_origdNorm2 != 0.0)
-            {
-                queue_push((m_moveto || m_after_moveto) ?
-                           agg::path_cmd_move_to : agg::path_cmd_line_to,
-                           m_nextX, m_nextY);
+        if (cmd == agg::path_cmd_stop) {
+            if (m_origdNorm2 != 0.0) {
+                queue_push((m_moveto || m_after_moveto) ? agg::path_cmd_move_to
+                                                        : agg::path_cmd_line_to,
+                           m_nextX,
+                           m_nextY);
                 m_moveto = false;
             }
-            queue_push((m_moveto || m_after_moveto) ?
-                       agg::path_cmd_move_to : agg::path_cmd_line_to,
-                       m_lastx, m_lasty);
+            queue_push((m_moveto || m_after_moveto) ? agg::path_cmd_move_to : agg::path_cmd_line_to,
+                       m_lastx,
+                       m_lasty);
             m_moveto = false;
             queue_push(agg::path_cmd_stop, 0.0, 0.0);
         }
 
         /* Return the first item in the queue, if any, otherwise
            indicate that we're done. */
-        if (queue_pop(&cmd, x, y))
-        {
+        if (queue_pop(&cmd, x, y)) {
             return cmd;
-        }
-        else
-        {
+        } else {
             return agg::path_cmd_stop;
         }
     }
 
-private:
-    VertexSource* m_source;
-    bool          m_simplify;
-    double        m_simplify_threshold;
+  private:
+    VertexSource *m_source;
+    bool m_simplify;
+    double m_simplify_threshold;
 
-    bool   m_moveto;
-    bool   m_after_moveto;
+    bool m_moveto;
+    bool m_after_moveto;
     double m_lastx, m_lasty;
-    bool   m_clipped;
+    bool m_clipped;
 
     double m_origdx;
     double m_origdy;
     double m_origdNorm2;
     double m_dnorm2Max;
-    bool   m_lastMax;
+    bool m_lastMax;
     double m_nextX;
     double m_nextY;
     double m_lastWrittenX;
     double m_lastWrittenY;
 
-    inline void
-    _push(double* x, double* y)
+    inline void _push(double *x, double *y)
     {
         queue_push(agg::path_cmd_line_to, m_nextX, m_nextY);
 
         /* If we clipped some segments between this line and the next line
            we are starting, we also need to move to the last point. */
-        if (m_clipped)
-        {
+        if (m_clipped) {
             queue_push(agg::path_cmd_move_to, m_lastx, m_lasty);
-        }
-        else if (!m_lastMax)
-        {
+        } else if (!m_lastMax) {
             /* If the last line was not the longest line, then move
                back to the end point of the last line in the
                sequence. Only do this if not clipped, since in that
@@ -815,8 +795,8 @@ private:
 
         m_dnorm2Max = m_origdNorm2;
         m_lastMax = true;
-        m_lastWrittenX = m_queue[m_queue_write-1].x;
-        m_lastWrittenY = m_queue[m_queue_write-1].y;
+        m_lastWrittenX = m_queue[m_queue_write - 1].x;
+        m_lastWrittenY = m_queue[m_queue_write - 1].y;
         m_lastx = m_nextX = *x;
         m_lasty = m_nextY = *y;
 
@@ -824,4 +804,98 @@ private:
     }
 };
 
+template <class VertexSource>
+class Sketch
+{
+  public:
+    /*
+       scale: the scale of the wiggle perpendicular to the original
+       line (in pixels)
+
+       length: the base wavelength of the wiggle along the
+       original line (in pixels)
+
+       randomness: the factor that the sketch length will randomly
+       shrink and expand.
+    */
+    Sketch(VertexSource &source, double scale, double length, double randomness)
+        : m_source(&source),
+          m_scale(scale),
+          m_length(length),
+          m_randomness(randomness),
+          m_segmented(source),
+          m_last_x(0.0),
+          m_last_y(0.0),
+          m_has_last(false),
+          m_p(0.0),
+          m_rand(0)
+    {
+        rewind(0);
+    }
+
+    unsigned vertex(double *x, double *y)
+    {
+        if (m_scale == 0.0) {
+            return m_source->vertex(x, y);
+        }
+
+        unsigned code = m_segmented.vertex(x, y);
+
+        if (code == agg::path_cmd_move_to) {
+            m_has_last = false;
+            m_p = 0.0;
+        }
+
+        if (m_has_last) {
+            // We want the "cursor" along the sine wave to move at a
+            // random rate.
+            double d_rand = m_rand.get_double();
+            double d_M_PI = 3.14159265358979323846;
+            m_p += pow(m_randomness, d_rand * 2.0 - 1.0);
+            double r = sin(m_p / (m_length / (d_M_PI * 2.0))) * m_scale;
+            double den = m_last_x - *x;
+            double num = m_last_y - *y;
+            double len = num * num + den * den;
+            m_last_x = *x;
+            m_last_y = *y;
+            if (len != 0) {
+                len = sqrt(len);
+                *x += r * num / len;
+                *y += r * -den / len;
+            }
+        } else {
+            m_last_x = *x;
+            m_last_y = *y;
+        }
+
+        m_has_last = true;
+
+        return code;
+    }
+
+    inline void rewind(unsigned path_id)
+    {
+        m_has_last = false;
+        m_p = 0.0;
+        if (m_scale != 0.0) {
+            m_rand.seed(0);
+            m_segmented.rewind(path_id);
+        } else {
+            m_source->rewind(path_id);
+        }
+    }
+
+  private:
+    VertexSource *m_source;
+    double m_scale;
+    double m_length;
+    double m_randomness;
+    agg::conv_segmentator<VertexSource> m_segmented;
+    double m_last_x;
+    double m_last_y;
+    bool m_has_last;
+    double m_p;
+    RandomNumberGenerator m_rand;
+};
+
 #endif // __PATH_CONVERTERS_H__
